#
#  AppController.rb
#

$version = "aes"
bundle = "#{File.dirname(__FILE__)}/seven_to_x.bundle"
if File.exist?(bundle)
	require 'seven_to_x.bundle'
else
	p "NO BUNDLE #{bundle} !!!\n"
end

class AppController < OSX::NSObject
	ib_outlets :deactivateMenuItem, :open, :quit
	ib_outlets :mainWindow, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton
	ib_outlets :progressPanel, :progressView, :progressIndicator, :progressMessage
	ib_outlets :exportDialog, :saveToRadioButtons
	ib_outlets :goodPopupMenu, :seqClipNamesFavoritesCheckbox, :mp4TimecodeCheckbox, :dayTimecodeCheckbox, :dtdCheckbox

	def initialize
		default_values = {"Good" => "Favorite", "SeqClipNamesFavorites" => 0, "MP4timecode" => 1, "24Htimecode" => 0, "SaveToLocation" => 1, "DTDversion" => 0}
		defaultsController = OSX::NSUserDefaultsController.sharedUserDefaultsController
		defaultsController.setInitialValues(default_values)
		### check for Application Support subfolder
		paths = OSX::NSFileManager.defaultManager.URLsForDirectory_inDomains(OSX::NSApplicationSupportDirectory, OSX::NSUserDomainMask)
		if paths.length > 0
			@app_support_subfolder = paths[0].path.stringByExpandingTildeInPath + "/com.AssistedEditing.7toX"
			Dir.mkdir(@app_support_subfolder) unless File.directory?(@app_support_subfolder)
			# help_pdf = "7toX for Final Cut Pro - Help.pdf"
			# bundle_pdf_path = OSX::NSBundle.mainBundle.bundlePath + "/Contents/Resources/#{help_pdf}"
			# container_pdf_path = @app_support_subfolder + "/#{help_pdf}"
			# defaultManager = OSX::NSFileManager.defaultManager
			# result, error = defaultManager.removeItemAtPath_error(container_pdf_path)
			# result, error = defaultManager.copyItemAtPath_toPath_error(bundle_pdf_path, container_pdf_path)
		end
		@show_open = true
	end

	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		startup()
		if @this_rego[0]["active"]
			# showMainView()
		else
			@deactivateMenuItem.setTitle("Activate...")
			check_for_rego(@this_rego)
		end
		# puts "applicationWillFinishLaunching"
		OSX::LetsMove.PFMoveToApplicationsFolderIfNecessary # check_for_expired_beta THEN check LetsMove PFMoveToApplicationsFolderIfNecessary
		# get the bundle identifier for Final Cut Pro or Trial
		fcpx_app = OSX::NSWorkspace.sharedWorkspace.absolutePathForAppBundleWithIdentifier('com.apple.FinalCut')
		if fcpx_app.nil?
			fcpt_app = OSX::NSWorkspace.sharedWorkspace.absolutePathForAppBundleWithIdentifier('com.apple.FinalCutTrial')
			if fcpt_app.nil?
				@bundleIdentifier = nil
			else
				@bundleIdentifier = 'com.apple.FinalCutTrial'
			end
		else
			@bundleIdentifier = 'com.apple.FinalCut'
		end
		# verify Final Cut Pro or Trial version number
		unless @bundleIdentifier.nil?
			fcpx_path = OSX::NSWorkspace.sharedWorkspace.absolutePathForAppBundleWithIdentifier( @bundleIdentifier)
			unless fcpx_path.nil?
				fcpx_bundle = OSX::NSBundle.bundleWithPath(fcpx_path)
				unless fcpx_bundle.nil?
					fcpx_version = fcpx_bundle.infoDictionary.objectForKey("CFBundleShortVersionString").to_s
					min_version = "10.0.6"
					if fcpx_version < min_version
						alert = OSX::NSAlert.alloc.init
						alert.setMessageText("Please update Final Cut Pro X")
						alert.setInformativeText("7toX for Final Cut Pro requires version #{min_version} or later. This is a free update available in the App Store.")
						alert.setAlertStyle(OSX::NSCriticalAlertStyle)
						alert.addButtonWithTitle("Go to the App Store")
						alert.runModal
						system('open', 'macappstore://showUpdatesPage')
						OSX::NSApp.terminate(self)
					end
				end
			end
		end
		@openPanel = OSX::NSOpenPanel.openPanel # retain as instance variables
		@savePanel = OSX::NSSavePanel.savePanel
		@xml_paths = Array.new
	end
	
	def showDeactivation
		@webView.setDrawsBackground(false)
		@mainWindow.makeKeyAndOrderFront(self)
		if @deactivateMenuItem.title=="Activate..."
			showActivationView(nil, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
			@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
			showActivationView(nil, WEB_APP + "destroy/" + @this_rego[0]["number"].to_s)
		end
		@closeRegoButton.setTitle("Cancel")
	end
	ib_action :showDeactivation # Activate or Deactivate
	
	def showActivationView(frame, url)
		@webView.setDrawsBackground(false)
		@mainWindow.makeKeyAndOrderFront(self)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			@mainWindow.close
			open_fcp7xml(self)
		else
			OSX::NSApp.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def application_openFile(sender, file)
		puts @this_rego.inspect
		@file = file
		if @this_rego[0]["active"]
			@show_open = false
			@progressPanel.makeKeyAndOrderFront(self)
			@progressIndicator.setIndeterminate(true)
			# @progressIndicator.setUsesThreadedAnimation(true)
			@progressIndicator.startAnimation(self)
			@progressMessage.setStringValue("Opening...\n")
			@progressPanel.displayIfNeeded
			$dtd = nil
			if @dtdCheckbox.state==1 # don't make asset-clips
				$dtd = "1.5"
			end
			Thread.new { translate_from_7_to_X(@file) }
			return true
		else
			check_for_rego(@this_rego)
		end
	end
	
	def applicationDidFinishLaunching(sender) # sent after application:openFile:
		if @show_open
			open_fcp7xml(self)
		end
	end
		
	def open_fcp7xml(sender)
		openPanel = OSX::NSOpenPanel.openPanel
		openPanel.setTitle("7toX for Final Cut Pro")
		openPanel.setMessage("Choose one or more Final Cut Pro XML files to translate")
		openPanel.setAllowedFileTypes(["xml"])
		openPanel.setAllowsMultipleSelection(true)
		buttonClicked = openPanel.runModal
		if buttonClicked==1 # NSOKButton
			@xml_paths = Array.new
			openPanel.URLs.each do |url|
				@xml_paths << url.path
			end
			file = @xml_paths.shift
			application_openFile(self, file)
		end
	end
	ib_action :open_fcp7xml
	
	def translationDidFinish(file)
		OSX::NSDocumentController.sharedDocumentController.noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath_isDirectory(file, false) ) # add this file to the Recent menu
		@fcp7xml = File.basename(file)
		if @offline_files.length > 0 # report offline files if needed
			alert = OSX::NSAlert.alloc.init
			alert.setDelegate(self)
			if @offline_files.length==1
				alert.setMessageText("There is one missing media file that will need to be relinked in Final Cut Pro X")
			else
				alert.setMessageText("There are #{@offline_files.length} missing media files that will need to be relinked in Final Cut Pro X")
			end
			alert.setInformativeText("See “Relink clips to media files” in Final Cut Pro X Help for more information.")
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			# alert.setShowsHelp(true) # triggers alertShowHelp delegate method
			alert.addButtonWithTitle("Continue")
			alert.addButtonWithTitle("Save a Batch List")
			buttonClicked = alert.runModal
			if buttonClicked==OSX::NSAlertFirstButtonReturn
				# Continue
			else
				batch_name = File.basename(file, ".xml")
				batch_list = "Name\tMedia Start\tMedia End\tReel\tSource\r" + @offline_files.join("\r") # Batch List needs \r instead of \n
				@savePanel.setMessage("Save a Batch List of missing media files")
				@savePanel.setNameFieldStringValue("#{batch_name} Batch List.txt")
				@savePanel.setAllowedFileTypes(["txt"])
				@savePanel.setExtensionHidden(true)
				@savePanel.setCanSelectHiddenExtension(false)
				buttonClicked = @savePanel.runModal
				if buttonClicked == OSX::NSOKButton
					save_this_file = File.new(@savePanel.URL.path.to_s, "w")
					save_this_file.puts batch_list
					save_this_file.close
				end
				# then continue...
			end
		end
		if @bundleIdentifier.nil? # FCPX not installed
			saveXML()
		else
			is_running = OSX::NSRunningApplication.runningApplicationsWithBundleIdentifier( @bundleIdentifier)
			if is_running.empty? # FCPX is not running - ask what to do
				OSX::NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@exportDialog, @progressPanel, self, nil, nil)
			else # FCPX is running so send it the translation
				importXML()
			end
		end
	end

	def closeExportDialog(sender)
		OSX::NSApp.endSheet(@exportDialog)
		@exportDialog.orderOut(self)
		case @saveToRadioButtons.selectedCell.tag
		when 1
			importXML()
		when 2
			saveXML()
		end
	end
	ib_action :closeExportDialog
	
	def importXML
		name = @fcp7xml.sub(File.extname(@fcp7xml), "") # deals with file extension .XML
		@progressMessage.setStringValue("Sending '#{name}' to Final Cut Pro X...\n")
		@progressIndicator.setIndeterminate(true)
		# @progressIndicator.setUsesThreadedAnimation(true)
		@progressIndicator.startAnimation(self)
		@progressPanel.displayIfNeeded
		Dir.mkdir(@app_support_subfolder) unless File.directory?(@app_support_subfolder)
		case $dtd
		when "1.2", "1.3"
			save_event_xml(@new_xml_file)
		when "1.6"
			saveLibraryXML(@new_xml_file)
		else
			save_library_xml(@new_xml_file)
		end
		sleep 2
		unless @bundleIdentifier.nil?
			fcpx_path = OSX::NSWorkspace.sharedWorkspace.URLForApplicationWithBundleIdentifier(@bundleIdentifier).path
			OSX::NSWorkspace.sharedWorkspace.openFile_withApplication(@new_xml_file, fcpx_path) unless fcpx_path.nil?
		end
		openNextFile
	end
	
	def saveXML
		@savePanel.setMessage("Save the new Final Cut Pro X XML file")
		@savePanel.setNameFieldStringValue(@fcp7xml.sub(File.extname(@fcp7xml), " 7toX.fcpxml"))
		@savePanel.setAllowedFileTypes(["fcpxml"])
		@savePanel.setExtensionHidden(true)
		@savePanel.setCanSelectHiddenExtension(false)
		buttonClicked = @savePanel.runModal
		@savePanel.close
		if buttonClicked == OSX::NSOKButton
			path_to_fcpxml = @savePanel.URL.path.to_s
			# TODO: use writeToFile:options:error: to try saving a file and presenting an error (such as no permissions) if there is a problem, instead of checking before-hand...
			@progressMessage.setStringValue("Saving Final Cut Pro X XML file...\n")
			@progressIndicator.setIndeterminate(true)
			# @progressIndicator.setUsesThreadedAnimation(true)
			@progressIndicator.startAnimation(self)
			@progressPanel.displayIfNeeded
			case $dtd
			when "1.2", "1.3"
				save_event_xml(path_to_fcpxml)
			when "1.6"
				saveLibraryXML(path_to_fcpxml)
			else
				save_library_xml(path_to_fcpxml)
			end
		end
		@progressMessage.setStringValue("")
		@progressIndicator.stopAnimation(self)
		openNextFile
	end
	
	def openNextFile
		if @xml_paths.empty?
			@progressPanel.close
		else
			file = @xml_paths.shift
			application_openFile(self, file)
		end
	end

	def showHelp(sender)
		openPDF("SendToX - Help.pdf")
	end
	ib_action :showHelp

	def alertShowHelp(alert) # Missing media alert dialog
#		system('open', "help:search='relink clips to media files' bookID=com.apple.FinalCutPro10.help")
		system('open', "http://help.apple.com/finalcutpro/mac/#ver26f5c8c9")
		return true
	end
	
	def showHelp2(sender)
		openPDF("7toX Activating and Deactivating.pdf")
	end
	ib_action :showHelp2

	def openPDF(filename)
		path_to_pdf = OSX::NSBundle.mainBundle.bundlePath + "/Contents/Resources/#{filename}"
		system('open', path_to_pdf)
		# system('open', @app_support_subfolder + "/#{filename}")
	end
	
	def emailSupport(sender)
    executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
		version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
		app = "#{executable} (version #{version})".to_ns.stringByAddingPercentEscapesUsingEncoding(4)
		body = "What went wrong?\n\n\nAttached is the XML file that caused the problem:\n\n\n".to_ns.stringByAddingPercentEscapesUsingEncoding(4)
		mailto = OSX::NSURL.URLWithString("mailto:support@IntelligentAssistance.com?subject=Bug%20Report%20for%20#{app}&body=#{body}")
		result = OSX::NSWorkspace.sharedWorkspace.openURL(mailto)
		if result==false
			contactus = OSX::NSURL.URLWithString("https://intelligentassistance.com/contact-us.html")
			OSX::NSWorkspace.sharedWorkspace.openURL(contactus)
		end
	end
	ib_action :emailSupport

	def visitWebsite(sender)
		system('open', 'https://intelligentassistance.com/sendtox.html')
	end
	ib_action :visitWebsite # Help > Visit Web Site

end
