def save_event_xml(new_xml_file)
	#puts "------------------------------------------ save_event_xml"
	begin
		@id = 0 # text-style-def unique id number
		builder = Nokogiri::XML::Builder.new do |fcpxml|
      executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
			version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
			fcpxml.comment(" #{executable} v#{version}, #{Time.now.strftime("%Y-%m-%d")} ") unless executable.nil?
			fcpxml.doc.create_internal_subset("fcpxml", nil, nil)
			fcpxml.fcpxml(:version => $dtd) {
				fcpxml.project(:name => @event[:name]) {
					fcpxml.resources {
						# Formats
						@formats.each_with_index do |f, i|
							#puts f.inspect
							if f[:fieldOrder].nil? # audio
								fcpxml.format(:id => f[:id], :name => f[:name])
							elsif f[:stillframe]
								fcpxml.format(:id => f[:id], :name => "FFVideoFormatRateUndefined", :width => f[:width], :height => f[:height])
							elsif f[:name].nil? # custom (fieldOrder?)
								fcpxml.format(:id => f[:id], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height])
							elsif f[:paspH].nil?
								fcpxml.format(:id => f[:id], :name => f[:name], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height])
							else
								fcpxml.format(:id => f[:id], :name => f[:name], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height], :paspH => f[:paspH], :paspV => f[:paspV])
							end
						end
						# Assets
						@assets.each_with_index do |a, i|
							#puts a.inspect
							case a[:type]
							when "av"
								fcpxml.asset(:id => a[:id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasVideo => "1", :hasAudio => "1", :audioSources => a[:audioSources], :audioChannels => a[:audioChannels], :audioRate => a[:samplerate])
							when "a"
								fcpxml.asset(:id => a[:id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasAudio => "1", :audioSources => a[:audioSources], :audioChannels => a[:audioChannels], :audioRate => a[:samplerate])
							when "v", "psd", "img"
								if a[:stillframe]
									fcpxml.asset(:id => a[:id], :src => a[:src], :duration => "0s", :start => "0s", :hasVideo => "1")
								else
									fcpxml.asset(:id => a[:id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasVideo => "1")
								end
							else # when could type be ""?
								fcpxml.asset(:id => a[:id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]))
							end
							unless a[:aux_tc].empty?
								query = "fcpxml/project/resources/asset"
								Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last) do |asset|
									asset.metadata {
										asset.md(:key => "com.apple.proapps.custom.soundroll", :displayName => "Sound Roll", :value => a[:aux_tc].first, :type => "string", :editable => "1",  :source => "custom source")
										asset.md(:key => "com.apple.proapps.custom.soundtc", :displayName => "Aux TC", :value => a[:aux_tc].last, :type => "string", :editable => "1",  :source => "custom source")
									}
								end
							end
						end
						# Multicams
						@multiclips.each_with_index do |multiclip, m|
							fcpxml.media(:id => multiclip[:id], :name => multiclip[:name]) {
								fcpxml.multicam(:format => multiclip[:format_id], :tcFormat => multiclip[:tcFormat], :tcStart => to_rational_seconds(multiclip[:frame], multiclip[:frameDuration])) {
									multiclip[:angles].each do |a|
										unless a.nil?
											fcpxml.send("mc-angle", :name => a[:name], :angleID => a[:id]+"angle") {
												fcpxml.gap(:duration => to_rational_seconds(a[:mediadelay], a[:frameDuration])) unless a[:mediadelay]==0
												fcpxml.clip(:name => a[:name], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :offset => to_rational_seconds(a[:mediadelay], a[:frameDuration]), :tcFormat => a[:tcFormat]) {
													case a[:type]
													when "v"
														fcpxml.video(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]))
													when "a"
														if a[:channels]=="stereo" # one or more stereo tracks
															fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration])) {
																unless a[:audioSources]==1
																	a[:audioSources].downto(2) do |src|
																		fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-src, :srcID => src)
																	end
																end
															}
														elsif a[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
																fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
															}
														# elsif a[:channels]=="surround"
														else # one or more mono tracks
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :srcCh => 1 ) {
																unless a[:channelcount]==1
																	if a[:audioSources]==1 # no srcID because audioSources==1
																		a[:audioChannels].downto(2) do |ch|
																			fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-ch, :srcCh => ch )
																		end
																	elsif a[:audioChannels]==1 || a[:audioChannels]==2
																		a[:audioSources].downto(2) do |src|
																			srcCh = "1"
																			unless a[:srcChs][src-1].nil?
																				srcCh = a[:srcChs][src-1]
																			end
																			fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
																		end
																	end
																end
															}
														end
													when "av"
														fcpxml.video(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration])) {
															if a[:channels]=="stereo" # one or more stereo tracks
																a[:audioSources].downto(1) do |src|
																	if src==1 # no srcID
																		fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1)
																	else
																		fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-src, :srcID => src)
																	end
																end
															elsif a[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
																fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
																fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
															# elsif a[:channels]=="surround"
															else # one or more mono tracks
																if a[:audioSources]==1 # no srcID because audioSources==1
																	a[:audioChannels].downto(1) do |ch|
																		fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-ch, :srcCh => ch )
																	end
																elsif a[:audioChannels]==1 || a[:audioChannels]==2
																	a[:audioSources].downto(1) do |src|
																		srcCh = "1"
																		unless a[:srcChs][src-1].nil?
																			srcCh = a[:srcChs][src-1]
																		end
																		fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
																	end
																end
															end
														}
													end
												}
											}
										end
									end
								}
							}
						end
						# Compound Clips - add each sequence as a reference compound clip
						#puts "working on #{@project_seqs.length} compound clips"
						# Compound Clips - add each nested sequence as a reference compound clip
						#puts "working on #{@compound_clips.length} compound clips"
						@nested_seqs.each do |nested_seq|
							#puts nested_seq[:name]
							cc_frameDuration = nested_seq[:frameDuration]
							fcpxml.media(:id => nested_seq[:nested_ref_id], :name => nested_seq[:name]) {
								fcpxml.sequence(:format => nested_seq[:format_id], :duration => to_rational_seconds(nested_seq[:duration], cc_frameDuration), :tcFormat => nested_seq[:nested_seq_tcFormat], :audioLayout => nested_seq[:nested_seq_audioLayout], :tcStart => to_rational_seconds(nested_seq[:frame], cc_frameDuration)) { # audioRate is expressed as 48k instead of 48000
									# if stereo, fcpxml.parent["layout"] = "stereo"
									fcpxml.note nested_seq[:metadata] unless nested_seq[:metadata].empty?
									fcpxml.spine {
										clip_array = nested_seq[:nested_array]
										last_clip_index = clip_array.first.length - 1
										if clip_array.first.empty?
											last_clip_index = 0
										else
											if clip_array.first[last_clip_index][:type]=="gap"
												last_clip_index -= 1
											end
											if clip_array.first[last_clip_index][:type]=="transition"
												last_clip_index -= 1
											end
										end
										#puts "#{clip_array.first.length} primary storyline clips"
										clip_array.first.each_with_index do |primary_clip, p| # clip_array.first is the V1 track, which becomes the primary storyline
											@error_tracker = ":\n'#{nested_seq[:name]}' Clip '#{primary_clip[:name]}'"
											storyline_builder(fcpxml, [primary_clip], nested_seq[:frame], nested_seq[:frame], cc_frameDuration, nested_seq[:height], true) # node, clips, adj, frame, frameDuration, height, primary=false
											path_to_node = Array.new
											case primary_clip[:type] # also check case nested_primary_clip[:type]
											when "title"
												primary_start = (3600 * 1/cc_frameDuration).round
												path_to_node << "title"
											when "gap"
												primary_start = (3600 * 1/cc_frameDuration).round
												path_to_node << "gap"
											when "generator"
												primary_start = (3600 * 1/cc_frameDuration).round
												path_to_node << "video"
											when "transition"
												# ignore - can't be connected to
											when "mc"
												primary_start = (primary_clip[:mc_frame] / primary_clip[:speed_adj]) + primary_clip[:in] # bugfix for mc
												path_to_node << "mc-clip"
											when "nested"
												primary_start = (primary_clip[:frame] / primary_clip[:speed_adj]) + primary_clip[:in] # WAS nested_seq[:frame]
												path_to_node << "clip"
											else # clip
												# refactor the primary clip's first frame to the sequences frame rate (Bug fix for clips and secondary storylines connected to a primary clip whose frame rate is different from the timeline's frame rate)
												refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/cc_frameDuration)
												primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
												path_to_node << "clip"
											end
											primary_offset = primary_clip[:start]
											path_to_node << "spine"
											fcpxml.parent.ancestors.each { |a| path_to_node << a.name unless a.name=="document" }
											query = path_to_node.reverse.join("/")
											#puts "query = #{query}"
											#puts "************** primary_start = #{primary_start}", "primary_offset = #{primary_offset}"
											@compound_clip_name = nested_seq[:name]
											Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last) do |primary_node|
												unless primary_clip[:array_of_connecteds].nil? || primary_clip[:type]=="transition"
													# attach connected clips or secondary spines to the last primary clip
													# if primary_clip[:frameDuration].nil?
													# 	pc_frameDuration = cc_frameDuration
													# else
													# 	pc_frameDuration = primary_clip[:frameDuration]
													# end
													# puts "pc_frameDuration = #{pc_frameDuration}"
													primary_clip[:array_of_connecteds].each do |connecteds|
														unless connecteds.empty?
															if connecteds.length==1 && connecteds.first[:type]=="transition"
																#puts "CONNECTING A SINGLE TRANSITION !!!!!!"
															end
															#puts "************** primary_start - primary_offset + connecteds.first[:start] = #{primary_start} - #{primary_offset} + #{connecteds.first[:start]}"
															if connecteds.length==1 && connecteds.first[:type]!="transition" # connected clip
																storyline_builder(primary_node, connecteds, primary_start - primary_offset, nested_seq[:frame], cc_frameDuration, nested_seq[:height]) # node, clips, adj, frame, frameDuration, height, primary=false
															elsif connecteds.length > 1 # secondary storyline
																primary_node.spine(:lane => connecteds.first[:lane], :offset => to_rational_seconds(primary_start - primary_offset + connecteds.first[:start], cc_frameDuration)) {
																	storyline_builder(primary_node, connecteds, connecteds.first[:start]*-1, nested_seq[:frame], cc_frameDuration, nested_seq[:height])
																}
															end
														end
													end
												end
												# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
												# pp primary_clip
												unless primary_start.nil? || primary_clip[:type]=="transition"
													if p==last_clip_index
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], primary_clip[:tail_markers], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], cc_frameDuration)
													else
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], [], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], cc_frameDuration)
													end
													markers.each { |m| primary_node << m }
													chmarkers.each { |m| primary_node << m }
												end
												add_intrinsic_params_audio(primary_node, primary_clip)
												add_filters(primary_node, primary_clip) unless primary_clip[:filters].nil? || primary_clip[:type]=="mc"
												if primary_clip[:text]!=nil && $dtd=="1.2" # 1.3 is already dealt with in storyline_builder
													primary_clip[:text].each { |txt| primary_node.text_(txt) }
												end
												unless primary_clip[:reel_name].nil? && primary_clip[:scene].nil? && primary_clip[:shottake].nil? # transitions, slugs etc.
													# primary_node.metadata {
													# 	"primary_node.metadata line 240"
													# 	#puts "primary_clip[:reel_name]", primary_clip[:reel_name].inspect
													# 	# FIXME unless primary_clip[:reel_name].empty?
													# 	primary_node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => primary_clip[:reel_name])
													# 	primary_node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => primary_clip[:scene])
													# 	primary_node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => primary_clip[:shottake])
													# 	primary_node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => primary_clip[:label])
													# }
												end
											end
										end
									}
								}
							}
						end # end @nested_seqs.each
						@project_seqs.each do |project_seq|
							#puts project_seq[:name]
							seq_frameDuration = project_seq[:frameDuration]
							fcpxml.media(:id => project_seq[:id], :name => project_seq[:name]) {
								fcpxml.sequence(:format => project_seq[:format_id], :duration => to_rational_seconds(project_seq[:duration], seq_frameDuration), :tcFormat => project_seq[:tcFormat], :audioLayout => project_seq[:audioLayout], :tcStart => to_rational_seconds(project_seq[:frame], seq_frameDuration)) { # audioRate is expressed as 48k instead of 48000
									# if stereo, fcpxml.parent["layout"] = "stereo"
									fcpxml.note project_seq[:metadata] unless project_seq[:metadata].empty?
									fcpxml.spine {
										clip_array = project_seq[:clip_array]
										last_clip_index = clip_array.first.length - 1
										if clip_array.first.empty?
											last_clip_index = 0
										else
											if clip_array.first[last_clip_index][:type]=="gap"
												last_clip_index -= 1
											end
											if clip_array.first[last_clip_index][:type]=="transition"
												last_clip_index -= 1
											end
										end
										#puts "#{clip_array.first.length} primary storyline clips"
										clip_array.first.each_with_index do |primary_clip, p| # clip_array.first is the V1 track, which becomes the primary storyline
											@error_tracker = ":\n'#{project_seq[:name]}' Clip '#{primary_clip[:name]}'"
											storyline_builder(fcpxml, [primary_clip], project_seq[:frame], project_seq[:frame], seq_frameDuration, project_seq[:height], true) # node, clips, adj, frame, frameDuration, height, primary=false
											path_to_node = Array.new
											case primary_clip[:type] # also check case nested_primary_clip[:type]
											when "title"
												primary_start = (3600 * 1/seq_frameDuration).round
												path_to_node << "title"
											when "gap"
												primary_start = (3600 * 1/seq_frameDuration).round
												path_to_node << "gap"
											when "generator"
												primary_start = (3600 * 1/seq_frameDuration).round
												path_to_node << "video"
											when "transition"
												# ignore - can't be connected to
											when "mc"
												primary_start = (primary_clip[:mc_frame] / primary_clip[:speed_adj]) + primary_clip[:in] # bugfix for mc
												path_to_node << "mc-clip"
											when "nested"
												primary_start = (primary_clip[:frame] / primary_clip[:speed_adj]) + primary_clip[:in] # WAS project_seq[:frame]
												path_to_node << "clip"
											else # clip
												# refactor the primary clip's first frame to the sequences frame rate (Bug fix for clips and secondary storylines connected to a primary clip whose frame rate is different from the timeline's frame rate)
												refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/seq_frameDuration)
												primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
												path_to_node << "clip"
											end
											primary_offset = primary_clip[:start]
											path_to_node << "spine"
											fcpxml.parent.ancestors.each { |a| path_to_node << a.name unless a.name=="document" }
											query = path_to_node.reverse.join("/")
											#puts "query = #{query}"
											#puts "************** primary_start = #{primary_start}", "primary_offset = #{primary_offset}"
											@project_seq_name = project_seq[:name]
											Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last) do |primary_node|
												unless primary_clip[:array_of_connecteds].nil? || primary_clip[:type]=="transition"
													# attach connected clips or secondary spines to the last primary clip
													# if primary_clip[:frameDuration].nil?
													# 	pc_frameDuration = seq_frameDuration
													# else
													# 	pc_frameDuration = primary_clip[:frameDuration]
													# end
													# puts "pc_frameDuration = #{pc_frameDuration}"
													primary_clip[:array_of_connecteds].each do |connecteds|
														unless connecteds.empty?
															if connecteds.length==1 && connecteds.first[:type]=="transition"
																#puts "CONNECTING A SINGLE TRANSITION !!!!!!"
															end
															#puts "************** primary_start - primary_offset + connecteds.first[:start] = #{primary_start} - #{primary_offset} + #{connecteds.first[:start]}"
															if connecteds.length==1 && connecteds.first[:type]!="transition" # connected clip
																storyline_builder(primary_node, connecteds, primary_start - primary_offset, project_seq[:frame], seq_frameDuration, project_seq[:height]) # node, clips, adj, frame, frameDuration, height, primary=false
															elsif connecteds.length > 1 # secondary storyline
																primary_node.spine(:lane => connecteds.first[:lane], :offset => to_rational_seconds(primary_start - primary_offset + connecteds.first[:start], seq_frameDuration)) {
																	storyline_builder(primary_node, connecteds, connecteds.first[:start]*-1, project_seq[:frame], seq_frameDuration, project_seq[:height])
																}
															end
														end
													end
												end
												# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
												# pp primary_clip
												unless primary_start.nil? || primary_clip[:type]=="transition"
													if p==last_clip_index
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], primary_clip[:tail_markers], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], seq_frameDuration)
													else
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], [], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], seq_frameDuration)
													end
													markers.each { |m| primary_node << m }
													chmarkers.each { |m| primary_node << m }
												end
												add_intrinsic_params_audio(primary_node, primary_clip)
												add_filters(primary_node, primary_clip) unless primary_clip[:filters].nil? || primary_clip[:type]=="mc"
												if primary_clip[:text]!=nil && $dtd=="1.2" # 1.3 is already dealt with in storyline_builder
													primary_clip[:text].each { |txt| primary_node.text_(txt) }
												end
												unless primary_clip[:reel_name].nil? && primary_clip[:scene].nil? && primary_clip[:shottake].nil? # transitions, slugs etc.
													# primary_node.metadata {
													# 	#puts "primary_node.metadata line 354"
													# 	#puts "primary_clip[:reel_name]", primary_clip[:reel_name].inspect
													# 	# FIXME unless primary_clip[:reel_name].empty?
													# 	primary_node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => primary_clip[:reel_name])
													# 	primary_node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => primary_clip[:scene])
													# 	primary_node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => primary_clip[:shottake])
													# 	primary_node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => primary_clip[:label])
													# }
												end
											end
										end
									}
								}
							}
						end # end @project_seqs.each
						# Fix up faulty posteroffset attribute
						fcpxml.doc.xpath("//chapter-marker[@posteroffset]").each do |cm|
							cm.delete("posteroffset")
							cm["posterOffset"] = "0s"
						end
						# Transitions
						fcpxml.effect(:id => "e1", :name => "Cross Dissolve", :uid => "FxPlug:4731E73A-8DAC-4113-9A30-AE85B1761265")
						if $dtd=="1.2"
							fcpxml.effect(:id => "ea1", :name => "Cross Dissolve", :uid => "FxPlug:4731E73A-8DAC-4113-9A30-AE85B1761265")
						else
							fcpxml.effect(:id => "ea1", :name => "Audio Crossfade", :uid => "FFAudioTransition")
						end
						fcpxml.effect(:id => "e2", :name => "Center", :uid => "FxPlug:539C8A29-BBED-4670-B774-338109A7DB68")
						fcpxml.effect(:id => "e3", :name => "Squares", :uid => ".../Transitions.localized/Objects.localized/Squares.localized/Squares.motr")
						fcpxml.effect(:id => "e4", :name => "Zoom", :uid => ".../Transitions.localized/Blurs.localized/Zoom.localized/Zoom.motr")
						fcpxml.effect(:id => "e5", :name => "Cube", :uid => "FxPlug:653CCCCD-6CB1-4602-B9B3-181B1C53E069")
						fcpxml.effect(:id => "e6", :name => "Flip", :uid => ".../Transitions.localized/Movements.localized/Flip.localized/Flip.motr")
						fcpxml.effect(:id => "e7", :name => "Swing", :uid => ".../Transitions.localized/Movements.localized/Swing.localized/Swing.motr")
						fcpxml.effect(:id => "e8", :name => "Fade To Color", :uid => "FxPlug:F779C565-486D-4633-8035-0374B4DB8F5C")
						fcpxml.effect(:id => "e9", :name => "Ripple", :uid => "FxPlug:D386C0D5-5BD3-4A0D-86FC-6B9878C76451")
						fcpxml.effect(:id => "e10", :name => "Letter X", :uid => "FxPlug:743DCA2A-B49F-4B19-8A52-B9575D631139")
						fcpxml.effect(:id => "e11", :name => "Circle", :uid => "FxPlug:1C52AC71-7116-4248-B51F-5F5641EA9EDD")
						fcpxml.effect(:id => "e12", :name => "Inset Wipe", :uid => "FxPlug:5F0E1FA2-C172-4AFF-88CB-64B0F08195E4")
						fcpxml.effect(:id => "e13", :name => "Wipe", :uid => "FxPlug:857E2FBA-98DB-411B-A88C-CE6ABC1F65D8")
						fcpxml.effect(:id => "e14", :name => "Page Curl", :uid => "FxPlug:21B9D89C-4162-43FF-BA07-B019989E4940")
						fcpxml.effect(:id => "e15", :name => "Gradient Image", :uid => "FxPlug:62199A78-FDBF-4E9B-BB25-9A1D62FD2D48")
						fcpxml.effect(:id => "e16", :name => "Mosaic", :uid => "FxPlug:66B87635-119C-40B2-BF6E-537A3165FB95")
						fcpxml.effect(:id => "e17", :name => "Clock", :uid => "FxPlug:B2C3F87B-2A21-4E13-8173-46ED4FEBC57A")
						fcpxml.effect(:id => "e18", :name => "Slide", :uid => "FxPlug:6AAB0D54-FCD8-4EBD-A62D-D352A5ED1648")
						fcpxml.effect(:id => "e19", :name => "Spin", :uid => "FxPlug:196B9DB2-28FD-420F-9BA1-3F0E9EEBBAAA")
						fcpxml.effect(:id => "e20", :name => "Band", :uid => "FxPlug:8AD995CC-8DD6-4652-A3D2-B1468DCC6203")
						fcpxml.effect(:id => "e21", :name => "Black Hole", :uid => ".../Transitions.localized/Movements.localized/Black Hole.localized/Black Hole.motr")
						# Slug and Generators
						fcpxml.effect(:id => "g1", :name => "Custom", :uid => ".../Generators.localized/Solids.localized/Custom.localized/Custom.motn")
						fcpxml.effect(:id => "g2", :name => "Gradient", :uid => ".../Generators.localized/Textures.localized/Gradient.localized/Gradient.motn")
						fcpxml.effect(:id => "g3", :name => "Shapes", :uid => ".../Generators.localized/Elements.localized/Shapes.localized/Shapes.motn")
						# Titles
						fcpxml.effect(:id => "t1", :name => "Custom", :uid => ".../Titles.localized/Build In:Out.localized/Custom.localized/Custom.moti")
						fcpxml.effect(:id => "t2", :name => "Middle", :uid => ".../Titles.localized/Lower Thirds.localized/Middle.localized/Middle.moti")
						fcpxml.effect(:id => "t3", :name => "Scrolling", :uid => ".../Titles.localized/Credits.localized/Scrolling.localized/Scrolling.moti")
						fcpxml.effect(:id => "t4", :name => "Ticker", :uid => ".../Titles.localized/Build In:Out.localized/Ticker.localized/Ticker.moti")
						fcpxml.effect(:id => "t5", :name => "Typewriter", :uid => ".../Titles.localized/Build In:Out.localized/Typewriter.localized/Typewriter.moti")
						# Video Filters
						fcpxml.effect(:id => "f1", :name => "Censor", :uid => ".../Effects.localized/Stylize.localized/Censor.localized/Censor.moef")
						fcpxml.effect(:id => "f2", :name => "Cartoon", :uid => ".../Effects.localized/Stylize.localized/Cartoon.localized/Cartoon.moef")
						fcpxml.effect(:id => "f3", :name => "Directional", :uid => ".../Effects.localized/Blur.localized/Directional.localized/Directional.moef")
						fcpxml.effect(:id => "f4", :name => "Gaussian", :uid => ".../Effects.localized/Blur.localized/Gaussian.localized/Gaussian.moef")
						fcpxml.effect(:id => "f5", :name => "Prism", :uid => ".../Effects.localized/Blur.localized/Prism.localized/Prism.moef")
						fcpxml.effect(:id => "f6", :name => "Radial", :uid => ".../Effects.localized/Blur.localized/Radial.localized/Radial.moef")
						fcpxml.effect(:id => "f7", :name => "Zoom", :uid => ".../Effects.localized/Blur.localized/Zoom.localized/Zoom.moef")
						fcpxml.effect(:id => "f8", :name => "Mask", :uid => ".../Effects.localized/Keying.localized/Mask.localized/Mask.moef")
						fcpxml.effect(:id => "f9", :name => "Simple Border", :uid => ".../Effects.localized/Stylize.localized/Simple Border.localized/Simple Border.moef")
						fcpxml.effect(:id => "f10", :name => "Droplet", :uid => ".../Effects.localized/Distortion.localized/Droplet.localized/Droplet.moef")
						fcpxml.effect(:id => "f11", :name => "Earthquake", :uid => ".../Effects.localized/Distortion.localized/Earthquake.localized/Earthquake.moef")
						fcpxml.effect(:id => "f12", :name => "Fisheye", :uid => ".../Effects.localized/Distortion.localized/Fisheye.localized/Fisheye.moef")
						fcpxml.effect(:id => "f13", :name => "Fun House", :uid => ".../Effects.localized/Distortion.localized/Fun House.localized/Fun House.moef")
						fcpxml.effect(:id => "f14", :name => "Insect Eye", :uid => ".../Effects.localized/Distortion.localized/Insect Eye.localized/Insect Eye.moef")
						fcpxml.effect(:id => "f15", :name => "Scrape", :uid => ".../Effects.localized/Distortion.localized/Scrape.localized/Scrape.moef")
						fcpxml.effect(:id => "f16", :name => "Wave", :uid => ".../Effects.localized/Distortion.localized/Wave.localized/Wave.moef")
						fcpxml.effect(:id => "f17", :name => "Glow", :uid => ".../Effects.localized/Light.localized/Glow.localized/Glow.moef")
						fcpxml.effect(:id => "f18", :name => "Bloom", :uid => ".../Effects.localized/Light.localized/Bloom.localized/Bloom.moef")
						fcpxml.effect(:id => "f19", :name => "Hue/Saturation", :uid => ".../Effects.localized/Basics.localized/Hue:Saturation.localized/Hue:Saturation.moef")
						fcpxml.effect(:id => "f20", :name => "Sepia", :uid => ".../Effects.localized/Basics.localized/Sepia.localized/Sepia.moef")
						fcpxml.effect(:id => "f21", :name => "Tint", :uid => ".../Effects.localized/Basics.localized/Tint.localized/Tint.moef")
						fcpxml.effect(:id => "f22", :name => "Broadcast Safe", :uid => ".../Effects.localized/Basics.localized/Broadcast Safe.localized/Broadcast Safe.moef")
						fcpxml.effect(:id => "f23", :name => "Keyer", :uid => "FxPlug:41122549-B8A6-470E-94DA-211294D20B62")
						fcpxml.effect(:id => "f24", :name => "Luma Keyer", :uid => "FxPlug:7E9178C5-7B0F-4B86-884D-FE79F568B6CE")
						fcpxml.effect(:id => "f25", :name => "Image Mask", :uid => ".../Effects.localized/Keying.localized/Image Mask.localized/Image Mask.moef")
						fcpxml.effect(:id => "f26", :name => "Crop &amp; Feather", :uid => ".../Effects.localized/Distortion.localized/Crop &amp; Feather.localized/Crop &amp; Feather.moef")
						fcpxml.effect(:id => "f27", :name => "Letterbox", :uid => ".../Effects.localized/Stylize.localized/Letterbox.localized/Letterbox.moef")
						fcpxml.effect(:id => "f28", :name => "Flipped", :uid => ".../Effects.localized/Distortion.localized/Flipped.localized/Flipped.moef")
						fcpxml.effect(:id => "f29", :name => "Mirror", :uid => ".../Effects.localized/Distortion.localized/Mirror.localized/Mirror.moef")
						fcpxml.effect(:id => "f30", :name => "Sharpen", :uid => ".../Effects.localized/Blur.localized/Sharpen.localized/Sharpen.moef")
						fcpxml.effect(:id => "f31", :name => "Add Noise", :uid => ".../Effects.localized/Stylize.localized/Add Noise.localized/Add Noise.moef")
						fcpxml.effect(:id => "f32", :name => "Aged Film", :uid => ".../Effects.localized/Stylize.localized/Aged Film.localized/Aged Film.moef")
						fcpxml.effect(:id => "f33", :name => "Bad TV", :uid => ".../Effects.localized/Stylize.localized/Bad TV.localized/Bad TV.moef")
						fcpxml.effect(:id => "f34", :name => "Pixellate", :uid => ".../Effects.localized/Stylize.localized/Pixellate.localized/Pixellate.moef")
						fcpxml.effect(:id => "f35", :name => "Halftone", :uid => ".../Effects.localized/Stylize.localized/Halftone.localized/Halftone.moef")
						fcpxml.effect(:id => "f36", :name => "Cross Hatch", :uid => ".../Effects.localized/Stylize.localized/Cross Hatch.localized/Cross Hatch.moef")
						fcpxml.effect(:id => "f37", :name => "Graphic", :uid => ".../Effects.localized/Stylize.localized/Graphic.localized/Graphic.moef")
						fcpxml.effect(:id => "f38", :name => "Tile", :uid => ".../Effects.localized/Tiling.localized/Tile.localized/Tile.moef")
						fcpxml.effect(:id => "f39", :name => "Vignette", :uid => ".../Effects.localized/Stylize.localized/Vignette.localized/Vignette.moef")
						fcpxml.effect(:id => "f40", :name => "Kaleidoscope", :uid => ".../Effects.localized/Tiling.localized/Kaleidoscope.localized/Kaleidoscope.moef")
						fcpxml.effect(:id => "f41", :name => "Trails", :uid => ".../Effects.localized/Distortion.localized/Trails.localized/Trails.moef")
						fcpxml.effect(:id => "f42", :name => "Flashing", :uid => ".../Effects.localized/Light.localized/Flashing.localized/Flashing.moef")
						fcpxml.effect(:id => "f43", :name => "Camcorder", :uid => ".../Effects.localized/Stylize.localized/Camcorder.localized/Camcorder.moef")
						fcpxml.effect(:id => "f44", :name => "Kaleidotile", :uid => ".../Effects.localized/Tiling.localized/Kaleidotile.localized/Kaleidotile.moef")
						fcpxml.effect(:id => "f45", :name => "Black & White", :uid => ".../Effects.localized/Basics.localized/Black & White.localized/Black & White.moef")
						fcpxml.effect(:id => "f124", :name => "Drop Shadow",  :uid => "FxPlug:9C13F991-BC99-4DC8-B150-381D7CCE183B")
						# Audio Filters
						fcpxml.effect(:id => "f100", :name => "AUBandpass", :uid => "AudioUnit: 0x61756678627061736170706c")
						fcpxml.effect(:id => "f101", :name => "AUDelay", :uid => "AudioUnit: 0x6175667864656c796170706c")
						fcpxml.effect(:id => "f102", :name => "AUDistortion", :uid => "AudioUnit: 0x61756678646973746170706c")
						fcpxml.effect(:id => "f103", :name => "AUDynamicsProcessor", :uid => "AudioUnit: 0x6175667864636d706170706c")
						fcpxml.effect(:id => "f104", :name => "AUFilter", :uid => "AudioUnit: 0x6175667866696c746170706c")
						fcpxml.effect(:id => "f105", :name => "AUGraphicEQ", :uid => "AudioUnit: 0x61756678677265716170706c")
						fcpxml.effect(:id => "f106", :name => "AUHighShelfFilter", :uid => "AudioUnit: 0x61756678687368666170706c")
						fcpxml.effect(:id => "f107", :name => "AUHipass", :uid => "AudioUnit: 0x61756678687061736170706c")
						fcpxml.effect(:id => "f108", :name => "AULowpass", :uid => "AudioUnit: 0x617566786c7061736170706c")
						fcpxml.effect(:id => "f109", :name => "AULowShelfFilter", :uid => "AudioUnit: 0x617566786c7368666170706c")
						fcpxml.effect(:id => "f110", :name => "AUMultibandCompressor", :uid => "AudioUnit: 0x617566786d636d706170706c")
						fcpxml.effect(:id => "f111", :name => "AUNetSend", :uid => "AudioUnit: 0x617566786e736e646170706c")
						fcpxml.effect(:id => "f112", :name => "AUParametricEQ", :uid => "AudioUnit: 0x61756678706d65716170706c")
						fcpxml.effect(:id => "f113", :name => "AUPeakLimiter", :uid => "AudioUnit: 0x617566786c6d74726170706c")
						fcpxml.effect(:id => "f114", :name => "AUPitch", :uid => "AudioUnit: 0x61756678746d70746170706c")
						fcpxml.effect(:id => "f115", :name => "AURogerBeep", :uid => "AudioUnit: 0x61756678726f67726170706c")
						fcpxml.effect(:id => "f116", :name => "AUSampleDelay", :uid => "AudioUnit: 0x6175667873646c796170706c")
						fcpxml.effect(:id => "f117", :name => "Channel EQ", :uid => "AudioUnit: 0x61756678000000ec454d4147")
						fcpxml.effect(:id => "f118", :name => "Echo Delay", :uid => "Echo Delay.Echo.audio.effectBundle")
						fcpxml.effect(:id => "f119", :name => "Noise Gate", :uid => "AudioUnit: 0x61756678000000b3454d4147")
						fcpxml.effect(:id => "f120", :name => "Gain", :uid => "AudioUnit: 0x61756678000000b7454d4147")
						fcpxml.effect(:id => "f121", :name => "AUMatrixReverb", :uid => "AudioUnit: 0x617566786d7265766170706c")
						fcpxml.effect(:id => "f122", :name => "DeEsser", :uid => "AudioUnit: 0x61756678000000a0454d4147")
						fcpxml.effect(:id => "f123", :name => "Compressor", :uid => "AudioUnit: 0x617566780000009a454d4147")
					}
					### add each media asset as a clip(s) with keywords
					good = "Favorite"
					if @goodPopupMenu.nil? || @goodPopupMenu.titleOfSelectedItem=="Favorite"
						# Favorite
					elsif @goodPopupMenu.titleOfSelectedItem=='"Good" Keyword'
						good = "Good"
					else
						good = "Circle Take"
					end
					bins = Array.new
					@assets.each do |asset|
						unless asset[:type]=="mc" || asset[:type]=="psd" # FIXME
							if asset[:type]=="a"
								format_id = "ft1" # FFVideoFormatRateUndefined - don't add format to asset if it's audio-only
							else
								format_id = asset[:format_id]
							end
							asset[:clips].each do |c|
								fcpxml.clip(:name => c[:name], :duration => to_rational_seconds(c[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset], asset[:frameDuration]), :tcFormat => asset[:tcFormat], :format => format_id) {
									fcpxml.note c[:metadata] unless c[:metadata].nil?
									case asset[:type]
									when "v"
										fcpxml.video(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]))
									when "a"
										if asset[:channels]=="stereo" # one or more stereo tracks
											fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration])) {
												unless asset[:audioSources]==1
													asset[:audioSources].downto(2) do |src|
														fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 1-src, :srcID => src)
													end
												end
											}
										elsif asset[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
											fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
												fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
											}
										# elsif asset[:channels]=="surround"
										else # one or more mono tracks
											fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :srcCh => 1 ) {
												unless asset[:channelcount]==1
													if asset[:audioSources]==1 # no srcID because audioSources==1
														asset[:audioChannels].downto(2) do |ch|
															fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 1-ch, :srcCh => ch )
														end
													elsif asset[:audioChannels]==1 || asset[:audioChannels]==2
														asset[:audioSources].downto(2) do |src|
															srcCh = "1"
															unless asset[:srcChs][src-1].nil?
																srcCh = asset[:srcChs][src-1]
															end
															fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
														end
													end
												end
											}
										end
									when "av"
										fcpxml.video(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration])) {
											if asset[:channels]=="stereo" # one or more stereo tracks
												asset[:audioSources].downto(1) do |src|
													if src==1 # no srcID
														fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -1)
													else
														fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 0-src, :srcID => src)
													end
												end
											elsif asset[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
												fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
												fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
											# elsif asset[:channels]=="surround"
											else # one or more mono tracks
												if asset[:audioSources]==1 # no srcID because audioSources==1
													asset[:audioChannels].downto(1) do |ch|
														fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 0-ch, :srcCh => ch )
													end
												elsif asset[:audioChannels]==1 || asset[:audioChannels]==2
													asset[:audioSources].downto(1) do |src|
														srcCh = "1"
														unless asset[:srcChs][src-1].nil?
															srcCh = asset[:srcChs][src-1]
														end
														fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
													end
												end
											end
										}
									end
									unless c[:markers].nil? || c[:markers].empty?
										markers, chmarkers = order_markers(c[:markers], [], [], [], asset[:frame] + c[:subclip_startoffset], 1, c[:in], c[:out], asset[:frameDuration]) # first frame takes into account subclip_startoffset for subclips
										markers.each { |m| fcpxml << m }
										chmarkers.each { |m| fcpxml << m }
									end
									if c[:good]
										if good=="Favorite"
											if $dtd=="1.2"
												fcpxml.rating(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => "favorite")
											else
												fcpxml.rating(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => "favorite", :name => "Good")
											end
										else
											fcpxml.keyword(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => good)
										end
									end
									c[:bins].each do |b|
										fcpxml.keyword(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + b[:in], asset[:frameDuration]), :duration => to_rational_seconds(b[:out] - b[:in], asset[:frameDuration]), :value => b[:name]) # CHANGED removed b[:metadata]
										bins << b[:name]
									end
									fcpxml.metadata {
										#puts "fcpxml.metadata line 590"
										fcpxml.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => c[:reel_name]) unless c[:reel_name].nil? || c[:reel_name].empty?
										fcpxml.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => c[:scene]) unless c[:scene].nil? || c[:scene].empty?
										fcpxml.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => c[:shottake]) unless c[:shottake].nil? || c[:shottake].empty?
										fcpxml.md(:key => "com.apple.proapps.studio.angle", :editable => "1", :value => c[:defaultangle]) unless c[:defaultangle].nil? || c[:defaultangle].empty?
										fcpxml.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => c[:label]) unless c[:label].nil? || c[:label].empty?
									}
								}
							end
						end
					end
					### add each merged clip as a clip with keywords
					@mergedclips.each_with_index do |mergeclip, m|
						vclip = mergeclip[:angles].shift
						aclips = mergeclip[:angles].compact
						if vclip.nil?
							print "skipping because vclip is nil:\n"
							#pp mergeclip
						else
							fcpxml.clip(:name => mergeclip[:name], :duration => to_rational_seconds(mergeclip[:duration], mergeclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame] + 0, mergeclip[:frameDuration]), :tcFormat => mergeclip[:tcFormat], :format => mergeclip[:format_id]) {
								fcpxml.spine {
									if mergeclip[:syncoffset] < 0
										gap_duration = to_rational_seconds(mergeclip[:syncoffset].abs, mergeclip[:frameDuration])
										fcpxml.gap(:offset => "0s", :start => "3600s", :duration => gap_duration)
									else
										gap_duration = "0s"
									end
									fcpxml.clip(:name => vclip[:name], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame] + 0, vclip[:frameDuration]), :offset => gap_duration, :tcFormat => vclip[:tcFormat], :format => vclip[:format_id]) {
										fcpxml.video(:offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration])) {
											if vclip[:type]=="av"
												if vclip[:channels]=="stereo"
													fcpxml.audio(:offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => -1)
												elsif vclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
													fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(asset[:frame], vclip[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
													fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(asset[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(asset[:frame], vclip[:frameDuration]), :start => to_rational_seconds(asset[:frame], vclip[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
												else # one or more mono tracks
													if vclip[:audioSources]==1 # no srcID because audioSources==1
														vclip[:audioChannels].downto(1) do |ch|
															fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => 0-ch, :srcCh => ch )
														end
													elsif vclip[:audioChannels]==1 || vclip[:audioChannels]==2
														vclip[:audioSources].downto(1) do |src|
															srcCh = "1"
															unless vclip[:srcChs][src-1].nil?
																srcCh = vclip[:srcChs][src-1]
															end
															fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
														end
													end
												end
											end
										}
										aclips.each_with_index do |aclip, a|
											fcpxml.clip(:name => aclip[:name], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame] + 0, aclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame] + mergeclip[:syncoffset], vclip[:frameDuration]), :tcFormat => aclip[:tcFormat], :format => aclip[:format_id], :lane => "-#{a+1}") {
												if aclip[:channels]=="stereo"
													fcpxml.audio(:offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]))
												elsif aclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
													fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(a[:frame], aclip[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
														fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(a[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(a[:frame], aclip[:frameDuration]), :start => to_rational_seconds(a[:frame], aclip[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
													}
												else # one or more mono tracks
													fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :srcID => 1 ) {
														unless aclip[:channelcount]==1
															if aclip[:audioSources]==1 # no srcID because audioSources==1
																aclip[:audioChannels].downto(2) do |ch|
																	fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-ch, :srcCh => ch )
																end
															elsif aclip[:audioChannels]==1 || aclip[:audioChannels]==2
																aclip[:audioSources].downto(2) do |src|
																	srcCh = "1"
																	unless aclip[:srcChs][src-1].nil?
																		srcCh = aclip[:srcChs][src-1]
																	end
																	fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
																end
															end
														end
													}
												end
											}
										end
									}
								}
								mergeclip[:bins].each do |b|
									fcpxml.keyword(:start => "0s", :duration => to_rational_seconds(mergeclip[:duration], mergeclip[:frameDuration]), :value => b[:name])
									bins << b[:name]
								end
							}
						end
					end
					# add each multiclip reference as an Event mc-clip
					@multiclips.each_with_index do |multiclip, m|
						fcpxml.send("mc-clip", :ref => multiclip[:id], :name => multiclip[:name], :duration => to_rational_seconds(multiclip[:duration], multiclip[:frameDuration])) {
							fcpxml.send("mc-source", :angleID => multiclip[:activevideoangle], :srcEnable => "all")
						}
					end
					# add each sequence as an Event ref-clip with keyword
					@project_seqs.each do |project_seq|
						seq_frameDuration = project_seq[:frameDuration]
						fcpxml.send("ref-clip", :ref => project_seq[:id], :name => project_seq[:name], :duration => to_rational_seconds(project_seq[:duration], seq_frameDuration), :start => to_rational_seconds(project_seq[:frame], seq_frameDuration)) {
							fcpxml.keyword(:value => "Sequences", :duration => to_rational_seconds(project_seq[:duration], seq_frameDuration), :start => to_rational_seconds(project_seq[:frame], seq_frameDuration))
						}
					end
					# add each keyword as a keyword-collection
					unless $dtd=="1.2"
						unless bins.empty?
							fcpxml.send("collection-folder", :name => "Bins") {
								bins.uniq.each do |bin|
									fcpxml.send("keyword-collection", :name => bin)
								end
							}
						end
						fcpxml.send("keyword-collection", :name => "Sequences") if @project_seqs.length > 0
					end
				}
			}
		end
		save_this_file = File.new(new_xml_file, "w")
		save_this_file.puts builder.to_xml(:indent => 2, :encoding => "UTF-8")
		save_this_file.close
	rescue => e
		if $ruby_cocoa==false
			raise
		else
			OSX::NSLog "#{e.message}\n#{e.backtrace.join("\n")}"
      executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
			version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
			alert = OSX::NSAlert.alloc.init
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			alert.setMessageText("There was a problem creating the translated XML#{@error_tracker}")
			alert.setInformativeText("Please zip up the XML file and email it with the subject “#{executable} #{version} error” to 7toX@assistedediting.com with this error message in the body text:\n
			#{e.message}\n
			#{e.backtrace.first}\n
			Thanks for helping us improve #{executable}!")
			alert.runModal
		end
	end
end

def save_library_xml(new_xml_file)
	#puts "------------------------------------------ save_library_xml"
	begin
		@id = 0 # text-style-def unique id number
		builder = Nokogiri::XML::Builder.new do |fcpxml|
        executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
			version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
			fcpxml.comment(" #{executable} v#{version}, #{Time.now.strftime("%Y-%m-%d")} ") unless executable.nil?
			fcpxml.doc.create_internal_subset("fcpxml", nil, nil)
			fcpxml.fcpxml(:version => $dtd) {
				fcpxml.send("import-options") {
					fcpxml.option(:key => "copy assets", :value => "0") # link to assets instead of copy
					if $ruby_cocoa==false
						fcpxml.option(:key => "suppress warnings", :value => "0")
					else
						fcpxml.option(:key => "suppress warnings", :value => "1")
					end
				}
				fcpxml.resources {
					# Formats
					@formats.each_with_index do |f, i|
						#puts f.inspect
						if f[:fieldOrder].nil? || f[:width].nil? || f[:height].nil? # audio-only
							fcpxml.format(:id => f[:id], :name => f[:name])
						elsif f[:stillframe]
							fcpxml.format(:id => f[:id], :name => "FFVideoFormatRateUndefined", :width => f[:width], :height => f[:height])
						elsif f[:name].nil? # custom (fieldOrder?)
							fcpxml.format(:id => f[:id], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height])
						elsif f[:paspH].nil?
							fcpxml.format(:id => f[:id], :name => f[:name], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height])
						else
							fcpxml.format(:id => f[:id], :name => f[:name], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height], :paspH => f[:paspH], :paspV => f[:paspV])
						end
					end
					# Assets
					@assets.each_with_index do |a, i|
						#puts a.inspect
						case a[:type]
						when "av"
							fcpxml.asset(:id => a[:id], :format =>a[:format_id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasVideo => "1", :hasAudio => "1", :audioSources => a[:audioSources], :audioChannels => a[:audioChannels], :audioRate => a[:samplerate])
						when "a"
							fcpxml.asset(:id => a[:id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasAudio => "1", :audioSources => a[:audioSources], :audioChannels => a[:audioChannels], :audioRate => a[:samplerate]) # no format id
						when "v", "psd", "img"
							if a[:stillframe]
								fcpxml.asset(:id => a[:id], :src => a[:src], :duration => "0s", :start => "0s", :hasVideo => "1") # no format id
							else
								fcpxml.asset(:id => a[:id], :format =>a[:format_id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasVideo => "1")
							end
						else # when could type be ""?
							fcpxml.asset(:id => a[:id], :format =>a[:format_id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]))
						end
						unless a[:aux_tc].empty?
							query = "//resources/asset"
							Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last) do |asset|
								asset.metadata {
									asset.md(:key => "com.apple.proapps.custom.soundroll", :displayName => "Sound Roll", :value => a[:aux_tc].first, :type => "string", :editable => "1",  :source => "custom source")
									asset.md(:key => "com.apple.proapps.custom.soundtc", :displayName => "Aux TC", :value => a[:aux_tc].last, :type => "string", :editable => "1",  :source => "custom source")
								}
							end
						end
					end
					# Multicams
					@multiclips.each_with_index do |multiclip, m|
						fcpxml.media(:id => multiclip[:id], :name => multiclip[:name]) {
							fcpxml.multicam(:format => multiclip[:format_id], :tcFormat => multiclip[:tcFormat], :tcStart => to_rational_seconds(multiclip[:frame], multiclip[:frameDuration])) {
								multiclip[:angles].each do |a|
									unless a.nil?
										fcpxml.send("mc-angle", :name => a[:name], :angleID => a[:id]+"angle") {
											fcpxml.gap(:duration => to_rational_seconds(a[:mediadelay], a[:frameDuration])) unless a[:mediadelay]==0
											fcpxml.clip(:name => a[:name], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :offset => to_rational_seconds(a[:mediadelay], a[:frameDuration]), :tcFormat => a[:tcFormat]) {
												case a[:type]
												when "v"
													fcpxml.video(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]))
												when "a"
													if a[:channels]=="stereo" # one or more stereo tracks
														fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration])) {
															unless a[:audioSources]==1
																a[:audioSources].downto(2) do |src|
																	fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-src, :srcID => src)
																end
															end
														}
													elsif a[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
														fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
														}
													# elsif a[:channels]=="surround"
													else # one or more mono tracks
														fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :srcCh => 1 ) {
															unless a[:channelcount]==1
																if a[:audioSources]==1 # no srcID because audioSources==1
																	a[:audioChannels].downto(2) do |ch|
																		fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-ch, :srcCh => ch )
																	end
																elsif a[:audioChannels]==1 || a[:audioChannels]==2
																	a[:audioSources].downto(2) do |src|
																		srcCh = "1"
																		unless a[:srcChs][src-1].nil?
																			srcCh = a[:srcChs][src-1]
																		end
																		fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
																	end
																end
															end
														}
													end
												when "av"
													fcpxml.video(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration])) {
														if a[:channels]=="stereo" # one or more stereo tracks
															a[:audioSources].downto(1) do |src|
																if src==1 # no srcID
																	fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1)
																else
																	fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-src, :srcID => src)
																end
															end
														elsif a[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
														# elsif a[:channels]=="surround"
														else # one or more mono tracks
															if a[:audioSources]==1 # no srcID because audioSources==1
																a[:audioChannels].downto(1) do |ch|
																	fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-ch, :srcCh => ch )
																end
															elsif a[:audioChannels]==1 || a[:audioChannels]==2
																a[:audioSources].downto(1) do |src|
																	srcCh = "1"
																	unless a[:srcChs][src-1].nil?
																		srcCh = a[:srcChs][src-1]
																	end
																	fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
																end
															end
														end
													}
												end
											}
										}
									end
								end
							}
						}
					end
					# Compound Clips - add each nested sequence as a reference compound clip
					#puts "working on #{@compound_clips.length} compound clips"
					@nested_seqs.each do |nested_seq|
						#puts nested_seq[:name]
						cc_frameDuration = nested_seq[:frameDuration]
						fcpxml.media(:id => nested_seq[:nested_ref_id], :name => nested_seq[:name]) {
							fcpxml.sequence(:format => nested_seq[:format_id], :duration => to_rational_seconds(nested_seq[:duration], cc_frameDuration), :tcFormat => nested_seq[:nested_seq_tcFormat], :audioLayout => nested_seq[:nested_seq_audioLayout], :tcStart => to_rational_seconds(nested_seq[:frame], cc_frameDuration)) { # audioRate is expressed as 48k instead of 48000
								# if stereo, fcpxml.parent["layout"] = "stereo"
								fcpxml.note nested_seq[:metadata] unless nested_seq[:metadata].empty?
								fcpxml.spine {
									clip_array = nested_seq[:nested_array]
									last_clip_index = clip_array.first.length - 1
									if clip_array.first.empty?
										last_clip_index = 0
									else
										if clip_array.first[last_clip_index][:type]=="gap"
											last_clip_index -= 1
										end
										if clip_array.first[last_clip_index][:type]=="transition"
											last_clip_index -= 1
										end
									end
									#puts "#{clip_array.first.length} primary storyline clips"
									clip_array.first.each_with_index do |primary_clip, p| # clip_array.first is the V1 track, which becomes the primary storyline
										@error_tracker = ":\n'#{nested_seq[:name]}' Clip '#{primary_clip[:name]}'"
										storyline_builder(fcpxml, [primary_clip], nested_seq[:frame], nested_seq[:frame], cc_frameDuration, nested_seq[:height], true) # node, clips, adj, frame, frameDuration, height, primary=false
										case primary_clip[:type] # also check case nested_primary_clip[:type]
										when "title"
											primary_start = (3600 * 1/cc_frameDuration).round
										when "gap"
											primary_start = (3600 * 1/cc_frameDuration).round
										when "generator"
											primary_start = (3600 * 1/cc_frameDuration).round
										when "transition"
											# ignore - can't be connected to
										when "mc"
											primary_start = (primary_clip[:mc_frame] / primary_clip[:speed_adj]) + primary_clip[:in] # bugfix for mc
										when "nested"
											primary_start = (primary_clip[:frame] / primary_clip[:speed_adj]) + primary_clip[:in] # WAS nested_seq[:frame]
										else # clip
											# refactor the primary clip's first frame to the sequences frame rate (Bug fix for clips and secondary storylines connected to a primary clip whose frame rate is different from the timeline's frame rate)
											refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/cc_frameDuration)
											primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
										end
										primary_offset = primary_clip[:start]
										path_to_primary_spine = ["spine"]
										fcpxml.parent.ancestors.each { |a| path_to_primary_spine << a.name unless a.name=="document" }
										query = path_to_primary_spine.reverse.join("/")
										#puts "query = #{query}"
										#puts "************** primary_start = #{primary_start}", "primary_offset = #{primary_offset}"
										@compound_clip_name = nested_seq[:name]
										Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last.element_children.last) do |primary_node| # the primary_node is the fcpxnl's last spine's last element_children node
											#puts "primary_node: #{primary_node.to_xml}"
											unless primary_clip[:array_of_connecteds].nil? || primary_clip[:type]=="transition"
												# attach connected clips or secondary spines to the last primary clip
												# if primary_clip[:frameDuration].nil?
												# 	pc_frameDuration = cc_frameDuration
												# else
												# 	pc_frameDuration = primary_clip[:frameDuration]
												# end
												# puts "pc_frameDuration = #{pc_frameDuration}"
												primary_clip[:array_of_connecteds].each do |connecteds|
													unless connecteds.empty?
														if connecteds.length==1 && connecteds.first[:type]=="transition"
															#puts "CONNECTING A SINGLE TRANSITION !!!!!!"
														end
														#puts "************** primary_start - primary_offset + connecteds.first[:start] = #{primary_start} - #{primary_offset} + #{connecteds.first[:start]}"
														if connecteds.length==1 && connecteds.first[:type]!="transition" # connected clip
															storyline_builder(primary_node, connecteds, primary_start - primary_offset, nested_seq[:frame], cc_frameDuration, nested_seq[:height]) # node, clips, adj, frame, frameDuration, height, primary=false
														elsif connecteds.length > 1 # secondary storyline
															primary_node.spine(:lane => connecteds.first[:lane], :offset => to_rational_seconds(primary_start - primary_offset + connecteds.first[:start], cc_frameDuration)) {
																storyline_builder(primary_node, connecteds, connecteds.first[:start]*-1, nested_seq[:frame], cc_frameDuration, nested_seq[:height])
															}
														end
													end
												end
											end
											# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
											# pp primary_clip
											unless primary_start.nil? || primary_clip[:type]=="transition"
												if p==last_clip_index
													markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], primary_clip[:tail_markers], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], cc_frameDuration)
												else
													markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], [], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], cc_frameDuration)
												end
												markers.each { |m| primary_node << m }
												chmarkers.each { |m| primary_node << m }
											end
											# add_intrinsic_params_audio(primary_node, primary_clip)
											add_filters(primary_node, primary_clip) unless primary_clip[:filters].nil? || primary_clip[:type]=="mc"
											if primary_clip[:text]!=nil && $dtd=="1.2" # 1.3 is already dealt with in storyline_builder
												primary_clip[:text].each { |txt| primary_node.text_(txt) }
											end
											unless primary_clip[:reel_name].nil? && primary_clip[:scene].nil? && primary_clip[:shottake].nil? # transitions, slugs etc.
												primary_node.metadata {
													#puts "primary_clip[:reel_name]", primary_clip[:reel_name].inspect
													# FIXME unless primary_clip[:reel_name].empty?
													primary_node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => primary_clip[:reel_name])
													primary_node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => primary_clip[:scene])
													primary_node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => primary_clip[:shottake])
													primary_node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => primary_clip[:label])
												}
											end
										end
									end
								}
							}
						}
					end # end @nested_seqs.each
					# Transitions
					fcpxml.effect(:id => "e1", :name => "Cross Dissolve", :uid => "FxPlug:4731E73A-8DAC-4113-9A30-AE85B1761265")
					fcpxml.effect(:id => "ea1", :name => "Audio Crossfade", :uid => "FFAudioTransition")
					fcpxml.effect(:id => "e2", :name => "Center", :uid => "FxPlug:539C8A29-BBED-4670-B774-338109A7DB68")
					fcpxml.effect(:id => "e3", :name => "Squares", :uid => ".../Transitions.localized/Objects.localized/Squares.localized/Squares.motr")
					fcpxml.effect(:id => "e4", :name => "Zoom", :uid => ".../Transitions.localized/Blurs.localized/Zoom.localized/Zoom.motr")
					fcpxml.effect(:id => "e5", :name => "Cube", :uid => "FxPlug:653CCCCD-6CB1-4602-B9B3-181B1C53E069")
					fcpxml.effect(:id => "e6", :name => "Flip", :uid => ".../Transitions.localized/Movements.localized/Flip.localized/Flip.motr")
					fcpxml.effect(:id => "e7", :name => "Swing", :uid => ".../Transitions.localized/Movements.localized/Swing.localized/Swing.motr")
					fcpxml.effect(:id => "e8", :name => "Fade To Color", :uid => "FxPlug:F779C565-486D-4633-8035-0374B4DB8F5C")
					fcpxml.effect(:id => "e9", :name => "Ripple", :uid => "FxPlug:D386C0D5-5BD3-4A0D-86FC-6B9878C76451")
					fcpxml.effect(:id => "e10", :name => "Letter X", :uid => "FxPlug:743DCA2A-B49F-4B19-8A52-B9575D631139")
					fcpxml.effect(:id => "e11", :name => "Circle", :uid => "FxPlug:1C52AC71-7116-4248-B51F-5F5641EA9EDD")
					fcpxml.effect(:id => "e12", :name => "Inset Wipe", :uid => "FxPlug:5F0E1FA2-C172-4AFF-88CB-64B0F08195E4")
					fcpxml.effect(:id => "e13", :name => "Wipe", :uid => "FxPlug:857E2FBA-98DB-411B-A88C-CE6ABC1F65D8")
					fcpxml.effect(:id => "e14", :name => "Page Curl", :uid => "FxPlug:21B9D89C-4162-43FF-BA07-B019989E4940")
					fcpxml.effect(:id => "e15", :name => "Gradient Image", :uid => "FxPlug:62199A78-FDBF-4E9B-BB25-9A1D62FD2D48")
					fcpxml.effect(:id => "e16", :name => "Mosaic", :uid => "FxPlug:66B87635-119C-40B2-BF6E-537A3165FB95")
					fcpxml.effect(:id => "e17", :name => "Clock", :uid => "FxPlug:B2C3F87B-2A21-4E13-8173-46ED4FEBC57A")
					fcpxml.effect(:id => "e18", :name => "Slide", :uid => "FxPlug:6AAB0D54-FCD8-4EBD-A62D-D352A5ED1648")
					fcpxml.effect(:id => "e19", :name => "Spin", :uid => "FxPlug:196B9DB2-28FD-420F-9BA1-3F0E9EEBBAAA")
					fcpxml.effect(:id => "e20", :name => "Band", :uid => "FxPlug:8AD995CC-8DD6-4652-A3D2-B1468DCC6203")
					fcpxml.effect(:id => "e21", :name => "Black Hole", :uid => ".../Transitions.localized/Movements.localized/Black Hole.localized/Black Hole.motr")
					# Slug and Generators
					fcpxml.effect(:id => "g1", :name => "Custom", :uid => ".../Generators.localized/Solids.localized/Custom.localized/Custom.motn")
					fcpxml.effect(:id => "g2", :name => "Gradient", :uid => ".../Generators.localized/Textures.localized/Gradient.localized/Gradient.motn")
					fcpxml.effect(:id => "g3", :name => "Shapes", :uid => ".../Generators.localized/Elements.localized/Shapes.localized/Shapes.motn")
					# Titles
					fcpxml.effect(:id => "t1", :name => "Custom", :uid => ".../Titles.localized/Build In:Out.localized/Custom.localized/Custom.moti")
					fcpxml.effect(:id => "t2", :name => "Middle", :uid => ".../Titles.localized/Lower Thirds.localized/Middle.localized/Middle.moti")
					fcpxml.effect(:id => "t3", :name => "Scrolling", :uid => ".../Titles.localized/Credits.localized/Scrolling.localized/Scrolling.moti")
					fcpxml.effect(:id => "t4", :name => "Ticker", :uid => ".../Titles.localized/Build In:Out.localized/Ticker.localized/Ticker.moti")
					fcpxml.effect(:id => "t5", :name => "Typewriter", :uid => ".../Titles.localized/Build In:Out.localized/Typewriter.localized/Typewriter.moti")
					# Video Filters
					fcpxml.effect(:id => "f1", :name => "Censor", :uid => ".../Effects.localized/Stylize.localized/Censor.localized/Censor.moef")
					fcpxml.effect(:id => "f2", :name => "Cartoon", :uid => ".../Effects.localized/Stylize.localized/Cartoon.localized/Cartoon.moef")
					fcpxml.effect(:id => "f3", :name => "Directional", :uid => ".../Effects.localized/Blur.localized/Directional.localized/Directional.moef")
					fcpxml.effect(:id => "f4", :name => "Gaussian", :uid => ".../Effects.localized/Blur.localized/Gaussian.localized/Gaussian.moef")
					fcpxml.effect(:id => "f5", :name => "Prism", :uid => ".../Effects.localized/Blur.localized/Prism.localized/Prism.moef")
					fcpxml.effect(:id => "f6", :name => "Radial", :uid => ".../Effects.localized/Blur.localized/Radial.localized/Radial.moef")
					fcpxml.effect(:id => "f7", :name => "Zoom", :uid => ".../Effects.localized/Blur.localized/Zoom.localized/Zoom.moef")
					fcpxml.effect(:id => "f8", :name => "Mask", :uid => ".../Effects.localized/Keying.localized/Mask.localized/Mask.moef")
					fcpxml.effect(:id => "f9", :name => "Simple Border", :uid => ".../Effects.localized/Stylize.localized/Simple Border.localized/Simple Border.moef")
					fcpxml.effect(:id => "f10", :name => "Droplet", :uid => ".../Effects.localized/Distortion.localized/Droplet.localized/Droplet.moef")
					fcpxml.effect(:id => "f11", :name => "Earthquake", :uid => ".../Effects.localized/Distortion.localized/Earthquake.localized/Earthquake.moef")
					fcpxml.effect(:id => "f12", :name => "Fisheye", :uid => ".../Effects.localized/Distortion.localized/Fisheye.localized/Fisheye.moef")
					fcpxml.effect(:id => "f13", :name => "Fun House", :uid => ".../Effects.localized/Distortion.localized/Fun House.localized/Fun House.moef")
					fcpxml.effect(:id => "f14", :name => "Insect Eye", :uid => ".../Effects.localized/Distortion.localized/Insect Eye.localized/Insect Eye.moef")
					fcpxml.effect(:id => "f15", :name => "Scrape", :uid => ".../Effects.localized/Distortion.localized/Scrape.localized/Scrape.moef")
					fcpxml.effect(:id => "f16", :name => "Wave", :uid => ".../Effects.localized/Distortion.localized/Wave.localized/Wave.moef")
					fcpxml.effect(:id => "f17", :name => "Glow", :uid => ".../Effects.localized/Light.localized/Glow.localized/Glow.moef")
					fcpxml.effect(:id => "f18", :name => "Bloom", :uid => ".../Effects.localized/Light.localized/Bloom.localized/Bloom.moef")
					fcpxml.effect(:id => "f19", :name => "Hue/Saturation", :uid => ".../Effects.localized/Basics.localized/Hue:Saturation.localized/Hue:Saturation.moef")
					fcpxml.effect(:id => "f20", :name => "Sepia", :uid => ".../Effects.localized/Basics.localized/Sepia.localized/Sepia.moef")
					fcpxml.effect(:id => "f21", :name => "Tint", :uid => ".../Effects.localized/Basics.localized/Tint.localized/Tint.moef")
					fcpxml.effect(:id => "f22", :name => "Broadcast Safe", :uid => ".../Effects.localized/Basics.localized/Broadcast Safe.localized/Broadcast Safe.moef")
					fcpxml.effect(:id => "f23", :name => "Keyer", :uid => "FxPlug:41122549-B8A6-470E-94DA-211294D20B62")
					fcpxml.effect(:id => "f24", :name => "Luma Keyer", :uid => "FxPlug:7E9178C5-7B0F-4B86-884D-FE79F568B6CE")
					fcpxml.effect(:id => "f25", :name => "Image Mask", :uid => ".../Effects.localized/Keying.localized/Image Mask.localized/Image Mask.moef")
					fcpxml.effect(:id => "f26", :name => "Crop &amp; Feather", :uid => ".../Effects.localized/Distortion.localized/Crop &amp; Feather.localized/Crop &amp; Feather.moef")
					fcpxml.effect(:id => "f27", :name => "Letterbox", :uid => ".../Effects.localized/Stylize.localized/Letterbox.localized/Letterbox.moef")
					fcpxml.effect(:id => "f28", :name => "Flipped", :uid => ".../Effects.localized/Distortion.localized/Flipped.localized/Flipped.moef")
					fcpxml.effect(:id => "f29", :name => "Mirror", :uid => ".../Effects.localized/Distortion.localized/Mirror.localized/Mirror.moef")
					fcpxml.effect(:id => "f30", :name => "Sharpen", :uid => ".../Effects.localized/Blur.localized/Sharpen.localized/Sharpen.moef")
					fcpxml.effect(:id => "f31", :name => "Add Noise", :uid => ".../Effects.localized/Stylize.localized/Add Noise.localized/Add Noise.moef")
					fcpxml.effect(:id => "f32", :name => "Aged Film", :uid => ".../Effects.localized/Stylize.localized/Aged Film.localized/Aged Film.moef")
					fcpxml.effect(:id => "f33", :name => "Bad TV", :uid => ".../Effects.localized/Stylize.localized/Bad TV.localized/Bad TV.moef")
					fcpxml.effect(:id => "f34", :name => "Pixellate", :uid => ".../Effects.localized/Stylize.localized/Pixellate.localized/Pixellate.moef")
					fcpxml.effect(:id => "f35", :name => "Halftone", :uid => ".../Effects.localized/Stylize.localized/Halftone.localized/Halftone.moef")
					fcpxml.effect(:id => "f36", :name => "Cross Hatch", :uid => ".../Effects.localized/Stylize.localized/Cross Hatch.localized/Cross Hatch.moef")
					fcpxml.effect(:id => "f37", :name => "Graphic", :uid => ".../Effects.localized/Stylize.localized/Graphic.localized/Graphic.moef")
					fcpxml.effect(:id => "f38", :name => "Tile", :uid => ".../Effects.localized/Tiling.localized/Tile.localized/Tile.moef")
					fcpxml.effect(:id => "f39", :name => "Vignette", :uid => ".../Effects.localized/Stylize.localized/Vignette.localized/Vignette.moef")
					fcpxml.effect(:id => "f40", :name => "Kaleidoscope", :uid => ".../Effects.localized/Tiling.localized/Kaleidoscope.localized/Kaleidoscope.moef")
					fcpxml.effect(:id => "f41", :name => "Trails", :uid => ".../Effects.localized/Distortion.localized/Trails.localized/Trails.moef")
					fcpxml.effect(:id => "f42", :name => "Flashing", :uid => ".../Effects.localized/Light.localized/Flashing.localized/Flashing.moef")
					fcpxml.effect(:id => "f43", :name => "Camcorder", :uid => ".../Effects.localized/Stylize.localized/Camcorder.localized/Camcorder.moef")
					fcpxml.effect(:id => "f44", :name => "Kaleidotile", :uid => ".../Effects.localized/Tiling.localized/Kaleidotile.localized/Kaleidotile.moef")
					fcpxml.effect(:id => "f45", :name => "Black & White", :uid => ".../Effects.localized/Basics.localized/Black & White.localized/Black & White.moef")
					fcpxml.effect(:id => "f124", :name => "Drop Shadow",  :uid => "FxPlug:9C13F991-BC99-4DC8-B150-381D7CCE183B")
					# Audio Filters
					fcpxml.effect(:id => "f100", :name => "AUBandpass", :uid => "AudioUnit: 0x61756678627061736170706c")
					fcpxml.effect(:id => "f101", :name => "AUDelay", :uid => "AudioUnit: 0x6175667864656c796170706c")
					fcpxml.effect(:id => "f102", :name => "AUDistortion", :uid => "AudioUnit: 0x61756678646973746170706c")
					fcpxml.effect(:id => "f103", :name => "AUDynamicsProcessor", :uid => "AudioUnit: 0x6175667864636d706170706c")
					fcpxml.effect(:id => "f104", :name => "AUFilter", :uid => "AudioUnit: 0x6175667866696c746170706c")
					fcpxml.effect(:id => "f105", :name => "AUGraphicEQ", :uid => "AudioUnit: 0x61756678677265716170706c")
					fcpxml.effect(:id => "f106", :name => "AUHighShelfFilter", :uid => "AudioUnit: 0x61756678687368666170706c")
					fcpxml.effect(:id => "f107", :name => "AUHipass", :uid => "AudioUnit: 0x61756678687061736170706c")
					fcpxml.effect(:id => "f108", :name => "AULowpass", :uid => "AudioUnit: 0x617566786c7061736170706c")
					fcpxml.effect(:id => "f109", :name => "AULowShelfFilter", :uid => "AudioUnit: 0x617566786c7368666170706c")
					fcpxml.effect(:id => "f110", :name => "AUMultibandCompressor", :uid => "AudioUnit: 0x617566786d636d706170706c")
					fcpxml.effect(:id => "f111", :name => "AUNetSend", :uid => "AudioUnit: 0x617566786e736e646170706c")
					fcpxml.effect(:id => "f112", :name => "AUParametricEQ", :uid => "AudioUnit: 0x61756678706d65716170706c")
					fcpxml.effect(:id => "f113", :name => "AUPeakLimiter", :uid => "AudioUnit: 0x617566786c6d74726170706c")
					fcpxml.effect(:id => "f114", :name => "AUPitch", :uid => "AudioUnit: 0x61756678746d70746170706c")
					fcpxml.effect(:id => "f115", :name => "AURogerBeep", :uid => "AudioUnit: 0x61756678726f67726170706c")
					fcpxml.effect(:id => "f116", :name => "AUSampleDelay", :uid => "AudioUnit: 0x6175667873646c796170706c")
					fcpxml.effect(:id => "f117", :name => "Channel EQ", :uid => "AudioUnit: 0x61756678000000ec454d4147")
					fcpxml.effect(:id => "f118", :name => "Echo Delay", :uid => "Echo Delay.Echo.audio.effectBundle")
					fcpxml.effect(:id => "f119", :name => "Noise Gate", :uid => "AudioUnit: 0x61756678000000b3454d4147")
					fcpxml.effect(:id => "f120", :name => "Gain", :uid => "AudioUnit: 0x61756678000000b7454d4147")
					fcpxml.effect(:id => "f121", :name => "AUMatrixReverb", :uid => "AudioUnit: 0x617566786d7265766170706c")
					fcpxml.effect(:id => "f122", :name => "DeEsser", :uid => "AudioUnit: 0x61756678000000a0454d4147")
					fcpxml.effect(:id => "f123", :name => "Compressor", :uid => "AudioUnit: 0x617566780000009a454d4147")
				}
				good = "Favorite"
				if @goodPopupMenu.nil? || @goodPopupMenu.titleOfSelectedItem=="Favorite"
					# Favorite
				elsif @goodPopupMenu.titleOfSelectedItem=='"Good" Keyword'
					good = "Good"
				else
					good = "Circle Take"
				end
				bins = Array.new
				fcpxml.library {
					fcpxml.event(:name => @event[:name]) {
						### add each sequence as a project/sequence
						@project_seqs.each do |project_seq|
							#puts project_seq[:name]
							seq_frameDuration = project_seq[:frameDuration]
							if project_seq[:merged]
								# add clip/spine
							else
								# add project/sequence/spine
							end
							fcpxml.project(:id => project_seq[:id], :name => project_seq[:name]) {
								fcpxml.sequence(:format => project_seq[:format_id], :duration => to_rational_seconds(project_seq[:duration], seq_frameDuration), :tcFormat => project_seq[:tcFormat], :audioLayout => project_seq[:audioLayout], :tcStart => to_rational_seconds(project_seq[:frame], seq_frameDuration)) { # audioRate is expressed as 48k instead of 48000
									unless project_seq[:keywords].empty?
										fcpxml.parent["keywords"] = project_seq[:keywords]
									end
									fcpxml.note project_seq[:metadata] unless project_seq[:metadata].empty?
									fcpxml.spine {
										clip_array = project_seq[:clip_array]
										last_clip_index = clip_array.first.length - 1
										if clip_array.first.empty?
											last_clip_index = 0
										else
											if clip_array.first[last_clip_index][:type]=="gap"
												last_clip_index -= 1
											end
											if clip_array.first[last_clip_index][:type]=="transition"
												last_clip_index -= 1
											end
										end
										#puts "#{clip_array.first.length} primary storyline clips"
										clip_array.first.each_with_index do |primary_clip, p| # clip_array.first is the V1 track, which becomes the primary storyline
											@error_tracker = ":\n'#{project_seq[:name]}' Clip '#{primary_clip[:name]}'"
											#puts @error_tracker
											storyline_builder(fcpxml, [primary_clip], project_seq[:frame], project_seq[:frame], seq_frameDuration, project_seq[:height], true) # node, clips, adj, frame, frameDuration, height, primary=false
											#puts "primary_clip[:type] = #{primary_clip[:type]}"
											case primary_clip[:type] # also check case nested_primary_clip[:type]
											when "title"
												primary_start = (3600 * 1/seq_frameDuration).round
											when "gap"
												primary_start = (3600 * 1/seq_frameDuration).round
											when "generator"
												primary_start = (3600 * 1/seq_frameDuration).round
											when "transition"
												# ignore - can't be connected to
											when "mc"
												primary_start = (primary_clip[:mc_frame] / primary_clip[:speed_adj]) + primary_clip[:in] # bugfix for mc
											when "nested"
												primary_start = (primary_clip[:frame] / primary_clip[:speed_adj]) + primary_clip[:in]
												#puts "nested, primary_start = (#{primary_clip[:frame]} / #{primary_clip[:speed_adj]}) + #{primary_clip[:in]} = #{primary_start}"
											else # clip
												# refactor the primary clip's first frame to the sequences frame rate (Bug fix for clips and secondary storylines connected to a primary clip whose frame rate is different from the timeline's frame rate)
												refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/seq_frameDuration)
												primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
											end
											primary_offset = primary_clip[:start]
											path_to_primary_spine = ["spine"]

											fcpxml.parent.ancestors.each { |a| path_to_primary_spine << a.name unless a.name=="document" }
											query = path_to_primary_spine.reverse.join("/")
											#puts "query = #{query}"
											#puts "************** primary_start = #{primary_start}", "primary_offset = #{primary_offset}"
											@project_seq_name = project_seq[:name]
											Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last.element_children.last) do |primary_node| # the primary_node is the fcpxnl's last spine's last element_children node
												unless primary_clip[:array_of_connecteds].nil? || primary_clip[:type]=="transition"
													# attach connected clips or secondary spines to the last primary clip
													# if primary_clip[:frameDuration].nil?
													# 	pc_frameDuration = seq_frameDuration
													# else
													# 	pc_frameDuration = primary_clip[:frameDuration]
													# end
													# puts "pc_frameDuration = #{pc_frameDuration}"
													primary_clip[:array_of_connecteds].each do |connecteds|
														connecteds.compact! # bug fix for nil in the array
														if connecteds.empty?
															#puts "connecteds is empty"
														else
															if connecteds.length==1 && connecteds.first[:type]=="transition"
																#puts "CONNECTING A SINGLE TRANSITION !!!!!!"
															end
															#puts "************** primary_start - primary_offset + connecteds.first[:start] = #{primary_start} - #{primary_offset} + #{connecteds.first[:start]}"
															if connecteds.length==1 && connecteds.first[:type]!="transition" # connected clip
																storyline_builder(primary_node, connecteds, primary_start - primary_offset, project_seq[:frame], seq_frameDuration, project_seq[:height]) # node, clips, adj, frame, frameDuration, height, primary=false
															elsif connecteds.length > 1 # secondary storyline
																primary_node.spine(:lane => connecteds.first[:lane], :offset => to_rational_seconds(primary_start - primary_offset + connecteds.first[:start], seq_frameDuration)) {
																	storyline_builder(primary_node, connecteds, connecteds.first[:start]*-1, project_seq[:frame], seq_frameDuration, project_seq[:height])
																}
															end
														end
													end
												end
												# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
												#pp primary_clip
												unless primary_start.nil? || primary_clip[:type]=="transition"
													if p==last_clip_index
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], primary_clip[:tail_markers], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], seq_frameDuration)
													else
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], [], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], seq_frameDuration)
													end
													markers.each { |m| primary_node << m }
													chmarkers.each { |m| primary_node << m }
												end
												add_intrinsic_params_audio(primary_node, primary_clip)
												add_filters(primary_node, primary_clip) unless primary_clip[:filters].nil? || primary_clip[:type]=="mc"
												unless primary_clip[:reel_name].nil? && primary_clip[:scene].nil? && primary_clip[:shottake].nil? # transitions, slugs etc.
													primary_node.metadata {
														#puts "primary_clip[:reel_name]", primary_clip[:reel_name].inspect
														primary_node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => primary_clip[:reel_name]) unless primary_clip[:reel_name].empty?
														primary_node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => primary_clip[:scene]) unless primary_clip[:scene].empty?
														primary_node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => primary_clip[:shottake]) unless primary_clip[:shottake].empty?
														primary_node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => primary_clip[:label]) unless primary_clip[:label].empty?
													}
												end
											end
										end
									} # end fcpxml.spine
								} # end fcpxml.sequence
							} # end fcpxml.project
						end # end @project_seqs.each
						### add each media asset as a clip(s) with keywords
						#puts "add each media asset as a clip(s) with keywords"
						@assets.each do |asset|
							unless asset[:type]=="mc" || asset[:type]=="psd" # FIXME
								if asset[:type]=="a"
									format_id = "ft1" # FFVideoFormatRateUndefined - don't add format to asset if it's audio-only
								else
									format_id = asset[:format_id]
								end
								asset[:clips].each do |c|
									fcpxml.clip(:name => c[:name], :duration => to_rational_seconds(c[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset], asset[:frameDuration]), :tcFormat => asset[:tcFormat], :format => format_id) {
										fcpxml.note c[:metadata] unless c[:metadata].nil?
										case asset[:type]
										when "v"
											fcpxml.video(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]))
										when "a"
											if asset[:channels]=="stereo" # one or more stereo tracks
												fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration])) {
													unless asset[:audioSources]==1
														asset[:audioSources].downto(2) do |src|
															fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 1-src, :srcID => src)
														end
													end
												}
											elsif asset[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
												fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
													fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
												}
											# elsif asset[:channels]=="surround"
											else # one or more mono tracks
												fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :srcCh => 1 ) {
													unless asset[:channelcount]==1
														if asset[:audioSources]==1 # no srcID because audioSources==1
															asset[:audioChannels].downto(2) do |ch|
																fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 1-ch, :srcCh => ch )
															end
														elsif asset[:audioChannels]==1 || asset[:audioChannels]==2
															asset[:audioSources].downto(2) do |src|
																srcCh = "1"
																unless asset[:srcChs][src-1].nil?
																	srcCh = asset[:srcChs][src-1]
																end
																fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
															end
														end
													end
												}
											end
										when "av"
											fcpxml.video(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration])) {
												if asset[:channels]=="stereo" # one or more stereo tracks
													asset[:audioSources].downto(1) do |src|
														if src==1 # no srcID
															fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -1)
														else
															fcpxml.audio(:offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 0-src, :srcID => src)
														end
													end
												elsif asset[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
													fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
													fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
												# elsif asset[:channels]=="surround"
												else # one or more mono tracks
													if asset[:audioSources]==1 # no srcID because audioSources==1
														asset[:audioChannels].downto(1) do |ch|
															fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 0-ch, :srcCh => ch )
														end
													elsif asset[:audioChannels]==1 || asset[:audioChannels]==2
														asset[:audioSources].downto(1) do |src|
															srcCh = "1"
															unless asset[:srcChs][src-1].nil?
																srcCh = asset[:srcChs][src-1]
															end
															fcpxml.audio( :ref => asset[:id], :duration => to_rational_seconds(asset[:duration], asset[:frameDuration]), :offset => to_rational_seconds(asset[:frame], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame], asset[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
														end
													end
												end
											}
										end
										unless c[:markers].nil? || c[:markers].empty?
											markers, chmarkers = order_markers(c[:markers], [], [], [], asset[:frame] + c[:subclip_startoffset], 1, c[:in], c[:out], asset[:frameDuration]) # first frame takes into account subclip_startoffset for subclips
											markers.each { |m| fcpxml << m }
											chmarkers.each { |m| fcpxml << m }
										end
										if c[:good]
											if good=="Favorite"
												if $dtd=="1.2"
													fcpxml.rating(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => "favorite")
												else
													fcpxml.rating(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => "favorite", :name => "Good")
												end
											else
												fcpxml.keyword(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => good)
											end
										end
										c[:favorites] = [] if c[:favorites].nil?
										if @seqClipNamesFavoritesCheckbox!=nil && @seqClipNamesFavoritesCheckbox.state==1
											c[:favorites].each do |f|
												if $dtd=="1.2"
												else
													fcpxml.rating(:start => to_rational_seconds(asset[:frame] + f[:in], asset[:frameDuration]), :duration => to_rational_seconds(f[:out] - f[:in], asset[:frameDuration]), :value => "favorite", :name => f[:name])
												end
											end
										end
										c[:bins].each do |b|
											fcpxml.keyword(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + b[:in], asset[:frameDuration]), :duration => to_rational_seconds(b[:out] - b[:in], asset[:frameDuration]), :value => b[:name]) # CHANGED removed b[:metadata]
											bins << b[:name]
										end
										fcpxml.metadata {
											fcpxml.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => c[:reel_name]) unless c[:reel_name].nil? || c[:reel_name].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => c[:scene]) unless c[:scene].nil? || c[:scene].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => c[:shottake]) unless c[:shottake].nil? || c[:shottake].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.angle", :editable => "1", :value => c[:defaultangle]) unless c[:defaultangle].nil? || c[:defaultangle].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => c[:label]) unless c[:label].nil? || c[:label].empty?
										}
									}
								end
							end
						end
						### add each merged clip as a clip with keywords
						@mergedclips.each_with_index do |mergeclip, m|
							vclip = mergeclip[:angles].shift
							aclips = mergeclip[:angles].compact
							if vclip.nil? # NOTE merged clips in Event are handled differently to merged clips in a Project
								print "skipping because vclip is nil:\n"
								#pp mergeclip
							else
								fcpxml.clip(:name => mergeclip[:name], :duration => to_rational_seconds(mergeclip[:duration], mergeclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame] + 0, mergeclip[:frameDuration]), :tcFormat => mergeclip[:tcFormat], :format => mergeclip[:format_id]) {
									fcpxml.spine {
										if mergeclip[:syncoffset] < 0
											gap_duration = to_rational_seconds(mergeclip[:syncoffset].abs, mergeclip[:frameDuration])
											fcpxml.gap(:offset => "0s", :start => "3600s", :duration => gap_duration)
										else
											gap_duration = "0s"
										end
										fcpxml.clip(:name => vclip[:name], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame] + 0, vclip[:frameDuration]), :offset => gap_duration, :tcFormat => vclip[:tcFormat], :format => vclip[:format_id]) {
											fcpxml.video(:offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration])) {
												if vclip[:type]=="av"
													if vclip[:channels]=="stereo"
														fcpxml.audio(:offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => -1)
													elsif vclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
														fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(a[:frame], vclip[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
														fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(a[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(a[:frame], vclip[:frameDuration]), :start => to_rational_seconds(a[:frame], vclip[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
													else # one or more mono tracks
														if vclip[:audioSources]==1 # no srcID because audioSources==1
															vclip[:audioChannels].downto(1) do |ch|
																fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => 0-ch, :srcCh => ch )
															end
														elsif vclip[:audioChannels]==1 || vclip[:audioChannels]==2
															vclip[:audioSources].downto(1) do |src|
																srcCh = "1"
																unless vclip[:srcChs][src-1].nil?
																	srcCh = vclip[:srcChs][src-1]
																end
																fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
															end
														end
													end
												end
											}
											aclips.each_with_index do |aclip, a|
												fcpxml.clip(:name => aclip[:name], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame] + 0, aclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame] + mergeclip[:syncoffset], vclip[:frameDuration]), :tcFormat => aclip[:tcFormat], :format => aclip[:format_id], :lane => "-#{a+1}") {
													if aclip[:channels]=="stereo"
														fcpxml.audio(:offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]))
														unless aclip[:audioSources]==1
															aclip[:audioSources].downto(2) do |src|
																fcpxml.audio(:offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-src, :srcID => src)
															end
														end
													elsif aclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
														fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(a[:frame], aclip[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
															fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(a[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(a[:frame], aclip[:frameDuration]), :start => to_rational_seconds(a[:frame], aclip[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
														}
													else # one or more mono tracks
														fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :srcID => 1 ) {
															unless aclip[:channelcount]==1
																if aclip[:audioSources]==1 # no srcID because audioSources==1
																	aclip[:audioChannels].downto(2) do |ch|
																		fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-ch, :srcCh => ch )
																	end
																elsif aclip[:audioChannels]==1 || aclip[:audioChannels]==2
																	aclip[:audioSources].downto(2) do |src|
																		srcCh = "1"
																		unless aclip[:srcChs][src-1].nil?
																			srcCh = aclip[:srcChs][src-1]
																		end
																		fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
																	end
																end
															end
														}
													end
												}
											end
										}
									}
									mergeclip[:bins].each do |b|
										fcpxml.keyword(:start => "0s", :duration => to_rational_seconds(mergeclip[:duration], mergeclip[:frameDuration]), :value => b[:name])
										bins << b[:name]
									end
								}
							end
						end
						### add each multiclip reference as an Event mc-clip
						@multiclips.each_with_index do |multiclip, m|
							fcpxml.send("mc-clip", :ref => multiclip[:id], :name => multiclip[:name], :duration => to_rational_seconds(multiclip[:duration], multiclip[:frameDuration])) {
								fcpxml.send("mc-source", :angleID => multiclip[:activevideoangle], :srcEnable => "all")
							}
						end
						### add each nested sequence as an Event ref-clip with keyword
						@nested_seqs.each do |nested_seq|
							seq_frameDuration = nested_seq[:frameDuration]
							fcpxml.send("ref-clip", :ref => nested_seq[:nested_ref_id], :name => nested_seq[:name], :duration => to_rational_seconds(nested_seq[:duration], seq_frameDuration), :start => to_rational_seconds(nested_seq[:frame], seq_frameDuration)) { # FIXME: ref-clip duration is wrong
								fcpxml.keyword(:value => "Sequences", :duration => to_rational_seconds(nested_seq[:duration], seq_frameDuration), :start => to_rational_seconds(nested_seq[:frame], seq_frameDuration))
							}
						end
						### add each keyword as a keyword-collection
						unless bins.empty?
							fcpxml.send("collection-folder", :name => "Bins") {
								bins.uniq.each do |bin|
									fcpxml.send("keyword-collection", :name => bin)
								end
							}
						end
					}
				}
			}
			# Fix up faulty posteroffset attribute
			fcpxml.doc.xpath("//chapter-marker[@posteroffset]").each do |cm|
				cm.delete("posteroffset")
				cm["posterOffset"] = "0s"
			end
		end
		save_this_file = File.new(new_xml_file, "w")
		save_this_file.puts builder.to_xml(:indent => 2, :encoding => "UTF-8")
		save_this_file.close
	rescue => e
		if $ruby_cocoa==false
			raise
		else
			OSX::NSLog "#{e.message}\n#{e.backtrace.join("\n")}"
      executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
			version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
			alert = OSX::NSAlert.alloc.init
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			alert.setMessageText("There was a problem creating the translated XML#{@error_tracker}")
			alert.setInformativeText("Please zip up the XML file and email it with the subject “#{executable} #{version} error” to 7toX@assistedediting.com with this error message in the body text:\n
			#{e.message}\n
			#{e.backtrace.first}\n
			Thanks for helping us improve #{executable}!")
			alert.runModal
		end
	end
end

def storyline_builder(node, clips, adj, frame, frameDuration, height, primary=false)
	#puts "storyline_builder: #{clips.length} clips, adj=#{adj}, frame =#{frame}"
	first_clip_index = 0
	if clips.first[:type]=="transition"
		first_clip_index = 1
	end
	last_clip_index = clips.length - 1
	if clips.last[:type]=="transition" || clips.last[:type]=="gap"
		last_clip_index -= 1
	end
	if primary==false && clips.length==1
		connected_clip = true
	else
		connected_clip = false
	end
	#puts "primary = #{primary}, first_clip_index = #{first_clip_index}, last_clip_index = #{last_clip_index}, connected_clip = #{connected_clip}, frameDuration = #{frameDuration}"
	clips.each_with_index do |clip, c|
		@error_tracker = ":\n'#{@project_seq_name}' Clip '#{clip[:name]}'"
		#puts "  #{clip[:name]} is #{clip[:type]}"
		#puts "reel_name = #{clip[:reel_name].inspect}, scene = #{clip[:scene].inspect}, shottake = #{clip[:shottake].inspect}"
		#pp clip
		clip[:in] = 0 if clip[:in].nil?
		clip[:out] = clip[:duration] if clip[:out].nil?
		if c==last_clip_index && clips.last[:type]=="transition" && primary==false # last transitions in secondary storyline don't need adjustment
			clip[:end] = clip[:real_end]
		end
		#puts "clip type = #{clip[:type]}"
		case clip[:type]
		when "gap"
			node.gap( :name => clip[:name], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => "3600s" )
		when "generator"
			node.video( :name => clip[:name], :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => "3600s", :enabled => clip[:enabled] ) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				if $dtd!="1.2" && clip[:rgb]!=nil
					node.param(:name => "Color", :value => clip[:rgb], :key => "9999/10008/10006/2/1/1")
				end
				add_intrinsic_params_video(node, clip)
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
				end
			}
		when "transition"
			if $dtd=="1.2"
				node.transition( :name => clip[:name], :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration) )
			else
				node.transition( :name => clip[:name], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration) ) {
					node.send("filter-video", :ref => clip[:ref])
					node.send("filter-audio", :ref => 'ea1') # both filter-video and filter-audio are present for video-only and audio-only transitions
				}
			end
		when "title"
			title_name = "#{clip[:text].first} - #{clip[:name]}"
			node.title( :name => title_name, :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => "3600s", :enabled => clip[:enabled] ) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				clip[:text] = [] if clip[:text].nil?
				if $dtd!="1.2" # <!ELEMENT title (param*, text*, text-style-def*, note?, %intrinsic-params-video;, (%anchor_item;)*, (%marker_item;)*, filter-video*, metadata?)>
					if clip[:ref]=="t2" # lower third...
						node.param( :name => "Build In", :key => "9999/10321/2/101", :value => "0")
						node.param( :name => "Build Out", :key => "9999/10321/2/102", :value => "0")
						if clip[:text].length==1 # ...with only 1 line of text
							node.text_ {
								node.send("text-style", "", :fontSize => 20*3)
							}
						end
					end
					current_id = @id
					clip[:text].each_with_index do |txt, i| # can have more than one text line and style
						unless txt.empty?
							ts = @text_styles[clip[:ts_id]+i]
							@id += 1
							if ts.nil? || ts[:font].nil?
								node.text_ { # use underscore to disambiguate tag names from ruby methods
									node.send("text-style", txt, :fontSize => 20*3) # 20pts is default FCP7 font size
								}
							else
								node.text_ {
									node.send("text-style", txt, :ref => "ts#{@id}")
								}
							end
						end
					end
					@id = current_id # reset before looping again
					clip[:text].each_with_index do |txt, i| # can have more than one text line and style
						unless txt.empty?
							ts = @text_styles[clip[:ts_id]+i]
							@id += 1
							if ts.nil? || ts[:font].nil?
							else # text-style-def nodes come after text nodes
								node.send("text-style-def", :id => "ts#{@id}") {
									node.send("text-style", :font => ts[:font], :fontSize => ts[:fontSize], :fontColor => ts[:fontColor], :bold => ts[:bold], :italic => ts[:italic], :alignment => ts[:alignment], :lineSpacing => ts[:lineSpacing])
								}
							end
						end
					end
				end
				node.note clip[:metadata] unless clip[:metadata].empty?
				add_intrinsic_params_video(node, clip)
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
					if $dtd=="1.2" # <!ELEMENT title (note?, timeMap?, %intrinsic-params-video;, (%anchor_item;)*, (%marker_item;)*, (%video-filter;)*, metadata?, text*)>
						clip[:text].each { |txt| node.text_(txt) }
					end
				end
			}
		when "psd"
			primary_clip = clip[:nested_array].first.first # first item in the first track
			#puts "psd"
			#pp clip[:nested_array]
			#puts primary_clip[:duration], frameDuration
			if primary_clip[:type]=="gap"
				#puts "primary_clip is gap; changing"
				#puts "OR NOT..." if clip[:nested_array].first[1].nil?
				primary_clip = clip[:nested_array].first[1] unless clip[:nested_array].first[1].nil? # HB Trailer
			end
			#pp primary_clip
			if primary_clip.nil?
        # <gap name="Gap" offset="43292249/12000s" duration="13013/1600s" start="3600s">
				# pp clip
				# puts "primary_clip is nil"; exit
			end
			node.clip( :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :enabled => clip[:enabled], :format => clip[:format_id] ) { # CHANGED from :format => primary_clip 1/13
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				add_intrinsic_params_video(node, clip)
				node.video( :role => primary_clip[:v_track], :name => primary_clip[:name], :duration => to_rational_seconds(primary_clip[:duration], frameDuration), :ref => primary_clip[:ref], :enabled => primary_clip[:enabled], :srcID => "1" ) {
					if primary_clip[:type]=="gap" || primary_clip[:ref].nil?
						node.parent.name = "gap"
						node.parent.remove_attribute("role")
						node.parent.remove_attribute("ref")
						node.parent.remove_attribute("enabled")
						node.parent.remove_attribute("srcID")
					end
					unless primary_clip[:basic_motion].nil?
						#puts primary_clip[:width].inspect, primary_clip[:height].inspect
						#puts primary_clip[:basic_motion].inspect
						#puts height.inspect
						primary_clip[:adj_conform] = "none"
						position = "#{primary_clip[:width].to_f * primary_clip[:basic_motion][:center_horiz] / height * 100} #{primary_clip[:height].to_f * primary_clip[:basic_motion][:center_vert] * -1 / height * 100}" # CHANGED to_f in case :width or :height is nil
						primary_clip[:basic_motion][:adj_transform_position] = position
					end
					add_intrinsic_params_video(node, primary_clip)
					unless primary_clip[:array_of_connecteds].nil?
						# attach photoshop layers to the primary clip
						primary_clip[:array_of_connecteds].compact.each_with_index do |layer, l|
							#puts "Layer #{l+2}"
							unless layer.empty? || primary_clip[:ref].nil?
								node.video( :name => layer.first[:name], :lane => l+1, :srcID => l+2,  :duration => to_rational_seconds(primary_clip[:duration], frameDuration), :ref => primary_clip[:ref], :enabled => layer.first[:enabled] ) {
									unless layer.first[:basic_motion].nil?
										#puts layer.first[:width].inspect, layer.first[:height].inspect, layer.first[:basic_motion][:center_horiz].inspect, height.inspect,  layer.first[:basic_motion][:center_vert].inspect
										layer.first[:adj_conform] = "none"
										position = "#{layer.first[:width].to_f * layer.first[:basic_motion][:center_horiz] / height * 100} #{layer.first[:height].to_f * layer.first[:basic_motion][:center_vert] * -1 / height * 100}" # recalc for each layer; to_f in case :width or :height is nil
										layer.first[:basic_motion][:adj_transform_position] = position
									end
									add_intrinsic_params_video(node, layer.first)
								}
							end
						end
					end
				}
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
			# NOTE: when there's a speed adjustment, the clip's start has to be altered, and the video's duration has to be altered in the opposite direction... then the marker start has to be altered in the same direction
		when "mc"
			node.send("mc-clip", :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:mc_frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :ref => clip[:mc_id]) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip) unless $dtd > "1.3"
				add_filters(node, clip, "a") unless $dtd > "1.3"
				node.send("mc-source", :angleID => clip[:activevideoangle], :srcEnable => "video") {
					add_intrinsic_params_video(node, clip) if $dtd > "1.3"
					add_filters(node, clip, "v")
				}
				node.send("mc-source", :angleID => clip[:activeaudioangle], :srcEnable => "audio")
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					# video filters were added to mc-source
					add_filters(node, clip, "a") if $dtd > "1.3"
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "l"
			node.video( :name => clip[:name], :role => clip[:v_track], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :ref => clip[:ref], :lane => clip[:lane], :srcID => clip[:lane]+1, :enabled => clip[:enabled] ) {
				clip[:adj_conform] = "none"
				clip[:basic_motion][:adj_transform_position] = "#{clip[:width].to_f * clip[:basic_motion][:center_horiz] / height * 100} #{clip[:height].to_f * clip[:basic_motion][:center_vert] * -1 / height * 100}" unless clip[:basic_motion].nil? # recalc for each layer; to_f in case :width or :height is nil
				add_intrinsic_params_video(node, clip)
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
				end
			}
		when "v"
			#puts "  adj = #{adj}, clip[:frame] = #{clip[:frame]}, clip[:end] = #{clip[:end]}, clip[:start] = #{clip[:start]}, frameDuration = #{frameDuration}, clip[:frameDuration] = #{clip[:frameDuration]}, clip[:in] = #{clip[:in]}"
			node.clip( :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
				# node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], frameDuration, clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
				node.parent["lane"] = "#{clip[:lane]}" if connected_clip && primary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip)
				node.video( :role => clip[:v_track], :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :ref => clip[:ref], :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]) )
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "a"
			#puts "frameDuration = #{frameDuration}, clip[:frameDuration] = #{clip[:frameDuration]}"
			node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip)
				if clip[:channels]=="stereo" # one or more stereo tracks
					node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track] ) {
						unless clip[:audioSources]==1
							#pp clip if clip[:audioSources].nil?
							clip[:audioSources].downto(2) do |src|
								node.audio(:offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 1-src, :srcID => src)
							end
						end
					}
				elsif clip[:channels]=="stereotomono" # dual mono from stereo (uses srcCh & outCh)
					node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :srcCh => 1, :outCh => "L" ) {
						node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => -1, :srcCh => 2, :outCh => "R" )
					}
				# elsif clip[:channels]=="surround"
				else # one or more mono tracks
					node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :srcCh => 1 ) {
						unless clip[:channelcount]==1
							if clip[:audioSources]==1 # no srcID because audioSources==1
								clip[:audioChannels].downto(2) do |ch|
									node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 1-ch, :srcCh => ch )
								end
							elsif clip[:audioChannels]==1 || clip[:audioChannels]==2
								clip[:audioSources].downto(2) do |src|
									srcCh = "1"
									unless clip[:srcChs][src-1].nil?
										srcCh = clip[:srcChs][src-1]
									end
									node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 1-src, :srcCh => srcCh, :srcID => src )
								end
							end
						end
					}
				end
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_intrinsic_params_audio(node, clip)
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "av" # could have audioDuration and audioStart
			# if connected_clip && clip[:name]=="A001C001_140702_R3VJ"
			# 	pc_frameDuration = Rational(1001,24000)
			# elsif connected_clip && clip[:name]=="F001C001_140702_R3VJ"
			# 	pc_frameDuration = Rational(1001,48000)
			# else
			# 	pc_frameDuration = frameDuration
			# end
			# puts "pc_frameDuration = #{pc_frameDuration}, clip[:frameDuration] = #{clip[:frameDuration]}" # use pc_frameDuration for :offset???
			#puts "  adj = #{adj}, clip[:frame] = #{clip[:frame]}, clip[:end] = #{clip[:end]}, clip[:start] = #{clip[:start]}, frameDuration = #{frameDuration}, clip[:frameDuration] = #{clip[:frameDuration]}, clip[:in] = #{clip[:in]}"
			#puts "    NEW offset = to_rational_seconds(#{clip[:start]} + #{adj}, #{clip[:frameDuration]}) = #{to_rational_seconds(clip[:start] + adj, clip[:frameDuration])}"
			#puts "clip[:srcs] = #{clip[:srcs].inspect}, clip[:channels] = #{clip[:channels]}"
			node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :audioDuration => to_rational_seconds(clip[:a_end]-clip[:a_start], frameDuration), :audioStart => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:a_in], frameDuration), :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip)
				node.video( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:v_track] ) {
					if clip[:channels]=="stereo"
						clip[:audioSources].downto(1) do |src|
							if src==1 # no srcID
								node.audio(:offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => -1)
							else
								node.audio(:offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 0-src, :srcID => src)
							end
						end
						# elsif clip[:channels]=="surround"
						# 	node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:a_end]-clip[:a_start], frameDuration), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:a_in], frameDuration), :format => clip[:format_id], :lane => "-1" ){
						# 		node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :outCh => "L" ){
						# 			node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => "-5", :outCh => "Rs", :srcID => 6 )
						# 			node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => "-4", :outCh => "Ls", :srcID => 5 )
						# 			node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => "-3", :outCh => "LFE", :srcID => 4 )
						# 			node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => "-2", :outCh => "C", :srcID => 3 )
						# 			node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => "-1", :outCh => "R", :srcID => 2 )
						# 		}
						# 	}
					elsif clip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
						node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => -2, :srcCh => 2, :outCh => "R" )
						node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => -1, :srcCh => 1, :outCh => "L" )
					else # one or more mono tracks
						if clip[:audioSources]==1 # no srcID because audioSources==1
							clip[:audioChannels].downto(1) do |ch|
								node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 0-ch, :srcCh => ch )
							end
						elsif clip[:audioChannels]==1 || clip[:audioChannels]==2
							clip[:audioSources].downto(1) do |src|
								srcCh = "1"
								unless clip[:srcChs][src-1].nil?
									srcCh = clip[:srcChs][src-1]
								end
								node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 0-src, :srcCh => srcCh, :srcID => src )
							end
						end
					end
				}
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_intrinsic_params_audio(node, clip)
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "mv", "mav"
			# puts clip[:frame], clip[:a_syncoffset], clip[:synced_audio].first[:a_frame], clip[:in], clip[:a_in]
			#puts "frameDuration = #{frameDuration}, clip[:frameDuration] = #{clip[:frameDuration]}, adj = #{adj}, "
			#puts "node.clip( :name => #{clip[:name]}, :duration => to_rational_seconds(#{clip[:end]} - #{clip[:start]}, #{frameDuration}), :offset => to_rational_seconds(#{clip[:start]} + #{adj}, #{frameDuration}), :start => to_rational_seconds(#{clip[:frame]} / #{clip[:speed_adj]}, #{clip[:frameDuration]}, #{clip[:in]}, #{frameDuration}), :enabled => #{clip[:enabled]}, :audioDuration => to_rational_seconds(#{clip[:a_end]}-#{clip[:a_start]}, #{frameDuration}), :audioStart => to_rational_seconds(#{clip[:frame]} / #{clip[:speed_adj]}, #{clip[:frameDuration]}, #{clip[:a_in]}, #{frameDuration}), :tcFormat => #{clip[:tcFormat]}, :format => #{clip[:format_id]} ) { "
			a_frame = clip[:synced_audio].first[:a_frame]
			node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :audioDuration => to_rational_seconds(clip[:a_end]-clip[:a_start], frameDuration), :audioStart => to_rational_seconds(a_frame / clip[:speed_adj], clip[:frameDuration], clip[:a_in], frameDuration), :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip)
				node.spine {
					node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
						node.video( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :role => clip[:v_track] ) {
							node.audio( :lane => -1, :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track] ) unless clip[:type]=="mv"
						}
						clip[:synced_audio].each_with_index do |aclip, a|
							if aclip[:a_frame]==0 # linked clip?
								#puts "it's a linked clip"
								clip_frame = clip[:frame]
								clip_start = clip[:a_syncoffset] * -1
							else # merged clip
								#puts "it's a merged clip"
								# if clip[:frame] < aclip[:a_frame]
								# 	clip_frame = aclip[:a_frame] - (aclip[:a_frame] - clip[:frame]) - clip[:a_syncoffset]
								# else
								# 	clip_frame = aclip[:a_frame] - (aclip[:a_frame] - clip[:frame]) + clip[:a_syncoffset]
								# end
								# clip_start = aclip[:a_frame]
							end
							if aclip[:type]=="generator" # missing
								node.video( :lane => "-1", :name => aclip[:name], :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], frameDuration), :offset => to_rational_seconds(aclip[:a_frame], frameDuration), :start => "3600s", :enabled => aclip[:enabled] )
							else
								node.clip(:name => aclip[:name], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] + 0, aclip[:frameDuration]), :offset => to_rational_seconds(clip[:frame] + aclip[:a_syncoffset], clip[:frameDuration]), :enabled => aclip[:enabled], :tcFormat => aclip[:a_tcFormat], :format => aclip[:a_format_id], :lane => "-#{a+1}") {
									if aclip[:channels]=="stereo"
										node.audio(:offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :role => aclip[:a_track])
									elsif aclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
										node.audio(:offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]),  :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
											node.audio(:offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
										}
									else # one or more mono tracks
										node.audio( :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :srcCh => 1 ) {
											unless aclip[:channelcount]==1
												if aclip[:audioSources]==1 # no srcID because audioSources==1
													aclip[:audioChannels].downto(2) do |ch|
														node.audio( :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :lane => 1-ch, :srcCh => ch )
													end
												elsif aclip[:audioChannels]==1 || aclip[:audioChannels]==2
													aclip[:audioSources].downto(2) do |src|
														srcCh = "1"
														unless aclip[:srcChs][src-1].nil?
															srcCh = aclip[:srcChs][src-1]
														end
														node.audio( :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
													end
												end
											end
										}
									end
								}
							end
						end
					}
				}
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_intrinsic_params_audio(node, clip)
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "nested"#, "psd"
			if clip[:type]=="psd"
				#puts "psd"
				clip_name = clip[:name]
			else
				#puts "nested sequence"
				clip_name = "#{clip[:name]} Compound Clip"
			end
			# pp clip
			#puts clip[:speed_adj].inspect
			#puts adj.inspect
			#puts frameDuration.inspect
			#puts clip[:frame].inspect
			#puts clip[:frameDuration].inspect
			node.send("ref-clip", :ref => clip[:nested_ref_id], :name => clip_name, :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled]) {
				node.parent["lane"] = clip[:lane].to_s if connected_clip && primary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip)
				# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
				#puts "primary = #{primary}", "nested_primary_start nil? #{nested_primary_start.nil?}"
				unless primary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		end
	end
end

def add_intrinsic_params_video(node, clip)
	#puts "add_intrinsic_params_video"
	unless clip[:crop_trim_rect].nil?
		if $dtd=="1.2" || $dtd=="1.3"
			node.send("adjust-crop", :mode => "trim") {
				node.send("trim-rect", :top => clip[:crop_trim_rect][:top], :left => clip[:crop_trim_rect][:left], :bottom => clip[:crop_trim_rect][:bottom], :right => clip[:crop_trim_rect][:right]) {
					node.param(:name => "top", :keyValues => clip[:crop_trim_rect][:top_keyvalues], :keyTimes => clip[:crop_trim_rect][:top_keytimes]) unless clip[:crop_trim_rect][:top_keytimes].empty?
					node.param(:name => "bottom", :keyValues => clip[:crop_trim_rect][:bottom_keyvalues], :keyTimes => clip[:crop_trim_rect][:bottom_keytimes]) unless clip[:crop_trim_rect][:bottom_keytimes].empty?
					node.param(:name => "left", :keyValues => clip[:crop_trim_rect][:left_keyvalues], :keyTimes => clip[:crop_trim_rect][:left_keytimes]) unless clip[:crop_trim_rect][:left_keytimes].empty?
					node.param(:name => "right", :keyValues => clip[:crop_trim_rect][:right_keyvalues], :keyTimes => clip[:crop_trim_rect][:right_keytimes]) unless clip[:crop_trim_rect][:right_keytimes].empty?
				}
			}
		else
			node.send("adjust-crop", :mode => "trim") {
				node.send("trim-rect", :top => clip[:crop_trim_rect][:top], :left => clip[:crop_trim_rect][:left], :bottom => clip[:crop_trim_rect][:bottom], :right => clip[:crop_trim_rect][:right]) {
					unless clip[:crop_trim_rect][:top_keyframes].empty?
						node.param(:name => "top") {
							node.keyframeAnimation {
								clip[:crop_trim_rect][:top_keyframes].each do |kf|
									node.keyframe(:time => kf[:time], :value => kf[:value])
								end
							}
						}
					end
					unless clip[:crop_trim_rect][:bottom_keyframes].empty?
						node.param(:name => "bottom") {
							node.keyframeAnimation {
								clip[:crop_trim_rect][:bottom_keyframes].each do |kf|
									node.keyframe(:time => kf[:time], :value => kf[:value])
								end
							}
						}
					end
					unless clip[:crop_trim_rect][:left_keyframes].empty?
						node.param(:name => "left") {
							node.keyframeAnimation {
								clip[:crop_trim_rect][:left_keyframes].each do |kf|
									node.keyframe(:time => kf[:time], :value => kf[:value])
								end
							}
						}
					end
					unless clip[:crop_trim_rect][:right_keyframes].empty?
						node.param(:name => "right") {
							node.keyframeAnimation {
								clip[:crop_trim_rect][:right_keyframes].each do |kf|
									node.keyframe(:time => kf[:time], :value => kf[:value])
								end
							}
						}
					end
				}
			}
		end
	end
	unless clip[:adj_corners].nil?
		if $dtd=="1.2" || $dtd=="1.3"
			node.send("adjust-corners", :botLeft => clip[:adj_corners][:botLeft], :topLeft => clip[:adj_corners][:topLeft], :topRight => clip[:adj_corners][:topRight], :botRight => clip[:adj_corners][:botRight]) {
				node.param(:name => "topLeft", :keyValues => clip[:adj_corners][:topleft_keyvalues], :keyTimes => clip[:adj_corners][:topleft_keytimes]) unless clip[:adj_corners][:topleft_keytimes].empty?
				node.param(:name => "topRight", :keyValues => clip[:adj_corners][:topright_keyvalues], :keyTimes => clip[:adj_corners][:topright_keytimes]) unless clip[:adj_corners][:topright_keytimes].empty?
				node.param(:name => "botLeft", :keyValues => clip[:adj_corners][:botleft_keyvalues], :keyTimes => clip[:adj_corners][:botleft_keytimes]) unless clip[:adj_corners][:botleft_keytimes].empty?
				node.param(:name => "botRight", :keyValues => clip[:adj_corners][:botright_keyvalues], :keyTimes => clip[:adj_corners][:botright_keytimes]) unless clip[:adj_corners][:botright_keytimes].empty?
			}
		else
			node.send("adjust-corners", :botLeft => clip[:adj_corners][:botLeft], :topLeft => clip[:adj_corners][:topLeft], :topRight => clip[:adj_corners][:topRight], :botRight => clip[:adj_corners][:botRight]) {
				unless clip[:adj_corners][:topleft_keyframes].empty?
					node.param(:name => "topLeft") {
						node.keyframeAnimation {
							clip[:adj_corners][:topleft_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:adj_corners][:topright_keyframes].empty?
					node.param(:name => "topRight") {
						node.keyframeAnimation {
							clip[:adj_corners][:topright_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:adj_corners][:botleft_keyframes].empty?
					node.param(:name => "botLeft") {
						node.keyframeAnimation {
							clip[:adj_corners][:botleft_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:adj_corners][:botright_keyframes].empty?
					node.param(:name => "botRight") {
						node.keyframeAnimation {
							clip[:adj_corners][:botright_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
			}
		end
	end
	node.send("adjust-conform", :type => clip[:adj_conform]) unless clip[:adj_conform].nil?
	unless clip[:basic_motion].nil?
		if $dtd=="1.2" || $dtd=="1.3"
			node.send("adjust-transform", :position => clip[:basic_motion][:adj_transform_position], :rotation => clip[:basic_motion][:adj_transform_rotation], :scale => clip[:basic_motion][:adj_transform_scale], :anchor => clip[:basic_motion][:adj_transform_anchor]) {
				node.param(:name => "position", :keyValues => clip[:basic_motion][:position_keyvalues], :keyTimes => clip[:basic_motion][:position_keytimes]) unless clip[:basic_motion][:position_keytimes].empty?
				node.param(:name => "rotation", :keyValues => clip[:basic_motion][:rotation_keyvalues], :keyTimes => clip[:basic_motion][:rotation_keytimes]) unless clip[:basic_motion][:rotation_keytimes].empty?
				node.param(:name => "scale", :keyValues => clip[:basic_motion][:scale_keyvalues], :keyTimes => clip[:basic_motion][:scale_keytimes]) unless clip[:basic_motion][:scale_keytimes].empty?
				node.param(:name => "anchor", :keyValues => clip[:basic_motion][:anchor_keyvalues], :keyTimes => clip[:basic_motion][:anchor_keytimes]) unless clip[:basic_motion][:anchor_keytimes].empty?
			}
		else
			node.send("adjust-transform", :position => clip[:basic_motion][:adj_transform_position], :rotation => clip[:basic_motion][:adj_transform_rotation], :scale => clip[:basic_motion][:adj_transform_scale], :anchor => clip[:basic_motion][:adj_transform_anchor]) {
				unless clip[:basic_motion][:position_keyframes].empty?
					node.param(:name => "position") {
						node.keyframeAnimation {
							clip[:basic_motion][:position_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:basic_motion][:rotation_keyframes].empty?
					node.param(:name => "rotation") {
						node.keyframeAnimation {
							clip[:basic_motion][:rotation_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:basic_motion][:scale_keyframes].empty?
					node.param(:name => "scale") {
						node.keyframeAnimation {
							clip[:basic_motion][:scale_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:basic_motion][:anchor_keyframes].empty?
					node.param(:name => "anchor") {
						node.keyframeAnimation {
							clip[:basic_motion][:anchor_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
			}
		end
	end
	unless clip[:adj_blend].nil?
		if clip[:adj_blend][:mode].nil?
			if $dtd=="1.2" || $dtd=="1.3"
				node.send("adjust-blend", :amount => clip[:adj_blend][:amount]) {
					node.param(:name => "amount", :keyValues => clip[:adj_blend][:keyvalues], :keyTimes => clip[:adj_blend][:keytimes]) unless clip[:adj_blend][:keytimes].empty?
				}
			else
				node.send("adjust-blend", :amount => clip[:adj_blend][:amount]) {
					unless clip[:adj_blend][:keyframes].empty?
						node.param(:name => "amount") {
							node.keyframeAnimation {
								clip[:adj_blend][:keyframes].each do |kf|
									node.keyframe(:time => kf[:time], :value => kf[:value])
								end
							}
						}
					end
				}
			end
		else # clip[:adj_blend][:mode] != nil
			if $dtd=="1.2" || $dtd=="1.3"
				node.send("adjust-blend", :mode => clip[:adj_blend][:mode], :amount => clip[:adj_blend][:amount]) {
					node.param(:name => "amount", :keyValues => clip[:adj_blend][:keyvalues], :keyTimes => clip[:adj_blend][:keytimes]) unless clip[:adj_blend][:keytimes].empty?
				}
			else
				node.send("adjust-blend", :mode => clip[:adj_blend][:mode], :amount => clip[:adj_blend][:amount]) {
					unless clip[:adj_blend][:keyframes].empty?
						node.param(:name => "amount") {
							node.keyframeAnimation {
								clip[:adj_blend][:keyframes].each do |kf|
									node.keyframe(:time => kf[:time], :value => kf[:value])
								end
							}
						}
					end
				}
			end
		end
	end
end

def add_intrinsic_params_audio(node, clip)
	#puts "add_intrinsic_params_audio"
	if clip[:type]=="av" || clip[:type]=="a"
		if clip[:channels]=="stereo"
			channel = 0
			node.send("audio-source", :srcCh => "1, 2", :outCh => "L, R") {
				unless clip[:adj_volume].nil? || clip[:adj_volume][channel].nil?
					node.send("adjust-volume", :amount => clip[:adj_volume][channel][:amount]) {
						if $dtd=="1.2" || $dtd=="1.3"
							node.param(:name => "amount", :keyValues => clip[:adj_volume][channel][:keyvalues], :keyTimes => clip[:adj_volume][channel][:keytimes]) unless clip[:adj_volume][channel][:keytimes].empty?
						else
							unless clip[:adj_volume][channel][:keyframes].empty?
								node.param(:name => "amount") {
									node.keyframeAnimation {
										clip[:adj_volume][channel][:keyframes].each do |kf|
											node.keyframe(:time => kf[:time], :value => kf[:value])
										end
									}
								}
							end
						end
					}
				end
				unless clip[:adj_panner].nil? || clip[:adj_panner][channel].nil?
					node.send("adjust-panner", :amount => clip[:adj_panner][channel][:amount]) {
						if $dtd=="1.2" || $dtd=="1.3"
							node.param(:name => "amount", :keyValues => clip[:adj_panner][channel][:keyvalues], :keyTimes => clip[:adj_panner][channel][:keytimes]) unless clip[:adj_panner][channel][:keytimes].empty?
						else
							unless clip[:adj_panner][channel][:keyframes].empty?
								node.param(:name => "amount") {
									node.keyframeAnimation {
										clip[:adj_panner][channel][:keyframes].each do |kf|
											node.keyframe(:time => kf[:time], :value => kf[:value])
										end
									}
								}
							end
						end
					}
				end
				unless clip[:mutes].nil? || clip[:mutes][channel].nil?
					node.mute(:start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :duration => to_rational_seconds(clip[:duration], clip[:frameDuration])) if clip[:mutes][channel]==true
				end
			}
		else # mono
			0.upto(clip[:channelcount]-1) do |channel|
				node.send("audio-source", :srcCh => channel+1) {
					unless clip[:adj_volume].nil? || clip[:adj_volume][channel].nil?
						node.send("adjust-volume", :amount => clip[:adj_volume][channel][:amount]) {
							if $dtd=="1.2" || $dtd=="1.3"
								node.param(:name => "amount", :keyValues => clip[:adj_volume][channel][:keyvalues], :keyTimes => clip[:adj_volume][channel][:keytimes]) unless clip[:adj_volume][channel][:keytimes].empty?
							else
								unless clip[:adj_volume][channel][:keyframes].empty?
									node.param(:name => "amount") {
										node.keyframeAnimation {
											clip[:adj_volume][channel][:keyframes].each do |kf|
												node.keyframe(:time => kf[:time], :value => kf[:value])
											end
										}
									}
								end
							end
						}
					end
					unless clip[:adj_panner].nil? || clip[:adj_panner][channel].nil?
						node.send("adjust-panner", :amount => clip[:adj_panner][channel][:amount]) {
							if $dtd=="1.2" || $dtd=="1.3"
								node.param(:name => "amount", :keyValues => clip[:adj_panner][channel][:keyvalues], :keyTimes => clip[:adj_panner][channel][:keytimes]) unless clip[:adj_panner][channel][:keytimes].empty?
							else
								unless clip[:adj_panner][channel][:keyframes].empty?
									node.param(:name => "amount") {
										node.keyframeAnimation {
											clip[:adj_panner][channel][:keyframes].each do |kf|
												node.keyframe(:time => kf[:time], :value => kf[:value])
											end
										}
									}
								end
							end
						}
					end
					unless clip[:mutes].nil? || clip[:mutes][channel].nil?
						node.mute(:start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :duration => to_rational_seconds(clip[:duration], clip[:frameDuration])) if clip[:mutes][channel]==true
					end
				}
			end
		end
	end
end

def add_filters(node, clip, type="a+v")
	unless clip[:filters].nil?
		if $dtd=="1.2"
			clip[:filters].each { |f| node.filter(:name => f[:name], :ref => f[:ref], :enabled => f[:enabled]) }
		else
			clip[:filters].each do |f|
				if f[:type]=="v"
					node.send("filter-video", :name => f[:name], :ref => f[:ref], :enabled => f[:enabled]) if (type=="a+v" || type=="v")
				else
					node.send("filter-audio", :name => f[:name], :ref => f[:ref], :enabled => f[:enabled]) if (type=="a+v" || type=="a")
				end
			end
		end
	end
end

def order_markers(user_markers, chapter_markers, head_markers, tail_markers, frame, speed_adj, inpoint, outpoint, frameDuration)
	#puts "order_markers", "frame = #{frame.inspect}, speed_adj = #{speed_adj.inspect} inpoint = #{inpoint.inspect}, outpoint = #{outpoint.inspect}, frameDuration = #{frameDuration}"
	speed_adj = 1 if speed_adj.nil?
	user_markers.compact!
	head_markers.compact!
	tail_markers.compact!
	#puts "#{user_markers.length} user_markers"
	#puts "#{head_markers.length} head_markers"
	#pp head_markers
	head_markers.each do |m|
		#pp m
		head_marker_in = m[:in] + inpoint
		conflict = true
		until conflict==false
			conflict = user_markers.any? { |u| u[:in]==head_marker_in }
			if conflict
				head_marker_in += 1
			end
		end
		until conflict==false
			conflict = chapter_markers.any? { |c| c[:in]==head_marker_in }
			if conflict
				head_marker_in += 1
			end
		end
		m[:in] = head_marker_in
		user_markers << m
	end
	#pp tail_markers
	tail_markers.each do |m|
		tail_marker_in = outpoint - m[:in]
		conflict = true
		until conflict==false
			conflict = user_markers.any? { |u| u[:in]==tail_marker_in }
			if conflict
				tail_marker_in -= 1
			end
		end
		until conflict==false
			conflict = chapter_markers.any? { |c| c[:in]==tail_marker_in }
			if conflict
				tail_marker_in -= 1
			end
		end
		m[:in] = tail_marker_in
		user_markers << m
	end
	markers = Array.new
	user_markers.sort_by { |u| u[:in] }.each do |m|
		#puts "start = to_rational_seconds(#{frame} / #{speed_adj}, #{m[:frameDuration]}, #{m[:in]}, #{frameDuration})"
		# calculated the same as: start = to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], seq_settings[:frameDuration])
		# because frame is in the clip's timebase; :in is in the sequence's timebase
		start = to_rational_seconds(frame / speed_adj, m[:frameDuration], m[:in], frameDuration)
		if m[:out].nil?
			duration = "#{m[:frameDuration]}s"
		else
			#puts "m[:out] = #{m[:out]}, m[:in] = #{m[:in]}"
			duration = to_rational_seconds(m[:out] - m[:in], frameDuration)
		end
		if m[:completed].nil? # blue markers
			markers << "<marker start=\"#{start}\" duration=\"#{duration}\" value=\"#{m[:value]}\" note=\"#{m[:note]}\"/>"
		else # to-do markers
			markers << "<marker start=\"#{start}\" duration=\"#{duration}\" value=\"#{m[:value]}\" note=\"#{m[:note]}\" completed=\"#{m[:completed]}\"/>"
		end
	end
	chmarkers = Array.new
	chapter_markers.each do |m|
		start = to_rational_seconds(frame / speed_adj, m[:frameDuration], m[:in], frameDuration)
		#puts "start = #{frame} / #{speed_adj}, #{m[:frameDuration]}, #{m[:in]}, #{frameDuration}"
		duration = "#{m[:frameDuration]}s"
		chmarkers << "<chapter-marker start=\"#{start}\" duration=\"#{duration}\" value=\"#{m[:value]}\" posterOffset='0s' />" # CHANGED chmarkers is an array of strings; posterOffset attribute is being added as posteroffset if there's an escaped ampersand (I've got a clean-up before writing out)
	end
	return markers, chmarkers
end

########## Version 1.6 FCPXML with support for asset-clips; sync-clips; audio-role-source and audio-channel-source and sync-source (instead of audio-source and audio-aux-source)
def saveLibraryXML(new_xml_file)
	puts "------------------------------------------ saveLibraryXML"
	begin
		@id = 0 # text-style-def unique id number
		builder = Nokogiri::XML::Builder.new do |fcpxml|
        executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
			version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
			fcpxml.comment(" #{executable} v#{version}, #{Time.now.strftime("%Y-%m-%d")} ") unless executable.nil?
			fcpxml.doc.create_internal_subset("fcpxml", nil, nil)
			fcpxml.fcpxml(:version => $dtd) {
				fcpxml.send("import-options") {
					fcpxml.option(:key => "copy assets", :value => "0") # link to assets instead of copy
					if $ruby_cocoa==false
						fcpxml.option(:key => "suppress warnings", :value => "0")
					else
						fcpxml.option(:key => "suppress warnings", :value => "1")
					end
				}
				fcpxml.resources {
					# Formats
					@formats.each_with_index do |f, i|
						#puts f.inspect
						if f[:fieldOrder].nil? || f[:width].nil? || f[:height].nil? # audio-only
							fcpxml.format(:id => f[:id], :name => f[:name])
						elsif f[:stillframe]
							fcpxml.format(:id => f[:id], :name => "FFVideoFormatRateUndefined", :width => f[:width], :height => f[:height])
						elsif f[:name].nil? # custom (fieldOrder?)
							fcpxml.format(:id => f[:id], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height])
						elsif f[:paspH].nil?
							fcpxml.format(:id => f[:id], :name => f[:name], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height])
						else
							fcpxml.format(:id => f[:id], :name => f[:name], :frameDuration => "#{f[:frameDuration]}s", :fieldOrder => f[:fieldOrder], :width => f[:width], :height => f[:height], :paspH => f[:paspH], :paspV => f[:paspV])
						end
					end
					# Assets
					@assets.each_with_index do |a, i|
						#puts a.inspect
						case a[:type]
						when "av"
							fcpxml.asset(:id => a[:id], :format =>a[:format_id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasVideo => "1", :hasAudio => "1", :audioSources => a[:audioSources], :audioChannels => a[:audioChannels], :audioRate => a[:samplerate])
						when "a"
							fcpxml.asset(:id => a[:id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasAudio => "1", :audioSources => a[:audioSources], :audioChannels => a[:audioChannels], :audioRate => a[:samplerate]) # no format id
						when "v", "psd", "img"
							if a[:stillframe]
								fcpxml.asset(:id => a[:id], :src => a[:src], :duration => "0s", :start => "0s", :hasVideo => "1") # no format id
							else
								fcpxml.asset(:id => a[:id], :format =>a[:format_id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :hasVideo => "1")
							end
						else # when could type be ""?
							fcpxml.asset(:id => a[:id], :format =>a[:format_id], :src => a[:src], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]))
						end
						unless a[:aux_tc].empty?
							query = "//resources/asset"
							Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last) do |asset|
								asset.metadata {
									asset.md(:key => "com.apple.proapps.custom.soundroll", :displayName => "Sound Roll", :value => a[:aux_tc].first, :type => "string", :editable => "1",  :source => "custom source")
									asset.md(:key => "com.apple.proapps.custom.soundtc", :displayName => "Aux TC", :value => a[:aux_tc].last, :type => "string", :editable => "1",  :source => "custom source")
								}
							end
						end
					end
					# Multicams
					@multiclips.each_with_index do |multiclip, m|
						fcpxml.media(:id => multiclip[:id], :name => multiclip[:name]) {
							fcpxml.multicam(:format => multiclip[:format_id], :tcFormat => multiclip[:tcFormat], :tcStart => to_rational_seconds(multiclip[:frame], multiclip[:frameDuration])) {
								multiclip[:angles].each do |a|
									unless a.nil?
										fcpxml.send("mc-angle", :name => a[:name], :angleID => a[:id]+"angle") {
											fcpxml.gap(:duration => to_rational_seconds(a[:mediadelay], a[:frameDuration])) unless a[:mediadelay]==0
											fcpxml.clip(:name => a[:name], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :offset => to_rational_seconds(a[:mediadelay], a[:frameDuration]), :tcFormat => a[:tcFormat]) {
												case a[:type]
												when "v"
													fcpxml.video(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]))
												when "a"
													if a[:channels]=="stereo" # one or more stereo tracks
														fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration])) {
															unless a[:audioSources]==1
																a[:audioSources].downto(2) do |src|
																	fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-src, :srcID => src)
																end
															end
														}
													elsif a[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
														fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
														}
													# elsif a[:channels]=="surround"
													else # one or more mono tracks
														fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :srcCh => 1 ) {
															unless a[:channelcount]==1
																if a[:audioSources]==1 # no srcID because audioSources==1
																	a[:audioChannels].downto(2) do |ch|
																		fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-ch, :srcCh => ch )
																	end
																elsif a[:audioChannels]==1 || a[:audioChannels]==2
																	a[:audioSources].downto(2) do |src|
																		srcCh = "1"
																		unless a[:srcChs][src-1].nil?
																			srcCh = a[:srcChs][src-1]
																		end
																		fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
																	end
																end
															end
														}
													end
												when "av"
													fcpxml.video(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration])) {
														if a[:channels]=="stereo" # one or more stereo tracks
															a[:audioSources].downto(1) do |src|
																if src==1 # no srcID
																	fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1)
																else
																	fcpxml.audio(:offset => to_rational_seconds(a[:frame], a[:frameDuration]), :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-src, :srcID => src)
																end
															end
														elsif a[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
															fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
														# elsif a[:channels]=="surround"
														else # one or more mono tracks
															if a[:audioSources]==1 # no srcID because audioSources==1
																a[:audioChannels].downto(1) do |ch|
																	fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-ch, :srcCh => ch )
																end
															elsif a[:audioChannels]==1 || a[:audioChannels]==2
																a[:audioSources].downto(1) do |src|
																	srcCh = "1"
																	unless a[:srcChs][src-1].nil?
																		srcCh = a[:srcChs][src-1]
																	end
																	fcpxml.audio( :ref => a[:id], :duration => to_rational_seconds(a[:duration], a[:frameDuration]), :offset => to_rational_seconds(a[:frame], a[:frameDuration]), :start => to_rational_seconds(a[:frame], a[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
																end
															end
														end
													}
												end
											}
										}
									end
								end
							}
						}
					end
					# Compound Clips - add each nested sequence as a reference compound clip
					@nested_seqs.each do |nested_seq|
						#puts nested_seq[:name]
						cc_frameDuration = nested_seq[:frameDuration]
						fcpxml.media(:id => nested_seq[:nested_ref_id], :name => nested_seq[:name]) {
							fcpxml.sequence(:format => nested_seq[:format_id], :duration => to_rational_seconds(nested_seq[:duration], cc_frameDuration), :tcFormat => nested_seq[:nested_seq_tcFormat], :audioLayout => nested_seq[:nested_seq_audioLayout], :tcStart => to_rational_seconds(nested_seq[:frame], cc_frameDuration)) { # audioRate is expressed as 48k instead of 48000
								# if stereo, fcpxml.parent["layout"] = "stereo"
								fcpxml.note nested_seq[:metadata] unless nested_seq[:metadata].empty?
								fcpxml.spine {
									clip_array = nested_seq[:nested_array]
									last_clip_index = clip_array.first.length - 1
									if clip_array.first.empty?
										last_clip_index = 0
									else
										if clip_array.first[last_clip_index][:type]=="gap"
											last_clip_index -= 1
										end
										if clip_array.first[last_clip_index][:type]=="transition"
											last_clip_index -= 1
										end
									end
									#puts "#{clip_array.first.length} primary storyline clips"
									clip_array.first.each_with_index do |primary_clip, p| # clip_array.first is the V1 track, which becomes the primary storyline
										@error_tracker = ":\n'#{nested_seq[:name]}' Clip '#{primary_clip[:name]}'"
										#puts "1. nested_seq:" if nested_seq[:height].nil?
										#pp nested_seq if nested_seq[:height].nil?
										storylineBuilder(fcpxml, [primary_clip], nested_seq[:frame], nested_seq[:frame], cc_frameDuration, nested_seq[:height], true) # node, clips, adj, frame, frameDuration, height, primary=false
										case primary_clip[:type] # also check case nested_primary_clip[:type]
										when "title"
											primary_start = (3600 * 1/cc_frameDuration).round
										when "gap"
											primary_start = (3600 * 1/cc_frameDuration).round
										when "generator"
											primary_start = (3600 * 1/cc_frameDuration).round
										when "transition"
											# ignore - can't be connected to
										when "mc"
											primary_start = (primary_clip[:mc_frame] / primary_clip[:speed_adj]) + primary_clip[:in] # bugfix for mc
										when "nested"
											primary_start = (primary_clip[:frame] / primary_clip[:speed_adj]) + primary_clip[:in] # WAS nested_seq[:frame]
										else # clip
											# refactor the primary clip's first frame to the sequences frame rate (Bug fix for clips and secondary storylines connected to a primary clip whose frame rate is different from the timeline's frame rate)
											refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/cc_frameDuration)
											primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
										end
										primary_offset = primary_clip[:start]
										path_to_primary_spine = ["spine"]
										fcpxml.parent.ancestors.each { |a| path_to_primary_spine << a.name unless a.name=="document" }
										query = path_to_primary_spine.reverse.join("/")
										#puts "query = #{query}"
										#puts "************** primary_start = #{primary_start}", "primary_offset = #{primary_offset}"
										@compound_clip_name = nested_seq[:name]
										Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last.element_children.last) do |primary_node| # the primary_node is the fcpxnl's last spine's last element_children node
											unless primary_clip[:array_of_connecteds].nil? || primary_clip[:type]=="transition"
												# attach connected clips or secondary spines to the last primary clip
												# if primary_clip[:frameDuration].nil?
												# 	pc_frameDuration = cc_frameDuration
												# else
												# 	pc_frameDuration = primary_clip[:frameDuration]
												# end
												# puts "pc_frameDuration = #{pc_frameDuration}"
												primary_clip[:array_of_connecteds].each do |connecteds|
													unless connecteds.empty?
														if connecteds.length==1 && connecteds.first[:type]=="transition"
															#puts "CONNECTING A SINGLE TRANSITION !!!!!!"
														end
														#puts "************** primary_start - primary_offset + connecteds.first[:start] = #{primary_start} - #{primary_offset} + #{connecteds.first[:start]}"
														#puts "2. nested_seq: #{nested_seq.inspect}" if nested_seq[:height].nil?
														if connecteds.length==1 && connecteds.first[:type]!="transition" # connected clip
															storylineBuilder(primary_node, connecteds, primary_start - primary_offset, nested_seq[:frame], cc_frameDuration, nested_seq[:height]) # node, clips, adj, frame, frameDuration, height, primary=false
														elsif connecteds.length > 1 # secondary storyline
															primary_node.spine(:lane => connecteds.first[:lane], :offset => to_rational_seconds(primary_start - primary_offset + connecteds.first[:start], cc_frameDuration)) {
																storylineBuilder(primary_node, connecteds, connecteds.first[:start]*-1, nested_seq[:frame], cc_frameDuration, nested_seq[:height])
															}
														end
													end
												end
											end
											# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
											# pp primary_clip
											unless primary_start.nil? || primary_clip[:type]=="transition"
												if p==last_clip_index
													markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], primary_clip[:tail_markers], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], cc_frameDuration)
												else
													markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], [], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], cc_frameDuration)
												end
												markers.each { |m| primary_node << m }
												chmarkers.each { |m| primary_node << m }
											end
											# add_intrinsic_params_audio(primary_node, primary_clip)
											add_filters(primary_node, primary_clip) unless primary_clip[:filters].nil? || primary_clip[:type]=="mc"
											unless primary_clip[:reel_name].nil? && primary_clip[:scene].nil? && primary_clip[:shottake].nil? # transitions, slugs etc.
												primary_node.metadata {
													#puts "primary_clip[:reel_name]", primary_clip[:reel_name].inspect
													# FIXME unless primary_clip[:reel_name].empty?
													primary_node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => primary_clip[:reel_name])
													primary_node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => primary_clip[:scene])
													primary_node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => primary_clip[:shottake])
													primary_node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => primary_clip[:label])
												}
											end
										end
									end
								}
							}
						}
					end # end @nested_seqs.each
					# Transitions
					fcpxml.effect(:id => "e1", :name => "Cross Dissolve", :uid => "FxPlug:4731E73A-8DAC-4113-9A30-AE85B1761265")
					fcpxml.effect(:id => "ea1", :name => "Audio Crossfade", :uid => "FFAudioTransition")
					fcpxml.effect(:id => "e2", :name => "Center", :uid => "FxPlug:539C8A29-BBED-4670-B774-338109A7DB68")
					fcpxml.effect(:id => "e3", :name => "Squares", :uid => ".../Transitions.localized/Objects.localized/Squares.localized/Squares.motr")
					fcpxml.effect(:id => "e4", :name => "Zoom", :uid => ".../Transitions.localized/Blurs.localized/Zoom.localized/Zoom.motr")
					fcpxml.effect(:id => "e5", :name => "Cube", :uid => "FxPlug:653CCCCD-6CB1-4602-B9B3-181B1C53E069")
					fcpxml.effect(:id => "e6", :name => "Flip", :uid => ".../Transitions.localized/Movements.localized/Flip.localized/Flip.motr")
					fcpxml.effect(:id => "e7", :name => "Swing", :uid => ".../Transitions.localized/Movements.localized/Swing.localized/Swing.motr")
					fcpxml.effect(:id => "e8", :name => "Fade To Color", :uid => "FxPlug:F779C565-486D-4633-8035-0374B4DB8F5C")
					fcpxml.effect(:id => "e9", :name => "Ripple", :uid => "FxPlug:D386C0D5-5BD3-4A0D-86FC-6B9878C76451")
					fcpxml.effect(:id => "e10", :name => "Letter X", :uid => "FxPlug:743DCA2A-B49F-4B19-8A52-B9575D631139")
					fcpxml.effect(:id => "e11", :name => "Circle", :uid => "FxPlug:1C52AC71-7116-4248-B51F-5F5641EA9EDD")
					fcpxml.effect(:id => "e12", :name => "Inset Wipe", :uid => "FxPlug:5F0E1FA2-C172-4AFF-88CB-64B0F08195E4")
					fcpxml.effect(:id => "e13", :name => "Wipe", :uid => "FxPlug:857E2FBA-98DB-411B-A88C-CE6ABC1F65D8")
					fcpxml.effect(:id => "e14", :name => "Page Curl", :uid => "FxPlug:21B9D89C-4162-43FF-BA07-B019989E4940")
					fcpxml.effect(:id => "e15", :name => "Gradient Image", :uid => "FxPlug:62199A78-FDBF-4E9B-BB25-9A1D62FD2D48")
					fcpxml.effect(:id => "e16", :name => "Mosaic", :uid => "FxPlug:66B87635-119C-40B2-BF6E-537A3165FB95")
					fcpxml.effect(:id => "e17", :name => "Clock", :uid => "FxPlug:B2C3F87B-2A21-4E13-8173-46ED4FEBC57A")
					fcpxml.effect(:id => "e18", :name => "Slide", :uid => "FxPlug:6AAB0D54-FCD8-4EBD-A62D-D352A5ED1648")
					fcpxml.effect(:id => "e19", :name => "Spin", :uid => "FxPlug:196B9DB2-28FD-420F-9BA1-3F0E9EEBBAAA")
					fcpxml.effect(:id => "e20", :name => "Band", :uid => "FxPlug:8AD995CC-8DD6-4652-A3D2-B1468DCC6203")
					fcpxml.effect(:id => "e21", :name => "Black Hole", :uid => ".../Transitions.localized/Movements.localized/Black Hole.localized/Black Hole.motr")
					# Slug and Generators
					fcpxml.effect(:id => "g1", :name => "Custom", :uid => ".../Generators.localized/Solids.localized/Custom.localized/Custom.motn")
					fcpxml.effect(:id => "g2", :name => "Gradient", :uid => ".../Generators.localized/Textures.localized/Gradient.localized/Gradient.motn")
					fcpxml.effect(:id => "g3", :name => "Shapes", :uid => ".../Generators.localized/Elements.localized/Shapes.localized/Shapes.motn")
					# Titles
					fcpxml.effect(:id => "t1", :name => "Custom", :uid => ".../Titles.localized/Build In:Out.localized/Custom.localized/Custom.moti")
					fcpxml.effect(:id => "t2", :name => "Middle", :uid => ".../Titles.localized/Lower Thirds.localized/Middle.localized/Middle.moti")
					fcpxml.effect(:id => "t3", :name => "Scrolling", :uid => ".../Titles.localized/Credits.localized/Scrolling.localized/Scrolling.moti")
					fcpxml.effect(:id => "t4", :name => "Ticker", :uid => ".../Titles.localized/Build In:Out.localized/Ticker.localized/Ticker.moti")
					fcpxml.effect(:id => "t5", :name => "Typewriter", :uid => ".../Titles.localized/Build In:Out.localized/Typewriter.localized/Typewriter.moti")
					# Video Filters
					fcpxml.effect(:id => "f1", :name => "Censor", :uid => ".../Effects.localized/Stylize.localized/Censor.localized/Censor.moef")
					fcpxml.effect(:id => "f2", :name => "Cartoon", :uid => ".../Effects.localized/Stylize.localized/Cartoon.localized/Cartoon.moef")
					fcpxml.effect(:id => "f3", :name => "Directional", :uid => ".../Effects.localized/Blur.localized/Directional.localized/Directional.moef")
					fcpxml.effect(:id => "f4", :name => "Gaussian", :uid => ".../Effects.localized/Blur.localized/Gaussian.localized/Gaussian.moef")
					fcpxml.effect(:id => "f5", :name => "Prism", :uid => ".../Effects.localized/Blur.localized/Prism.localized/Prism.moef")
					fcpxml.effect(:id => "f6", :name => "Radial", :uid => ".../Effects.localized/Blur.localized/Radial.localized/Radial.moef")
					fcpxml.effect(:id => "f7", :name => "Zoom", :uid => ".../Effects.localized/Blur.localized/Zoom.localized/Zoom.moef")
					fcpxml.effect(:id => "f8", :name => "Mask", :uid => ".../Effects.localized/Keying.localized/Mask.localized/Mask.moef")
					fcpxml.effect(:id => "f9", :name => "Simple Border", :uid => ".../Effects.localized/Stylize.localized/Simple Border.localized/Simple Border.moef")
					fcpxml.effect(:id => "f10", :name => "Droplet", :uid => ".../Effects.localized/Distortion.localized/Droplet.localized/Droplet.moef")
					fcpxml.effect(:id => "f11", :name => "Earthquake", :uid => ".../Effects.localized/Distortion.localized/Earthquake.localized/Earthquake.moef")
					fcpxml.effect(:id => "f12", :name => "Fisheye", :uid => ".../Effects.localized/Distortion.localized/Fisheye.localized/Fisheye.moef")
					fcpxml.effect(:id => "f13", :name => "Fun House", :uid => ".../Effects.localized/Distortion.localized/Fun House.localized/Fun House.moef")
					fcpxml.effect(:id => "f14", :name => "Insect Eye", :uid => ".../Effects.localized/Distortion.localized/Insect Eye.localized/Insect Eye.moef")
					fcpxml.effect(:id => "f15", :name => "Scrape", :uid => ".../Effects.localized/Distortion.localized/Scrape.localized/Scrape.moef")
					fcpxml.effect(:id => "f16", :name => "Wave", :uid => ".../Effects.localized/Distortion.localized/Wave.localized/Wave.moef")
					fcpxml.effect(:id => "f17", :name => "Glow", :uid => ".../Effects.localized/Light.localized/Glow.localized/Glow.moef")
					fcpxml.effect(:id => "f18", :name => "Bloom", :uid => ".../Effects.localized/Light.localized/Bloom.localized/Bloom.moef")
					fcpxml.effect(:id => "f19", :name => "Hue/Saturation", :uid => ".../Effects.localized/Basics.localized/Hue:Saturation.localized/Hue:Saturation.moef")
					fcpxml.effect(:id => "f20", :name => "Sepia", :uid => ".../Effects.localized/Basics.localized/Sepia.localized/Sepia.moef")
					fcpxml.effect(:id => "f21", :name => "Tint", :uid => ".../Effects.localized/Basics.localized/Tint.localized/Tint.moef")
					fcpxml.effect(:id => "f22", :name => "Broadcast Safe", :uid => ".../Effects.localized/Basics.localized/Broadcast Safe.localized/Broadcast Safe.moef")
					fcpxml.effect(:id => "f23", :name => "Keyer", :uid => "FxPlug:41122549-B8A6-470E-94DA-211294D20B62")
					fcpxml.effect(:id => "f24", :name => "Luma Keyer", :uid => "FxPlug:7E9178C5-7B0F-4B86-884D-FE79F568B6CE")
					fcpxml.effect(:id => "f25", :name => "Image Mask", :uid => ".../Effects.localized/Keying.localized/Image Mask.localized/Image Mask.moef")
					fcpxml.effect(:id => "f26", :name => "Crop &amp; Feather", :uid => ".../Effects.localized/Distortion.localized/Crop &amp; Feather.localized/Crop &amp; Feather.moef")
					fcpxml.effect(:id => "f27", :name => "Letterbox", :uid => ".../Effects.localized/Stylize.localized/Letterbox.localized/Letterbox.moef")
					fcpxml.effect(:id => "f28", :name => "Flipped", :uid => ".../Effects.localized/Distortion.localized/Flipped.localized/Flipped.moef")
					fcpxml.effect(:id => "f29", :name => "Mirror", :uid => ".../Effects.localized/Distortion.localized/Mirror.localized/Mirror.moef")
					fcpxml.effect(:id => "f30", :name => "Sharpen", :uid => ".../Effects.localized/Blur.localized/Sharpen.localized/Sharpen.moef")
					fcpxml.effect(:id => "f31", :name => "Add Noise", :uid => ".../Effects.localized/Stylize.localized/Add Noise.localized/Add Noise.moef")
					fcpxml.effect(:id => "f32", :name => "Aged Film", :uid => ".../Effects.localized/Stylize.localized/Aged Film.localized/Aged Film.moef")
					fcpxml.effect(:id => "f33", :name => "Bad TV", :uid => ".../Effects.localized/Stylize.localized/Bad TV.localized/Bad TV.moef")
					fcpxml.effect(:id => "f34", :name => "Pixellate", :uid => ".../Effects.localized/Stylize.localized/Pixellate.localized/Pixellate.moef")
					fcpxml.effect(:id => "f35", :name => "Halftone", :uid => ".../Effects.localized/Stylize.localized/Halftone.localized/Halftone.moef")
					fcpxml.effect(:id => "f36", :name => "Cross Hatch", :uid => ".../Effects.localized/Stylize.localized/Cross Hatch.localized/Cross Hatch.moef")
					fcpxml.effect(:id => "f37", :name => "Graphic", :uid => ".../Effects.localized/Stylize.localized/Graphic.localized/Graphic.moef")
					fcpxml.effect(:id => "f38", :name => "Tile", :uid => ".../Effects.localized/Tiling.localized/Tile.localized/Tile.moef")
					fcpxml.effect(:id => "f39", :name => "Vignette", :uid => ".../Effects.localized/Stylize.localized/Vignette.localized/Vignette.moef")
					fcpxml.effect(:id => "f40", :name => "Kaleidoscope", :uid => ".../Effects.localized/Tiling.localized/Kaleidoscope.localized/Kaleidoscope.moef")
					fcpxml.effect(:id => "f41", :name => "Trails", :uid => ".../Effects.localized/Distortion.localized/Trails.localized/Trails.moef")
					fcpxml.effect(:id => "f42", :name => "Flashing", :uid => ".../Effects.localized/Light.localized/Flashing.localized/Flashing.moef")
					fcpxml.effect(:id => "f43", :name => "Camcorder", :uid => ".../Effects.localized/Stylize.localized/Camcorder.localized/Camcorder.moef")
					fcpxml.effect(:id => "f44", :name => "Kaleidotile", :uid => ".../Effects.localized/Tiling.localized/Kaleidotile.localized/Kaleidotile.moef")
					fcpxml.effect(:id => "f45", :name => "Black & White", :uid => ".../Effects.localized/Basics.localized/Black & White.localized/Black & White.moef")
					fcpxml.effect(:id => "f124", :name => "Drop Shadow",  :uid => "FxPlug:9C13F991-BC99-4DC8-B150-381D7CCE183B")
					# Audio Filters
					fcpxml.effect(:id => "f100", :name => "AUBandpass", :uid => "AudioUnit: 0x61756678627061736170706c")
					fcpxml.effect(:id => "f101", :name => "AUDelay", :uid => "AudioUnit: 0x6175667864656c796170706c")
					fcpxml.effect(:id => "f102", :name => "AUDistortion", :uid => "AudioUnit: 0x61756678646973746170706c")
					fcpxml.effect(:id => "f103", :name => "AUDynamicsProcessor", :uid => "AudioUnit: 0x6175667864636d706170706c")
					fcpxml.effect(:id => "f104", :name => "AUFilter", :uid => "AudioUnit: 0x6175667866696c746170706c")
					fcpxml.effect(:id => "f105", :name => "AUGraphicEQ", :uid => "AudioUnit: 0x61756678677265716170706c")
					fcpxml.effect(:id => "f106", :name => "AUHighShelfFilter", :uid => "AudioUnit: 0x61756678687368666170706c")
					fcpxml.effect(:id => "f107", :name => "AUHipass", :uid => "AudioUnit: 0x61756678687061736170706c")
					fcpxml.effect(:id => "f108", :name => "AULowpass", :uid => "AudioUnit: 0x617566786c7061736170706c")
					fcpxml.effect(:id => "f109", :name => "AULowShelfFilter", :uid => "AudioUnit: 0x617566786c7368666170706c")
					fcpxml.effect(:id => "f110", :name => "AUMultibandCompressor", :uid => "AudioUnit: 0x617566786d636d706170706c")
					fcpxml.effect(:id => "f111", :name => "AUNetSend", :uid => "AudioUnit: 0x617566786e736e646170706c")
					fcpxml.effect(:id => "f112", :name => "AUParametricEQ", :uid => "AudioUnit: 0x61756678706d65716170706c")
					fcpxml.effect(:id => "f113", :name => "AUPeakLimiter", :uid => "AudioUnit: 0x617566786c6d74726170706c")
					fcpxml.effect(:id => "f114", :name => "AUPitch", :uid => "AudioUnit: 0x61756678746d70746170706c")
					fcpxml.effect(:id => "f115", :name => "AURogerBeep", :uid => "AudioUnit: 0x61756678726f67726170706c")
					fcpxml.effect(:id => "f116", :name => "AUSampleDelay", :uid => "AudioUnit: 0x6175667873646c796170706c")
					fcpxml.effect(:id => "f117", :name => "Channel EQ", :uid => "AudioUnit: 0x61756678000000ec454d4147")
					fcpxml.effect(:id => "f118", :name => "Echo Delay", :uid => "Echo Delay.Echo.audio.effectBundle")
					fcpxml.effect(:id => "f119", :name => "Noise Gate", :uid => "AudioUnit: 0x61756678000000b3454d4147")
					fcpxml.effect(:id => "f120", :name => "Gain", :uid => "AudioUnit: 0x61756678000000b7454d4147")
					fcpxml.effect(:id => "f121", :name => "AUMatrixReverb", :uid => "AudioUnit: 0x617566786d7265766170706c")
					fcpxml.effect(:id => "f122", :name => "DeEsser", :uid => "AudioUnit: 0x61756678000000a0454d4147")
					fcpxml.effect(:id => "f123", :name => "Compressor", :uid => "AudioUnit: 0x617566780000009a454d4147")
				}
				good = "Favorite"
				if @goodPopupMenu.nil? || @goodPopupMenu.titleOfSelectedItem=="Favorite"
					# Favorite
				elsif @goodPopupMenu.titleOfSelectedItem=='"Good" Keyword'
					good = "Good"
				else
					good = "Circle Take"
				end
				bins = Array.new
				fcpxml.library {
					fcpxml.event(:name => @event[:name]) {
						### add each sequence as a project/sequence to the event
						@project_seqs.each do |project_seq|
							#puts project_seq[:name]
							seq_frameDuration = project_seq[:frameDuration]
							if project_seq[:merged]
								# add clip/spine
							else
								# add project/sequence/spine
							end
							fcpxml.project(:id => project_seq[:id], :name => project_seq[:name]) {
								fcpxml.sequence(:format => project_seq[:format_id], :duration => to_rational_seconds(project_seq[:duration], seq_frameDuration), :tcFormat => project_seq[:tcFormat], :audioLayout => project_seq[:audioLayout], :tcStart => to_rational_seconds(project_seq[:frame], seq_frameDuration)) { # audioRate is expressed as 48k instead of 48000
									unless project_seq[:keywords].empty?
										fcpxml.parent["keywords"] = project_seq[:keywords]
									end
									fcpxml.note project_seq[:metadata] unless project_seq[:metadata].empty?
									fcpxml.spine {
										clip_array = project_seq[:clip_array]
										last_clip_index = clip_array.first.length - 1
										if clip_array.first.empty?
											last_clip_index = 0
										else
											if clip_array.first[last_clip_index][:type]=="gap"
												last_clip_index -= 1
											end
											if clip_array.first[last_clip_index][:type]=="transition"
												last_clip_index -= 1
											end
										end
										#puts "#{clip_array.first.length} primary storyline clips"
										clip_array.first.each_with_index do |primary_clip, p| # clip_array.first is the V1 track, which becomes the primary storyline
											@error_tracker = ":\n'#{project_seq[:name]}' Clip '#{primary_clip[:name]}'"
											#puts @error_tracker
											#puts "3. project_seq: #{project_seq.inspect}" if project_seq[:height].nil?
											storylineBuilder(fcpxml, [primary_clip], project_seq[:frame], project_seq[:frame], seq_frameDuration, project_seq[:height], true) # node, clips, adj, frame, frameDuration, height, primary=false
											#puts fcpxml.to_xml(:indent => 2, :encoding => "UTF-8")

											#puts "primary_clip[:type] = #{primary_clip[:type]}"
											case primary_clip[:type] # also check case nested_primary_clip[:type]
											when "title"
												primary_start = (3600 * 1/seq_frameDuration).round
											when "gap"
												primary_start = (3600 * 1/seq_frameDuration).round
											when "generator"
												primary_start = (3600 * 1/seq_frameDuration).round
											when "transition"
												# ignore - can't be connected to
											when "mc"
												primary_start = (primary_clip[:mc_frame] / primary_clip[:speed_adj]) + primary_clip[:in] # bugfix for mc
											when "mv", "mav"
												refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/seq_frameDuration)
												primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
											when "nested"
												primary_start = (primary_clip[:frame] / primary_clip[:speed_adj]) + primary_clip[:in]
												#puts "nested, primary_start = (#{primary_clip[:frame]} / #{primary_clip[:speed_adj]}) + #{primary_clip[:in]} = #{primary_start}"
											else # clip
												# refactor the primary clip's first frame to the sequences frame rate (Bug fix for clips and secondary storylines connected to a primary clip whose frame rate is different from the timeline's frame rate)
												refactored_first_frame = primary_clip[:frame] / (1/primary_clip[:frameDuration]) * (1/seq_frameDuration)
												primary_start = (refactored_first_frame / primary_clip[:speed_adj]) + primary_clip[:in]
											end
											primary_offset = primary_clip[:start]
											path_to_primary_spine = ["spine"]
											fcpxml.parent.ancestors.each { |a| path_to_primary_spine << a.name unless a.name=="document" }
											query = path_to_primary_spine.reverse.join("/")
											#puts "query = #{query}"
											#puts "************** primary_start = #{primary_start}", "primary_offset = #{primary_offset}"
											@project_seq_name = project_seq[:name]
											Nokogiri::XML::Builder.with(fcpxml.doc.xpath(query).last.element_children.last) do |primary_node| # the primary_node is the fcpxnl's last spine's last element_children node
												# puts primary_node.to_xml(:indent => 2, :encoding => "UTF-8")
												unless primary_clip[:array_of_connecteds].nil? || primary_clip[:type]=="transition"
													# attach connected clips or secondary spines to the last primary clip
													primary_clip[:array_of_connecteds].each do |connecteds|
														connecteds.compact! # bug fix for nil in the array
														if connecteds.empty?
															#puts "connecteds is empty"
														else
															if connecteds.length==1 && connecteds.first[:type]=="transition"
																#puts "CONNECTING A SINGLE TRANSITION !!!!!!"
															end
															#puts "************** primary_start - primary_offset + connecteds.first[:start] = #{primary_start} - #{primary_offset} + #{connecteds.first[:start]}"
															#puts "4. project_seq: #{project_seq.inspect}" if project_seq[:height].nil?
															if connecteds.length==1 && connecteds.first[:type]!="transition" # connected clip
																storylineBuilder(primary_node, connecteds, primary_start - primary_offset, project_seq[:frame], seq_frameDuration, project_seq[:height]) # node, clips, adj, frame, frameDuration, height, primary=false
															elsif connecteds.length > 1 # secondary storyline
																primary_node.spine(:lane => connecteds.first[:lane], :offset => to_rational_seconds(primary_start - primary_offset + connecteds.first[:start], seq_frameDuration)) {
																	storylineBuilder(primary_node, connecteds, connecteds.first[:start]*-1, project_seq[:frame], seq_frameDuration, project_seq[:height])
																}
															end
														end
													end
												end
												# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
												#pp primary_clip
												unless primary_start.nil? || primary_clip[:type]=="transition"
													if p==last_clip_index
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], primary_clip[:tail_markers], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], seq_frameDuration)
													else
														markers, chmarkers = order_markers(primary_clip[:user_markers], primary_clip[:chapter_markers], primary_clip[:head_markers], [], primary_clip[:frame], primary_clip[:speed_adj], primary_clip[:in], primary_clip[:out], seq_frameDuration)
													end
													markers.each { |m| primary_node << m }
													chmarkers.each { |m| primary_node << m }
												end
												addIntrinsicParamsAudio(primary_node, primary_clip)
												add_filters(primary_node, primary_clip) unless primary_clip[:filters].nil? || primary_clip[:type]=="mc"
												unless primary_clip[:reel_name].nil? && primary_clip[:scene].nil? && primary_clip[:shottake].nil? # transitions, slugs etc.
													primary_node.metadata {
														#puts "primary_clip[:reel_name]", primary_clip[:reel_name].inspect
														primary_node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => primary_clip[:reel_name]) unless primary_clip[:reel_name].empty?
														primary_node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => primary_clip[:scene]) unless primary_clip[:scene].empty?
														primary_node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => primary_clip[:shottake]) unless primary_clip[:shottake].empty?
														primary_node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => primary_clip[:label]) unless primary_clip[:label].empty?
													}
												end
											end
										end
									} # end fcpxml.spine
								} # end fcpxml.sequence
							} # end fcpxml.project
						end # end @project_seqs.each
						### add each media asset as an Event asset-clip with keywords
						#puts "add each media asset as an asset-clip(s) with keywords"
						@assets.each do |asset|
							unless asset[:type]=="mc" || asset[:type]=="psd" || asset[:type]=="img" # img assets need to be clip/video
								if asset[:type]=="a"
									format_id = "ft1" # FFVideoFormatRateUndefined - don't add format to asset if it's audio-only
								else
									format_id = asset[:format_id]
								end
								asset[:clips].each do |c|
									fcpxml.send("asset-clip", :ref => asset[:id], :name => c[:name], :duration => to_rational_seconds(c[:duration], asset[:frameDuration]), :start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset], asset[:frameDuration]), :tcFormat => asset[:tcFormat], :format => format_id) {
										fcpxml.note c[:metadata] unless c[:metadata].nil?
										unless c[:markers].nil? || c[:markers].empty?
											markers, chmarkers = order_markers(c[:markers], [], [], [], asset[:frame] + c[:subclip_startoffset], 1, c[:in], c[:out], asset[:frameDuration]) # first frame takes into account subclip_startoffset for subclips
											markers.each { |m| fcpxml << m }
											chmarkers.each { |m| fcpxml << m }
										end
										if c[:good]
											if good=="Favorite"
												fcpxml.rating(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => "favorite", :name => "Good")
											else
												fcpxml.keyword(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + c[:in], asset[:frameDuration]), :duration => to_rational_seconds(c[:out] - c[:in], asset[:frameDuration]), :value => good)
											end
										end
										c[:favorites] = [] if c[:favorites].nil?
										if @seqClipNamesFavoritesCheckbox!=nil && @seqClipNamesFavoritesCheckbox.state==1
											c[:favorites].each do |f|
												fcpxml.rating(:start => to_rational_seconds(asset[:frame] + f[:in], asset[:frameDuration]), :duration => to_rational_seconds(f[:out] - f[:in], asset[:frameDuration]), :value => "favorite", :name => f[:name])
											end
										end
										c[:bins].each do |b|
											fcpxml.keyword(:start => to_rational_seconds(asset[:frame] + c[:subclip_startoffset] + b[:in], asset[:frameDuration]), :duration => to_rational_seconds(b[:out] - b[:in], asset[:frameDuration]), :value => b[:name]) # CHANGED removed b[:metadata]
											bins << b[:name]
										end
										fcpxml.metadata {
											fcpxml.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => c[:reel_name]) unless c[:reel_name].nil? || c[:reel_name].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => c[:scene]) unless c[:scene].nil? || c[:scene].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => c[:shottake]) unless c[:shottake].nil? || c[:shottake].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.angle", :editable => "1", :value => c[:defaultangle]) unless c[:defaultangle].nil? || c[:defaultangle].empty?
											fcpxml.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => c[:label]) unless c[:label].nil? || c[:label].empty?
										}
									}
								end
							end
						end
						### add each merged clip as an Event sync-clip with keywords
						@mergedclips.each_with_index do |mergeclip, m|
							vclip = mergeclip[:angles].shift
							aclips = mergeclip[:angles].compact
							if vclip.nil? # NOTE merged clips in Event are handled differently to merged clips in a Project
								print "skipping because vclip is nil:\n"
								#pp mergeclip
							else
								fcpxml.send("sync-clip", :name => mergeclip[:name], :duration => to_rational_seconds(mergeclip[:duration], mergeclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame] + 0, mergeclip[:frameDuration]), :tcFormat => mergeclip[:tcFormat], :format => mergeclip[:format_id]) {
									fcpxml.spine {
										if mergeclip[:syncoffset] < 0
											gap_duration = to_rational_seconds(mergeclip[:syncoffset].abs, mergeclip[:frameDuration])
											fcpxml.gap(:offset => "0s", :start => "3600s", :duration => gap_duration)
										else
											gap_duration = "0s"
										end
										fcpxml.clip(:name => vclip[:name], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame] + 0, vclip[:frameDuration]), :offset => gap_duration, :tcFormat => vclip[:tcFormat], :format => vclip[:format_id]) {
											fcpxml.video(:offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration])) {
												if vclip[:type]=="av"
													if vclip[:channels]=="stereo"
														fcpxml.audio(:offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => -1)
													elsif vclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
														fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(a[:frame], vclip[:frameDuration]), :lane => -2, :srcCh => 2, :outCh => "R" )
														fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(a[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(a[:frame], vclip[:frameDuration]), :start => to_rational_seconds(a[:frame], vclip[:frameDuration]), :lane => -1, :srcCh => 1, :outCh => "L" )
													else # one or more mono tracks
														if vclip[:audioSources]==1 # no srcID because audioSources==1
															vclip[:audioChannels].downto(1) do |ch|
																fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => 0-ch, :srcCh => ch )
															end
														elsif vclip[:audioChannels]==1 || vclip[:audioChannels]==2
															vclip[:audioSources].downto(1) do |src|
																srcCh = "1"
																unless vclip[:srcChs][src-1].nil?
																	srcCh = vclip[:srcChs][src-1]
																end
																fcpxml.audio( :ref => vclip[:id], :duration => to_rational_seconds(vclip[:duration], vclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :start => to_rational_seconds(vclip[:frame], vclip[:frameDuration]), :lane => 0-src, :srcCh => srcCh, :srcID => src )
															end
														end
													end
												end
											}
											aclips.each_with_index do |aclip, a|
												fcpxml.clip(:name => aclip[:name], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame] + 0, aclip[:frameDuration]), :offset => to_rational_seconds(vclip[:frame] + mergeclip[:syncoffset], vclip[:frameDuration]), :tcFormat => aclip[:tcFormat], :format => aclip[:format_id], :lane => "-#{a+1}") {
													if aclip[:channels]=="stereo"
														fcpxml.audio(:offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]))
														unless aclip[:audioSources]==1
															aclip[:audioSources].downto(2) do |src|
																fcpxml.audio(:offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-src, :srcID => src)
															end
														end
													elsif aclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
														fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(a[:frame], aclip[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
															fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(a[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(a[:frame], aclip[:frameDuration]), :start => to_rational_seconds(a[:frame], aclip[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
														}
													else # one or more mono tracks
														fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :srcID => 1 ) {
															unless aclip[:channelcount]==1
																if aclip[:audioSources]==1 # no srcID because audioSources==1
																	aclip[:audioChannels].downto(2) do |ch|
																		fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-ch, :srcCh => ch )
																	end
																elsif aclip[:audioChannels]==1 || aclip[:audioChannels]==2
																	aclip[:audioSources].downto(2) do |src|
																		srcCh = "1"
																		unless aclip[:srcChs][src-1].nil?
																			srcCh = aclip[:srcChs][src-1]
																		end
																		fcpxml.audio( :ref => aclip[:id], :duration => to_rational_seconds(aclip[:duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:frame], aclip[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
																	end
																end
															end
														}
													end
												}
											end
										}
									}
									mergeclip[:bins].each do |b|
										fcpxml.keyword(:start => "0s", :duration => to_rational_seconds(mergeclip[:duration], mergeclip[:frameDuration]), :value => b[:name])
										bins << b[:name]
									end
								}
							end
						end
						### add each multiclip reference as an Event mc-clip
						@multiclips.each_with_index do |multiclip, m|
							fcpxml.send("mc-clip", :ref => multiclip[:id], :name => multiclip[:name], :duration => to_rational_seconds(multiclip[:duration], multiclip[:frameDuration])) {
								fcpxml.send("mc-source", :angleID => multiclip[:activevideoangle], :srcEnable => "all")
							}
						end
						### add each nested sequence as an Event ref-clip with keyword
						@nested_seqs.each do |nested_seq|
							seq_frameDuration = nested_seq[:frameDuration]
							fcpxml.send("ref-clip", :ref => nested_seq[:nested_ref_id], :name => nested_seq[:name], :duration => to_rational_seconds(nested_seq[:duration], seq_frameDuration), :start => to_rational_seconds(nested_seq[:frame], seq_frameDuration)) {
								fcpxml.keyword(:value => "Sequences", :duration => to_rational_seconds(nested_seq[:duration], seq_frameDuration), :start => to_rational_seconds(nested_seq[:frame], seq_frameDuration))
							}
						end
						### add each keyword as a keyword-collection to the event
						unless bins.empty?
							fcpxml.send("collection-folder", :name => "Bins") {
								bins.uniq.each do |bin|
									fcpxml.send("keyword-collection", :name => bin)
								end
							}
						end
					}
				}
			}
			# Fix up faulty posteroffset attribute
			fcpxml.doc.xpath("//chapter-marker[@posteroffset]").each do |cm|
				cm.delete("posteroffset")
				cm["posterOffset"] = "0s"
			end
		end
		save_this_file = File.new(new_xml_file, "w")
		save_this_file.puts builder.to_xml(:indent => 2, :encoding => "UTF-8")
		save_this_file.close
	rescue => e
		if $ruby_cocoa==false
			raise
		else
			OSX::NSLog "#{e.message}\n#{e.backtrace.join("\n")}"
      executable = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleExecutable")
			version = OSX::NSBundle.mainBundle.infoDictionary.objectForKey("CFBundleShortVersionString")
			alert = OSX::NSAlert.alloc.init
			alert.setAlertStyle(OSX::NSInformationalAlertStyle)
			alert.setMessageText("There was a problem creating the translated XML#{@error_tracker}")
			alert.setInformativeText("Please zip up the XML file and email it with the subject “#{executable} #{version} error” to 7toX@assistedediting.com with this error message in the body text:\n
			#{e.message}\n
			#{e.backtrace.first}\n
			Thanks for helping us improve #{executable}!")
			alert.runModal
		end
	end
end

def storylineBuilder(node, clips, adj, frame, frameDuration, height, isPrimary=false)
	puts "storylineBuilder: #{clips.length} clips, adj=#{adj}, frame=#{frame}"
	first_clip_index = 0
	if clips.first[:type]=="transition"
		first_clip_index = 1
	end
	last_clip_index = clips.length - 1
	if clips.last[:type]=="transition" || clips.last[:type]=="gap"
		last_clip_index -= 1
	end
	if isPrimary==false && clips.length==1
		isConnected = true
	else
		isConnected = false
	end
	#puts "isPrimary = #{isPrimary}, first_clip_index = #{first_clip_index}, last_clip_index = #{last_clip_index}, isConnected = #{isConnected}, frameDuration = #{frameDuration}"
	clips.each_with_index do |clip, c|
		@error_tracker = ":\n'#{@project_seq_name}' Clip '#{clip[:name]}'"
		#puts "  CLIP #{clip[:name]} is #{clip[:type]}"
		#puts "reel_name = #{clip[:reel_name].inspect}, scene = #{clip[:scene].inspect}, shottake = #{clip[:shottake].inspect}"
		#pp clip
		clip[:in] = 0 if clip[:in].nil?
		clip[:out] = clip[:duration] if clip[:out].nil?
		if c==last_clip_index && clips.last[:type]=="transition" && isPrimary==false # last transitions in secondary storyline don't need adjustment
			clip[:end] = clip[:real_end]
		end
		#puts "clip type = #{clip[:type]}"
		case clip[:type]
		when "gap"
			node.gap( :name => clip[:name], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => "3600s" )
		when "generator"
			node.video( :name => clip[:name], :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => "3600s", :enabled => clip[:enabled] ) {
				node.parent["lane"] = clip[:lane].to_s if isConnected #&& isPrimary==false
				if clip[:rgb]!=nil
					node.param(:name => "Color", :value => clip[:rgb], :key => "9999/10008/10006/2/1/1")
				end
				addIntrinsicParamsVideo(node, clip)
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
				end
			}
		when "transition"
			node.transition( :name => clip[:name], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration) ) {
				node.send("filter-video", :ref => clip[:ref])
				node.send("filter-audio", :ref => 'ea1') # both filter-video and filter-audio are present for video-only and audio-only transitions
			}
		when "title"
			title_name = "#{clip[:text].first} - #{clip[:name]}"
			node.title( :name => title_name, :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => "3600s", :enabled => clip[:enabled] ) {
				node.parent["lane"] = clip[:lane].to_s if isConnected #&& isPrimary==false
				clip[:text] = [] if clip[:text].nil?
				#puts "clip[:text]"; pp clip[:text]
				# <!ELEMENT title (param*, text*, text-style-def*, note?, %intrinsic-params-video;, (%anchor_item;)*, (%marker_item;)*, filter-video*, metadata?)>
				if clip[:ref]=="t2" # lower third...
					node.param( :name => "Build In", :key => "9999/10321/2/101", :value => "0")
					node.param( :name => "Build Out", :key => "9999/10321/2/102", :value => "0")
					if clip[:text].length==1 # ...with only 1 line of text
						node.text_ {
							node.send("text-style", "", :fontSize => 20*3)
						}
					end
				end
				current_id = @id
				clip[:text].each_with_index do |txt, i| # can have more than one text line and style
					unless txt.empty?
						ts = @text_styles[clip[:ts_id]+i]
						@id += 1
						if ts.nil? || ts[:font].nil?
							node.text_ { # use underscore to disambiguate tag names from ruby methods
								node.send("text-style", txt, :fontSize => 20*3) # 20pts is default FCP7 font size
							}
						else
							node.text_ {
								node.send("text-style", txt, :ref => "ts#{@id}")
							}
						end
					end
				end
				@id = current_id # reset before looping again
				clip[:text].each_with_index do |txt, i| # can have more than one text line and style
					unless txt.empty?
						ts = @text_styles[clip[:ts_id]+i]
						@id += 1
						if ts.nil? || ts[:font].nil?
						else # text-style-def nodes come after text nodes
							node.send("text-style-def", :id => "ts#{@id}") {
								node.send("text-style", :font => ts[:font], :fontSize => ts[:fontSize], :fontColor => ts[:fontColor], :bold => ts[:bold], :italic => ts[:italic], :alignment => ts[:alignment], :lineSpacing => ts[:lineSpacing])
							}
						end
					end
				end
				node.note clip[:metadata] unless clip[:metadata].empty?
				addIntrinsicParamsVideo(node, clip)
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
				end
			}
		when "psd"
			primary_clip = clip[:nested_array].first.first # first item in the first track
			#puts "psd"
			#pp clip[:nested_array]
			#puts primary_clip[:duration], frameDuration
			if primary_clip[:type]=="gap"
				#puts "primary_clip is gap; changing"
				#puts "OR NOT..." if clip[:nested_array].first[1].nil?
				primary_clip = clip[:nested_array].first[1] unless clip[:nested_array].first[1].nil? # HB Trailer
			end
			#pp primary_clip
			if primary_clip.nil?
        # <gap name="Gap" offset="43292249/12000s" duration="13013/1600s" start="3600s">
				# pp clip
				# puts "primary_clip is nil"; exit
			end
			node.clip( :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :enabled => clip[:enabled], :format => clip[:format_id] ) { # CHANGED from :format => primary_clip 1/13
				node.parent["lane"] = clip[:lane].to_s if isConnected #&& isPrimary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				addIntrinsicParamsVideo(node, clip)
				node.video( :role => primary_clip[:v_track], :name => primary_clip[:name], :duration => to_rational_seconds(primary_clip[:duration], frameDuration), :ref => primary_clip[:ref], :enabled => primary_clip[:enabled], :srcID => "1" ) {
					if primary_clip[:type]=="gap" || primary_clip[:ref].nil?
						node.parent.name = "gap"
						node.parent.remove_attribute("role")
						node.parent.remove_attribute("ref")
						node.parent.remove_attribute("enabled")
						node.parent.remove_attribute("srcID")
					end
					unless primary_clip[:basic_motion].nil?
						#puts primary_clip[:width].inspect, primary_clip[:height].inspect
						#puts height.inspect
						#puts primary_clip[:basic_motion].inspect
						primary_clip[:adj_conform] = "none"
						position = "#{primary_clip[:width].to_f * primary_clip[:basic_motion][:center_horiz] / height * 100} #{primary_clip[:height].to_f * primary_clip[:basic_motion][:center_vert] * -1 / height * 100}" # CHANGED to_f in case :width or :height is nil
						primary_clip[:basic_motion][:adj_transform_position] = position
					end
					addIntrinsicParamsVideo(node, primary_clip)
					unless primary_clip[:array_of_connecteds].nil?
						# attach photoshop layers to the primary clip
						primary_clip[:array_of_connecteds].compact.each_with_index do |layer, l|
							#puts "Layer #{l+2}"
							unless layer.empty? || primary_clip[:ref].nil?
								node.video( :name => layer.first[:name], :lane => l+1, :srcID => l+2,  :duration => to_rational_seconds(primary_clip[:duration], frameDuration), :ref => primary_clip[:ref], :enabled => layer.first[:enabled] ) {
									unless layer.first[:basic_motion].nil?
										#puts layer.first[:width].inspect, layer.first[:height].inspect, layer.first[:basic_motion][:center_horiz].inspect, height.inspect,  layer.first[:basic_motion][:center_vert].inspect
										layer.first[:adj_conform] = "none"
										position = "#{layer.first[:width].to_f * layer.first[:basic_motion][:center_horiz] / height * 100} #{layer.first[:height].to_f * layer.first[:basic_motion][:center_vert] * -1 / height * 100}" # recalc for each layer; to_f in case :width or :height is nil
										layer.first[:basic_motion][:adj_transform_position] = position
									end
									addIntrinsicParamsVideo(node, layer.first)
								}
							end
						end
					end
				}
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
			# NOTE: when there's a speed adjustment, the clip's start has to be altered, and the video's duration has to be altered in the opposite direction... then the marker start has to be altered in the same direction
		when "mc"
			node.send("mc-clip", :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:mc_frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :ref => clip[:mc_id]) {
				node.parent["lane"] = clip[:lane].to_s if isConnected #&& isPrimary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				node.send("mc-source", :angleID => clip[:activevideoangle], :srcEnable => "video") {
					addIntrinsicParamsVideo(node, clip)
					add_filters(node, clip, "v")
				}
				node.send("mc-source", :angleID => clip[:activeaudioangle], :srcEnable => "audio")
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					# video filters were added to mc-source
					add_filters(node, clip, "a")
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "l"
			node.video( :name => clip[:name], :role => clip[:v_track], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :ref => clip[:ref], :lane => clip[:lane], :srcID => clip[:lane]+1, :enabled => clip[:enabled] ) {
				clip[:adj_conform] = "none"
				clip[:basic_motion][:adj_transform_position] = "#{clip[:width].to_f * clip[:basic_motion][:center_horiz] / height * 100} #{clip[:height].to_f * clip[:basic_motion][:center_vert] * -1 / height * 100}" unless clip[:basic_motion].nil? # recalc for each layer; to_f in case :width or :height is nil
				addIntrinsicParamsVideo(node, clip)
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
				end
			}
		when "v", "a"
			#puts "clip[:name] = #{clip[:name]}, clip[:type] = #{clip[:type]}, clip[:asset_type] = #{clip[:asset_type]}"
			if clip[:type]=="v" && (clip[:asset_type]=="av" || clip[:asset_type]=="img") # can't use asset-clip
				node.clip( :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
					# node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], frameDuration, clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
					node.parent["lane"] = "#{clip[:lane]}" if isConnected && isPrimary==false
					node.note clip[:metadata] unless clip[:metadata].empty?
					node << clip[:timeMap]
					addIntrinsicParamsVideo(node, clip)
					# clip[:duration] = 0 if clip[:asset_type]=="img"
					node.video( :role => clip[:v_track], :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :ref => clip[:ref], :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]) )
					unless isPrimary
						if c==last_clip_index
							markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
						else
							markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
						end
						markers.each { |m| node << m }
						chmarkers.each { |m| node << m }
						add_filters(node, clip)
						node.metadata {
							node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
							node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
							node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
							node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
						}
					end
				}
			elsif clip[:type]=="a" && clip[:asset_type]=="av" # can't use asset-clip
				node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
					node.parent["lane"] = clip[:lane].to_s if isConnected && isPrimary==false
					node.note clip[:metadata] unless clip[:metadata].empty?
					node << clip[:timeMap]
					if clip[:channels]=="stereo" # one or more stereo tracks
						node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track] ) {
							unless clip[:audioSources]==1
								#pp clip if clip[:audioSources].nil?
								clip[:audioSources].downto(2) do |src|
									node.audio(:offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 1-src, :srcID => src)
								end
							end
						}
					elsif clip[:channels]=="stereotomono" # dual mono from stereo (uses srcCh & outCh)
						node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :srcCh => 1, :outCh => "L" ) {
							node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => -1, :srcCh => 2, :outCh => "R" )
						}
					# elsif clip[:channels]=="surround"
					else # one or more mono tracks
						node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :srcCh => 1 ) {
							unless clip[:channelcount]==1
								if clip[:audioSources]==1 # no srcID because audioSources==1
									clip[:audioChannels].downto(2) do |ch|
										node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 1-ch, :srcCh => ch )
									end
								elsif clip[:audioChannels]==1 || clip[:audioChannels]==2
									clip[:audioSources].downto(2) do |src|
										srcCh = "1"
										unless clip[:srcChs][src-1].nil?
											srcCh = clip[:srcChs][src-1]
										end
										node.audio( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track], :lane => 1-src, :srcCh => srcCh, :srcID => src )
									end
								end
							end
						}
					end
					unless isPrimary
						if c==last_clip_index
							markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
						else
							markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
						end
						markers.each { |m| node << m }
						chmarkers.each { |m| node << m }
						addIntrinsicParamsAudio(node, clip)
						add_filters(node, clip)
						node.metadata {
							node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
							node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
							node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
							node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
						}
					end
				}
			else # asset-clip!
				node.send("asset-clip", :ref => clip[:ref], :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :videoRole => clip[:v_track], :format => clip[:format_id]) {
					node.parent["lane"] = "#{clip[:lane]}" if isConnected #&& isPrimary==false
					# optional: srcEnable (all | audio | video) "all" DOES NOT IMPORT
					node.note clip[:metadata] unless clip[:metadata].empty?
					node << clip[:timeMap]
					addIntrinsicParamsVideo(node, clip)
					unless isPrimary
						if c==last_clip_index
							markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
						else
							markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
						end
						markers.each { |m| node << m }
						chmarkers.each { |m| node << m }
						addIntrinsicParamsAudio(node, clip)
						add_filters(node, clip)
						node.metadata {
							node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
							node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
							node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
							node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
						}
					end
				}
			end
		when "av" # could have audioDuration and audioStart
			#puts clip[:v_track].inspect
			node.send("asset-clip", :ref => clip[:ref], :name => clip[:name], :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :audioDuration => to_rational_seconds(clip[:a_end]-clip[:a_start], frameDuration), :audioStart => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:a_in], frameDuration), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :videoRole => clip[:v_track], :format => clip[:format_id]) {
				node.parent["lane"] = "#{clip[:lane]}" if isConnected #&& isPrimary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				addIntrinsicParamsVideo(node, clip)
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					addIntrinsicParamsAudio(node, clip)
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "mv", "mav"
			# puts clip[:frame], clip[:a_syncoffset], clip[:synced_audio].first[:a_frame], clip[:in], clip[:a_in]
			#puts "frameDuration = #{frameDuration}, clip[:frameDuration] = #{clip[:frameDuration]}, adj = #{adj}, "
			a_frame = clip[:synced_audio].first[:a_frame]
			node.send("sync-clip", :name => clip[:name], :duration => to_rational_seconds(clip[:end] - clip[:start], frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled], :audioDuration => to_rational_seconds(clip[:a_end]-clip[:a_start], frameDuration), :audioStart => to_rational_seconds(a_frame / clip[:speed_adj], clip[:frameDuration], clip[:a_in], frameDuration), :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
				node.parent["lane"] = clip[:lane].to_s if isConnected #&& isPrimary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				addIntrinsicParamsVideo(node, clip)
				node.spine {
					node.clip( :name => clip[:name], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :enabled => clip[:enabled], :tcFormat => clip[:tcFormat], :format => clip[:format_id] ) {
						node.video( :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration]), :role => clip[:v_track] ) {
							node.audio( :lane => -1, :ref => clip[:ref], :duration => to_rational_seconds(clip[:duration], clip[:frameDuration]), :offset => to_rational_seconds(clip[:frame], clip[:frameDuration]), :start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :role => clip[:a_track] ) unless clip[:type]=="mv"
						}
						clip[:synced_audio].each_with_index do |aclip, a|
							if aclip[:a_frame]==0 # linked clip?
								#puts "it's a linked clip"
								clip_frame = clip[:frame]
								clip_start = clip[:a_syncoffset] * -1
							else # merged clip
								#puts "it's a merged clip"
								# if clip[:frame] < aclip[:a_frame]
								# 	clip_frame = aclip[:a_frame] - (aclip[:a_frame] - clip[:frame]) - clip[:a_syncoffset]
								# else
								# 	clip_frame = aclip[:a_frame] - (aclip[:a_frame] - clip[:frame]) + clip[:a_syncoffset]
								# end
								# clip_start = aclip[:a_frame]
							end
							if aclip[:type]=="generator" # missing
								node.video( :lane => "-1", :name => aclip[:name], :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], frameDuration), :offset => to_rational_seconds(aclip[:a_frame], frameDuration), :start => "3600s", :enabled => aclip[:enabled] )
							else
								node.clip(:name => aclip[:name], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] + 0, aclip[:frameDuration]), :offset => to_rational_seconds(clip[:frame] + aclip[:a_syncoffset], clip[:frameDuration]), :enabled => aclip[:enabled], :tcFormat => aclip[:a_tcFormat], :format => aclip[:a_format_id], :lane => "-#{a+1}") {
									if aclip[:channels]=="stereo"
										node.audio(:offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :role => aclip[:a_track])
									elsif aclip[:channels]=="stereotomono" # dual mono (uses srcCh & outCh)
										node.audio(:offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]),  :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :srcCh => 1, :outCh => "L" ) {
											node.audio(:offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :lane => -1, :srcCh => 2, :outCh => "R" )
										}
									else # one or more mono tracks
										node.audio( :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :srcCh => 1 ) {
											unless aclip[:channelcount]==1
												if aclip[:audioSources]==1 # no srcID because audioSources==1
													aclip[:audioChannels].downto(2) do |ch|
														node.audio( :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :lane => 1-ch, :srcCh => ch )
													end
												elsif aclip[:audioChannels]==1 || aclip[:audioChannels]==2
													aclip[:audioSources].downto(2) do |src|
														srcCh = "1"
														unless aclip[:srcChs][src-1].nil?
															srcCh = aclip[:srcChs][src-1]
														end
														node.audio( :ref => aclip[:a_ref], :duration => to_rational_seconds(aclip[:a_duration], aclip[:frameDuration]), :offset => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :start => to_rational_seconds(aclip[:a_frame] / clip[:speed_adj], aclip[:frameDuration]), :lane => 1-src, :srcCh => srcCh, :srcID => src )
													end
												end
											end
										}
									end
								}
							end
						end
					}
				}
				#puts "mv or mav clip: isPrimary = #{isPrimary}"
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					addIntrinsicParamsAudio(node, clip)
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		when "nested"
			clip_name = "#{clip[:name]} Compound Clip"
			# pp clip
			#puts clip[:speed_adj].inspect
			#puts adj.inspect
			#puts frameDuration.inspect
			#puts clip[:frame].inspect
			#puts clip[:frameDuration].inspect
			node.send("ref-clip", :ref => clip[:nested_ref_id], :name => clip_name, :duration => to_rational_seconds((clip[:end] - clip[:start]), frameDuration), :offset => to_rational_seconds(clip[:start] + adj, frameDuration), :start => to_rational_seconds(clip[:frame] / clip[:speed_adj], clip[:frameDuration], clip[:in], frameDuration), :enabled => clip[:enabled]) {
				node.parent["lane"] = clip[:lane].to_s if isConnected #&& isPrimary==false
				node.note clip[:metadata] unless clip[:metadata].empty?
				node << clip[:timeMap]
				add_intrinsic_params_video(node, clip)
				# add primary clip's extras (marker, keyword, filter and text have to go after connected clips/storylines)
				#puts "isPrimary = #{isPrimary}", "nested_primary_start nil? #{nested_primary_start.nil?}"
				unless isPrimary
					if c==last_clip_index
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], clip[:tail_markers], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					else
						markers, chmarkers = order_markers(clip[:user_markers], clip[:chapter_markers], clip[:head_markers], [], clip[:frame], clip[:speed_adj], clip[:in], clip[:out], frameDuration)
					end
					markers.each { |m| node << m }
					chmarkers.each { |m| node << m }
					add_filters(node, clip)
					node.metadata {
						node.md(:key => "com.apple.proapps.studio.reel", :type => "string", :editable => "1", :value => clip[:reel_name])
						node.md(:key => "com.apple.proapps.studio.scene", :type => "string", :editable => "1", :value => clip[:scene])
						node.md(:key => "com.apple.proapps.studio.shot", :type => "string", :editable => "1", :value => clip[:shottake])
						node.md(:key => "com.apple.proapps.studio.metadataLocation", :type => "string", :editable => "1", :value => clip[:label])
					}
				end
			}
		end
	end
end

def addIntrinsicParamsVideo(node, clip)
	#puts "addIntrinsicParamsVideo"
	unless clip[:crop_trim_rect].nil?
		node.send("adjust-crop", :mode => "trim") {
			node.send("trim-rect", :top => clip[:crop_trim_rect][:top], :left => clip[:crop_trim_rect][:left], :bottom => clip[:crop_trim_rect][:bottom], :right => clip[:crop_trim_rect][:right]) {
				unless clip[:crop_trim_rect][:top_keyframes].empty?
					node.param(:name => "top") {
						node.keyframeAnimation {
							clip[:crop_trim_rect][:top_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:crop_trim_rect][:bottom_keyframes].empty?
					node.param(:name => "bottom") {
						node.keyframeAnimation {
							clip[:crop_trim_rect][:bottom_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:crop_trim_rect][:left_keyframes].empty?
					node.param(:name => "left") {
						node.keyframeAnimation {
							clip[:crop_trim_rect][:left_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
				unless clip[:crop_trim_rect][:right_keyframes].empty?
					node.param(:name => "right") {
						node.keyframeAnimation {
							clip[:crop_trim_rect][:right_keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
			}
		}
	end
	unless clip[:adj_corners].nil?
		node.send("adjust-corners", :botLeft => clip[:adj_corners][:botLeft], :topLeft => clip[:adj_corners][:topLeft], :topRight => clip[:adj_corners][:topRight], :botRight => clip[:adj_corners][:botRight]) {
			unless clip[:adj_corners][:topleft_keyframes].empty?
				node.param(:name => "topLeft") {
					node.keyframeAnimation {
						clip[:adj_corners][:topleft_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
			unless clip[:adj_corners][:topright_keyframes].empty?
				node.param(:name => "topRight") {
					node.keyframeAnimation {
						clip[:adj_corners][:topright_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
			unless clip[:adj_corners][:botleft_keyframes].empty?
				node.param(:name => "botLeft") {
					node.keyframeAnimation {
						clip[:adj_corners][:botleft_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
			unless clip[:adj_corners][:botright_keyframes].empty?
				node.param(:name => "botRight") {
					node.keyframeAnimation {
						clip[:adj_corners][:botright_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
		}
	end
	node.send("adjust-conform", :type => clip[:adj_conform]) unless clip[:adj_conform].nil?
	unless clip[:basic_motion].nil?
		node.send("adjust-transform", :position => clip[:basic_motion][:adj_transform_position], :rotation => clip[:basic_motion][:adj_transform_rotation], :scale => clip[:basic_motion][:adj_transform_scale], :anchor => clip[:basic_motion][:adj_transform_anchor]) {
			unless clip[:basic_motion][:position_keyframes].empty?
				node.param(:name => "position") {
					node.keyframeAnimation {
						clip[:basic_motion][:position_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
			unless clip[:basic_motion][:rotation_keyframes].empty?
				node.param(:name => "rotation") {
					node.keyframeAnimation {
						clip[:basic_motion][:rotation_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
			unless clip[:basic_motion][:scale_keyframes].empty?
				node.param(:name => "scale") {
					node.keyframeAnimation {
						clip[:basic_motion][:scale_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
			unless clip[:basic_motion][:anchor_keyframes].empty?
				node.param(:name => "anchor") {
					node.keyframeAnimation {
						clip[:basic_motion][:anchor_keyframes].each do |kf|
							node.keyframe(:time => kf[:time], :value => kf[:value])
						end
					}
				}
			end
		}
	end
	unless clip[:adj_blend].nil?
		if clip[:adj_blend][:mode].nil?
			node.send("adjust-blend", :amount => clip[:adj_blend][:amount]) {
				unless clip[:adj_blend][:keyframes].empty?
					node.param(:name => "amount") {
						node.keyframeAnimation {
							clip[:adj_blend][:keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
			}
		else # clip[:adj_blend][:mode] != nil
			node.send("adjust-blend", :mode => clip[:adj_blend][:mode], :amount => clip[:adj_blend][:amount]) {
				unless clip[:adj_blend][:keyframes].empty?
					node.param(:name => "amount") {
						node.keyframeAnimation {
							clip[:adj_blend][:keyframes].each do |kf|
								node.keyframe(:time => kf[:time], :value => kf[:value])
							end
						}
					}
				end
			}
		end
	end
end

def addIntrinsicParamsAudio(node, clip)
=begin
<!-- An 'audio-channel-source' element adjusts playback settings for a single channel-based audio component in a clip's primary audio layout -->
<!-- The primary audio layout is comprised of all audio from elements in the primary (lane 0) storyline. -->
<!ELEMENT audio-channel-source (%adjust-audio-enhancements;, %intrinsic-params-audio;, filter-audio*, mute*)>
<!ATTLIST audio-channel-source srcCh CDATA #REQUIRED>		<!-- source audio channels (comma separated, 1-based index) -->
<!ATTLIST audio-channel-source outCh CDATA #IMPLIED>		<!-- output audio channels (comma separated, from: L,R,C,LFE,Ls,Rs,X) %outputChannel -->
<!ATTLIST audio-channel-source role CDATA #IMPLIED>			<!-- output role assignment -->
<!ATTLIST audio-channel-source start %time; #IMPLIED>
<!ATTLIST audio-channel-source duration %time; #IMPLIED>
<!ATTLIST audio-channel-source enabled (0 | 1) '1'>
<!ATTLIST audio-channel-source active (0 | 1) '1'>

<!-- An 'audio-role-source' element adjusts playback settings for a single role-based audio component in a clip (parents are: mc-clip, ref-clip or sync-clip/sync-source) -->
<!ELEMENT audio-role-source (%adjust-audio-enhancements;, %intrinsic-params-audio;, filter-audio*, mute*)>
<!ATTLIST audio-role-source role CDATA #REQUIRED>		<!-- role the audio component is associated with -->
<!ATTLIST audio-role-source start %time; #IMPLIED>
<!ATTLIST audio-role-source duration %time; #IMPLIED>
<!ATTLIST audio-role-source enabled (0 | 1) '1'>
<!ATTLIST audio-role-source active (0 | 1) '1'>
=end
	if clip[:type]=="av" || clip[:type]=="a"
		if clip[:channels]=="stereo"
			channel = 0
			# FIXME: audio-source is now audio-role-source or audio-channel-source (and what about sync-source?)
			node.send("audio-channel-source", :srcCh => "1, 2", :outCh => "L, R", :role => clip[:a_track]) {
				unless clip[:adj_volume].nil? || clip[:adj_volume][channel].nil?
					node.send("adjust-volume", :amount => clip[:adj_volume][channel][:amount]) {
						unless clip[:adj_volume][channel][:keyframes].empty?
							node.param(:name => "amount") {
								node.keyframeAnimation {
									clip[:adj_volume][channel][:keyframes].each do |kf|
										node.keyframe(:time => kf[:time], :value => kf[:value])
									end
								}
							}
						end
					}
				end
				unless clip[:adj_panner].nil? || clip[:adj_panner][channel].nil?
					node.send("adjust-panner", :amount => clip[:adj_panner][channel][:amount]) {
						unless clip[:adj_panner][channel][:keyframes].empty?
							node.param(:name => "amount") {
								node.keyframeAnimation {
									clip[:adj_panner][channel][:keyframes].each do |kf|
										node.keyframe(:time => kf[:time], :value => kf[:value])
									end
								}
							}
						end
					}
				end
				unless clip[:mutes].nil? || clip[:mutes][channel].nil?
					node.mute(:start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :duration => to_rational_seconds(clip[:duration], clip[:frameDuration])) if clip[:mutes][channel]==true
				end
			}
		else # mono
			0.upto(clip[:channelcount]-1) do |channel|
				node.send("audio-channel-source", :srcCh => channel+1, :role => clip[:a_track]) {
					unless clip[:adj_volume].nil? || clip[:adj_volume][channel].nil?
						node.send("adjust-volume", :amount => clip[:adj_volume][channel][:amount]) {
							unless clip[:adj_volume][channel][:keyframes].empty?
								node.param(:name => "amount") {
									node.keyframeAnimation {
										clip[:adj_volume][channel][:keyframes].each do |kf|
											node.keyframe(:time => kf[:time], :value => kf[:value])
										end
									}
								}
							end
						}
					end
					unless clip[:adj_panner].nil? || clip[:adj_panner][channel].nil?
						node.send("adjust-panner", :amount => clip[:adj_panner][channel][:amount]) {
							unless clip[:adj_panner][channel][:keyframes].empty?
								node.param(:name => "amount") {
									node.keyframeAnimation {
										clip[:adj_panner][channel][:keyframes].each do |kf|
											node.keyframe(:time => kf[:time], :value => kf[:value])
										end
									}
								}
							end
						}
					end
					unless clip[:mutes].nil? || clip[:mutes][channel].nil?
						node.mute(:start => to_rational_seconds(clip[:frame], clip[:frameDuration]), :duration => to_rational_seconds(clip[:duration], clip[:frameDuration])) if clip[:mutes][channel]==true
					end
				}
			end
		end
	end
end

